package com.ekexiu.project.bridge.envdata;

import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.orm.annotation.dao.Batch;
import org.jfw.apt.orm.annotation.dao.DAO;
import org.jfw.apt.orm.annotation.dao.method.Exclude;
import org.jfw.apt.orm.annotation.dao.method.From;
import org.jfw.apt.orm.annotation.dao.method.IncludeFixSet;
import org.jfw.apt.orm.annotation.dao.method.OrderBy;
import org.jfw.apt.orm.annotation.dao.method.Select;
import org.jfw.apt.orm.annotation.dao.method.SetSentence;
import org.jfw.apt.orm.annotation.dao.method.Where;
import org.jfw.apt.orm.annotation.dao.method.operator.Insert;
import org.jfw.apt.orm.annotation.dao.method.operator.PageSelect;
import org.jfw.apt.orm.annotation.dao.method.operator.QueryList;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectList;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectOne;
import org.jfw.apt.orm.annotation.dao.method.operator.Update;
import org.jfw.apt.orm.annotation.dao.method.operator.UpdateWith;
import org.jfw.apt.orm.annotation.dao.param.Alias;
import org.jfw.apt.orm.annotation.dao.param.GroupSqlColumn;
import org.jfw.apt.orm.annotation.dao.param.GtEq;
import org.jfw.apt.orm.annotation.dao.param.In;
import org.jfw.apt.orm.annotation.dao.param.Like;
import org.jfw.apt.orm.annotation.dao.param.LtEq;
import org.jfw.apt.orm.annotation.dao.param.Set;
import org.jfw.apt.orm.annotation.dao.param.SqlColumn;
import org.jfw.apt.orm.annotation.dao.param.UnEquals;
import org.jfw.apt.orm.core.defaultImpl.StringHandler;
import org.jfw.util.PageQueryResult;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

/**
 * Created by TT on 2019/3/8.
 */
@DAO
public interface EnvDataDao {

    @SelectList
    @Where(" CREATE_TIME = (SELECT MAX(CREATE_TIME) FROM ENV_DATA) ")
    List<EnvData> query(Connection con) throws SQLException;

    @SelectList
    @OrderBy("ORDER BY CREATE_TIME DESC")
    List<EnvData> query(Connection con, @GtEq @Alias("createTime")String bt,@LtEq @Alias("createTime") String et) throws SQLException;

    @SelectList
    @OrderBy("ORDER BY CREATE_TIME DESC")
    List<EnvData> queryById(Connection con, @GtEq @Alias("createTime") String bt, int itemId) throws SQLException;

    @SelectList
    @OrderBy("ORDER BY CREATE_TIME DESC,ITEM_ID ASC")
    List<EnvData> query(Connection con, @GtEq @Alias("createTime") String bt, @LtEq @Alias("createTime") String et, @In int[] itemId) throws SQLException;

    @Insert
    int insert(Connection con, EnvServer envServer) throws SQLException;

    @Insert
    int insert(Connection con, EnvTransducer envTransducer) throws SQLException;

    @UpdateWith
    @From(EnvServer.class)
    @SetSentence("active = '0'")
    @Where("active = '1'")
    @IncludeFixSet("modifyTime")
    int logicDeleteServer(Connection con, @Set String modifier, String id) throws SQLException;

    @UpdateWith
    @From(EnvServer.class)
    @SetSentence("active = '0'")
    @Where("active = '1'")
    @IncludeFixSet("modifyTime")
    int logicDeleteServerByBridge(Connection con, @Set String modifier,String bridgeId) throws SQLException;

    @UpdateWith
    @From(EnvTransducer.class)
    @SetSentence("active = '0'")
    @Where("active = '1'")
    @IncludeFixSet("modifyTime")
    int logicDeleteTransducer(Connection con, @Set String modifier, String id) throws SQLException;

    @UpdateWith
    @From(EnvTransducer.class)
    @SetSentence("active = '0'")
    @Where("active = '1'")
    @IncludeFixSet("modifyTime")
    @Batch
    int[] logicDeleteTranByServers(Connection con, @Set String modifier, @Batch String[] serverId) throws SQLException;

    @Update
    @Exclude("active")
    int update(Connection con, EnvServer envServer) throws SQLException;

    @Update
    @Exclude("active")
    int update(Connection con, EnvTransducer envTransducer) throws SQLException;

    @SelectList
    List<EnvServer> queryByBridge(Connection con,@Nullable Boolean active, String bridgeId) throws SQLException;

    @QueryList
    @From(EnvTransducer.class)
    @Select(EnvTransducer.class)
    List<EnvTranLastValue> queryByServer(Connection con, @Nullable Boolean active, @In String[] serverId) throws SQLException;

    @SelectList
    @OrderBy("ORDER BY CODE ASC")
    List<EnvTransducer> queryTraByBridge(Connection con, @Nullable Boolean active, @SqlColumn(handlerClass = StringHandler.class,value = {"SERVER_ID IN (SELECT ID FROM ENV_SERVER WHERE BRIDGE_ID = ?)"}) String bridgeId) throws SQLException;


    @PageSelect
    @OrderBy("ORDER BY CODE")
    PageQueryResult<EnvServer> pageQuery(Connection con, @Nullable Boolean active, @Nullable @SqlColumn(handlerClass = StringHandler.class, value = {"BRIDGE_ID IN (SELECT ID FROM BRIDGE WHERE SHORT_NAME LIKE ?)"}) String key, @Nullable @Like String code, int pageSize, int pageNo) throws SQLException;

    @PageSelect
    @OrderBy("ORDER BY CODE")
    PageQueryResult<EnvTransducer> pageQuery(Connection con, @Nullable Boolean active, @Nullable @SqlColumn(handlerClass = StringHandler.class, value = {"SERVER_ID IN (SELECT ID FROM ENV_SERVER WHERE CODE LIKE ?)"}) String serverCode, @Nullable @GroupSqlColumn(handlerClass = StringHandler.class,value = {"SERVER_ID IN (SELECT ID FROM ENV_SERVER WHERE BRIDGE_ID IN (SELECT ID FROM BRIDGE WHERE SHORT_NAME LIKE ?))"}) String bridgeName,@Nullable String system, @Nullable Integer code, int pageSize, int pageNo) throws SQLException;

    @SelectOne
    @Nullable
    EnvServer queryEnvServer(Connection con, String id) throws SQLException;

    @SelectOne
    @Nullable
    EnvTransducer queryEnvTransducer(Connection con, String id) throws SQLException;

    @SelectOne
    @Nullable
    EnvServer querySerByCode(Connection con, String code, @Nullable @UnEquals String id, @Nullable Boolean active) throws SQLException;

    @SelectOne
    @Nullable
    EnvServer querySerBySeq(Connection con, String seq, @Nullable @UnEquals String id,@Nullable Boolean active) throws SQLException;

    @SelectOne
    @Nullable
    EnvServer querySerByIp(Connection con, String ipAddress, @Nullable @UnEquals String id, @Nullable Boolean active) throws SQLException;

    @SelectOne
    @Nullable
    EnvTransducer queryTraByCode(Connection con, Integer code, @Nullable @UnEquals String id, @Nullable Boolean active) throws SQLException;

    @SelectOne
    @Nullable
    EnvTransducer queryTraBySeq(Connection con, Integer seq, @Nullable @UnEquals String id,@Nullable Boolean active) throws SQLException;

    @SelectList
    @OrderBy("ORDER BY CODE")
    List<EnvServer> queryAllServer(Connection con, @Nullable String bridgeId, @Nullable Boolean active) throws SQLException;
}
