package com.ekexiu.project.bridge.envdata;

import com.ekexiu.project.bridge.system.vo.SessionUser;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.jdbc.PreparedStatementConfig;
import org.jfw.util.jdbc.ResultSetExtractor;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;

/**
 * Created by TT on 2019/3/8.
 */
@Path("/envData")
public class EnvDataService {
    private AtomicReference<List<EnvData>> lastData = new AtomicReference<List<EnvData>>();
    public static AtomicBoolean dolog=new AtomicBoolean(false);

    @Autowrie
    private EnvDataDao envDataDao;

    public EnvDataDao getEnvDataDao() {
        return envDataDao;
    }

    public void setEnvDataDao(EnvDataDao envDataDao) {
        this.envDataDao = envDataDao;
    }

    public void setLastData(List<EnvData> lastData) {
        if (lastData.size() == 0) {
            return;
        }
        this.lastData.set(lastData);
    }

    public EnvData query(int seq) {
        List<EnvData> dataList = lastData.get();
        if(dataList!=null) {
            for (EnvData envData : dataList) {
                if (envData.getItemId() == seq) {
                    return envData;
                }
            }
        }
        return null;
    }

    @Path("/openlog")
    @Get
    public void openlog() {
        dolog.set(true);
    }

    @Path("/closelog")
    @Get
    public void closelog() {
        dolog.set(false);
    }

    @Path("/byTime")
    @Get
    public List<EnvData> query(@JdbcConn Connection con, @DefaultValue("\"00000000000000\"") String bt, @DefaultValue("\"99999999999999\"") String et) throws SQLException {
        return envDataDao.query(con, bt, et);
    }

    @Path
    @Get
    public List<EnvData> query(@JdbcConn Connection con) throws SQLException {
        List<EnvData> res = lastData.get();
        if (res == null) {
            res = envDataDao.query(con);
            this.lastData.set(res);
        }
        return res;
    }


    @Path("/server")
    @Post
    public String insert(@JdbcConn(true)Connection con, @LoginUser SessionUser sessionUser, EnvServer envServer)throws SQLException {
        String id = StringUtil.buildUUID();
        envServer.setId(id);
        envServer.setActive(true);
        envServer.setCreator(sessionUser.getId());
        envServer.setModifier(sessionUser.getId());
        envDataDao.insert(con, envServer);
        return id;
    }

    @Path("/transducer")
    @Post
    public String insert(@JdbcConn(true)Connection con,@LoginUser SessionUser sessionUser,EnvTransducer envTransducer)throws SQLException {
        String id = StringUtil.buildUUID();
        envTransducer.setId(id);
        envTransducer.setActive(true);
        envTransducer.setCreator(sessionUser.getId());
        envTransducer.setModifier(sessionUser.getId());
        envDataDao.insert(con, envTransducer);
        return id;
    }

    @Path("/server/delete")
    @Get
    public void deleteServer(@JdbcConn(true)Connection con,@LoginUser SessionUser sessionUser,String id)throws SQLException {
        envDataDao.logicDeleteServer(con, sessionUser.getId(), id);
        String[] serverId = new String[1];
        serverId[0] = id;
        envDataDao.logicDeleteTranByServers(con, sessionUser.getId(), serverId);
    }

    @Path("/transducer/delete")
    @Get
    public void deleteTran(@JdbcConn(true)Connection con, @LoginUser SessionUser sessionUser,String id)throws SQLException {
        envDataDao.logicDeleteTransducer(con, sessionUser.getId(), id);
    }

    @Post
    @Path("/server/update")
    public void updateServer(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, EnvServer envServer) throws SQLException {
        envServer.setModifier(sessionUser.getId());
        envDataDao.update(con, envServer);
    }

    @Post
    @Path("/transducer/update")
    public void updateTransducer(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, EnvTransducer envTransducer) throws SQLException {
        envTransducer.setModifier(sessionUser.getId());
        envDataDao.update(con, envTransducer);
    }


    @Path("/server/pq")
    @Get
    public PageQueryResult<EnvServer> pageQuery(@JdbcConn Connection con, @Nullable Boolean active,@Nullable String key,@Nullable String code,int pageSize,int pageNo)throws SQLException {
        return envDataDao.pageQuery(con, active, key == null ? null : "%" + key + "%", code == null ? null : "%" + code + "%", pageSize, pageNo);
    }

    @Path("/transducer/pq")
    @Get
    public PageQueryResult<EnvTransducer> pageQuery(@JdbcConn Connection con,@Nullable Boolean active,@Nullable String serverCode,@Nullable String bridgeName,@Nullable String system,@Nullable Integer code,int pageSize,int pageNo)throws SQLException {
        return envDataDao.pageQuery(con, active, serverCode == null ? null : "%" + serverCode + "%", bridgeName == null ? null : "%" + bridgeName + "%", system, code, pageSize, pageNo);
    }

    @Path("/server/qo")
    @Get
    public EnvServer queryServer(@JdbcConn Connection con,String id)throws SQLException {
        return envDataDao.queryEnvServer(con, id);
    }

    @Path("/transducer/qo")
    @Get
    public EnvTransducer queryTransducer(@JdbcConn Connection con,String id)throws SQLException {
        return envDataDao.queryEnvTransducer(con, id);
    }

    @Get
    @Path("/server/checkSeq")
    public EnvServer serverBySeq(@JdbcConn Connection con, String seq, @Nullable String id, @Nullable Boolean active)throws SQLException {
        return this.envDataDao.querySerBySeq(con, seq,id,active);
    }

    @Get
    @Path("/server/checkCode")
    public EnvServer serverByCode(@JdbcConn Connection con,String code,@Nullable String id,@Nullable Boolean active)throws SQLException {
        return this.envDataDao.querySerByCode(con, code, id,active);
    }

    @Get
    @Path("/server/checkIp")
    public EnvServer serverByIp(@JdbcConn Connection con,String ipAddress,@Nullable String id,@Nullable Boolean active)throws SQLException {
        return this.envDataDao.querySerByIp(con, ipAddress, id, active);
    }

    @Get
    @Path("/transducer/checkSeq")
    public EnvTransducer transducerBySeq(@JdbcConn Connection con, Integer seq, @Nullable String id, @Nullable Boolean active)throws SQLException {
        return this.envDataDao.queryTraBySeq(con, seq,id,active);
    }

    @Get
    @Path("/transducer/checkCode")
    public EnvTransducer transducerByCode(@JdbcConn Connection con,Integer code,@Nullable String id,@Nullable Boolean active)throws SQLException {
        return this.envDataDao.queryTraByCode(con, code, id,active);
    }

    @Path("/all/byBridge")
    @Get
    @LoginUser
    public Map<String,Object> queryAll(@JdbcConn Connection con,@Nullable Boolean active,String bridgeId)throws SQLException {
        Map<String, Object> result = new HashMap<>();
        List<EnvServer> servers = envDataDao.queryByBridge(con, active, bridgeId);
        result.put("server", servers);
        if (servers.size() > 0) {
            String[] serverIds = new String[servers.size()];
            for (int i = 0; i < servers.size(); ++i) {
                serverIds[i] = servers.get(i).getId();
            }
            List<EnvTranLastValue> transducers = envDataDao.queryByServer(con, active, serverIds);
            for (EnvTranLastValue transducer : transducers) {
                final int id = transducer.getSeq();
                EnvData envData = query(id);
                if (envData != null) {
                    transducer.setLastValue(envData.getItemValue());
                }else {
                    transducer.setLastValue(JdbcUtil.queryDouble(con, "select item_value from env_data WHERE item_id = ? ORDER BY create_time desc limit 1", new PreparedStatementConfig() {
                        @Override
                        public void config(PreparedStatement preparedStatement) throws SQLException {
                            preparedStatement.setInt(1, id);
                        }
                    }, 0));
                }
            }
            result.put("transducer", transducers);
        } else {
            result.put("transducer", servers);
        }
        return result;
    }

    @Path("/history")
    @Get
    public Map<Integer,Object> history(@JdbcConn Connection con,Integer[] transducers,String bt)throws SQLException {
        Map<Integer, Object> map = new HashMap<>();
        for (Integer transducer : transducers) {
            map.put(transducer, envDataDao.queryById(con, bt, transducer));
        }
        return map;
    }

    @Path("/server/typeCode")
    @Get
    public List<TypeaheadItem> get(@JdbcConn Connection con,@Nullable String bridgeId,@Nullable final Boolean active)throws SQLException {
        List<EnvServer> envServers = envDataDao.queryAllServer(con, bridgeId, active);
        List<TypeaheadItem> results = new ArrayList<>();
        for (EnvServer envServer : envServers) {
            TypeaheadItem typeaheadItem = new TypeaheadItem();
            typeaheadItem.setCaption(envServer.getCode());
            typeaheadItem.setCode(envServer.getId());
            results.add(typeaheadItem);
        }
        return results;
    }

    @Path("/transducer/position")
    public List<String> getPosition(@JdbcConn Connection con)throws SQLException {
        return JdbcUtil.queryList(con, "SELECT POSITION FROM ENV_TRANSDUCER GROUP BY POSITION ORDER BY POSITION", new ResultSetExtractor<String>() {
            @Override
            public String extractData(ResultSet resultSet) throws SQLException {
                return resultSet.getString(1);
            }
        });
    }


    public static class TypeaheadItem{
        private String code;
        private String caption;
        public String getCode() {
            return code;
        }
        public void setCode(String code) {
            this.code = code;
        }
        public String getCaption() {
            return caption;
        }
        public void setCaption(String caption) {
            this.caption = caption;
        }

    }

}
