package com.ekexiu.project.bridge.resource.dao;

import com.ekexiu.project.bridge.resource.po.Transducer;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.orm.annotation.dao.Batch;
import org.jfw.apt.orm.annotation.dao.DAO;
import org.jfw.apt.orm.annotation.dao.method.Exclude;
import org.jfw.apt.orm.annotation.dao.method.From;
import org.jfw.apt.orm.annotation.dao.method.IncludeFixSet;
import org.jfw.apt.orm.annotation.dao.method.OrderBy;
import org.jfw.apt.orm.annotation.dao.method.SetSentence;
import org.jfw.apt.orm.annotation.dao.method.operator.Insert;
import org.jfw.apt.orm.annotation.dao.method.operator.PageSelect;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectList;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectOne;
import org.jfw.apt.orm.annotation.dao.method.operator.Update;
import org.jfw.apt.orm.annotation.dao.method.operator.UpdateWith;
import org.jfw.apt.orm.annotation.dao.param.In;
import org.jfw.apt.orm.annotation.dao.param.Set;
import org.jfw.apt.orm.annotation.dao.param.SqlColumn;
import org.jfw.apt.orm.annotation.dao.param.UnEquals;
import org.jfw.apt.orm.core.defaultImpl.StringHandler;
import org.jfw.util.PageQueryResult;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

/**
 * Created by TT on 2018/8/8.
 */
@DAO
public interface TransducerDao {

    @Insert
    int insert(Connection con, Transducer transducer) throws SQLException;

    @UpdateWith
    @From(Transducer.class)
    @SetSentence("active = '0'")
    @IncludeFixSet("modifyTime")
    int logicDelete(Connection con, @Set String modifier, String id) throws SQLException;

    @UpdateWith
    @Batch
    @From(Transducer.class)
    @SetSentence("active = '0'")
    @IncludeFixSet("modifyTime")
    int[] logicDeleteByDevices(Connection con, @Set String modifier,@Batch String[] deviceId) throws SQLException;

    @Update
    @Exclude("active")
    int update(Connection con, Transducer transducer) throws SQLException;

    @SelectOne
    @Nullable
    Transducer query(Connection con, String id) throws SQLException;

    @SelectOne
    @Nullable
    Transducer queryBySeqAndDeviceFromServer(Connection con,@SqlColumn(handlerClass = StringHandler.class,value = "DEVICE_ID IN (SELECT ID FROM COLLECT_DEVICE WHERE SERVER_ID IN (SELECT SERVER_ID FROM COLLECT_DEVICE WHERE ID = ?))")String deviceId, int seq,@Nullable @UnEquals String id,@Nullable Boolean active) throws SQLException;

    @SelectOne
    @Nullable
    Transducer queryByCode(Connection con,String code,@Nullable @UnEquals String id,@Nullable Boolean active) throws SQLException;

    @PageSelect
    @OrderBy("ORDER BY CODE")
    PageQueryResult<Transducer> pageQuery(Connection con, @Nullable Boolean active, @Nullable @SqlColumn(handlerClass = StringHandler.class, value = {"DEVICE_ID IN (SELECT ID FROM COLLECT_DEVICE WHERE CODE = ?)"}) String collectDeviceCode, @Nullable String code, int pageSize, int pageNo) throws SQLException;

    @SelectList
    List<Transducer> queryByDevice(Connection con,@Nullable Boolean active, String deviceId) throws SQLException;

    @SelectList
    List<Transducer> queryByUser(Connection con, @Nullable Boolean active,@In String[] deviceId) throws SQLException;
}
