package com.ekexiu.project.bridge.servlet;

import com.ekexiu.project.bridge.envdata.EnvData;
import com.ekexiu.project.bridge.envdata.EnvDataDao;
import com.ekexiu.project.bridge.envdata.EnvTransducer;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.RegionUtil;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFDataFormat;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.jfw.util.DateUtil;
import org.jfw.util.context.JfwAppContext;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.jdbc.PreparedStatementConfig;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;

/**
 * Created by TT on 2019/3/21.
 */
public class EnvDataExcelServlet extends HttpServlet {
    private static final long serialVersionUID = 5130286556431297299L;

    public static final int timeWidth = 20 * 256;
    public static final int valueWidth = 10 * 256;

    private DataSource dataSource;
    private EnvDataDao envDataDao;

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        String bridgeId = req.getParameter("bridgeId");
        String bt = req.getParameter("bt");
        String et = req.getParameter("et");
        byte[] os = generalExcel(bridgeId, bt, et);
        String time = DateUtil.formatDateTime(System.currentTimeMillis());
        resp.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        resp.setHeader("Content-Disposition", "attachment;filename=" + time.substring(0, 8) + ".xlsx");
        resp.setContentLength(os.length);
        OutputStream out = resp.getOutputStream();
        try {
            out.write(os);
            out.flush();
        } finally {
            out.close();
        }
    }

    private Connection getConnection() throws SQLException {
        return dataSource.getConnection();
    }

    static class Group {
        String time;
        String val;
    }

    static class AidRow {
        String time;
        List<Group> groups;

        public AidRow(String time, int num) {
            this.time = time;
            this.groups = new ArrayList<>(num);
            for (int i = 0; i < num; ++i) {
                this.groups.add(i, new Group());
            }
        }
    }

    private byte[] generalExcel(final String bridgeId, String bt, String et) throws IOException, ServletException {
        java.util.List<EnvTransducer> transducers = new ArrayList<>();
        List<EnvData> data = new ArrayList<>();
        try {
            Connection con = this.getConnection();
            try {
                transducers = envDataDao.queryTraByBridge(con, true, bridgeId);
                int[] transducerIds = new int[transducers.size()];
                for (int i = 0; i < transducers.size(); i++) {
                    transducerIds[i] = transducers.get(i).getSeq();
                    final String unit = transducers.get(i).getUnit();
                    transducers.get(i).setUnit(JdbcUtil.queryString(con, "select caption from sys_dict where dict_code = 'CSDW' and code = ?", new PreparedStatementConfig() {
                        @Override
                        public void config(PreparedStatement preparedStatement) throws SQLException {
                            preparedStatement.setString(1, unit);
                        }
                    }, "无效字典"));
                }
                data = envDataDao.query(con, bt, et, transducerIds);
            } finally {
                con.close();
            }
        } catch (SQLException e) {
            throw new ServletException(e);
        }
        return this.generalExcel("", transducers, data);
    }

    private byte[] generalExcel(final String bridgeId, List<EnvTransducer> transducers, List<EnvData> datas) throws IOException, ServletException {
        int seqs[] = new int[transducers.size()];
        int i = 0;
        for (EnvTransducer t : transducers) {
            seqs[i++] = t.getSeq();
        }
        List<AidRow> aidRows = new LinkedList<>();
        AidRow last = new AidRow(datas.get(0).getCreateTime(), seqs.length);
        String time = datas.get(0).getCreateTime();
        ListIterator<EnvData> it = datas.listIterator();
        while (it.hasNext()) {
            EnvData data = it.next();
            if (!data.getCreateTime().equals(time)) {
                aidRows.add(last);
                last = new AidRow(data.getCreateTime(), seqs.length);
                time = data.getCreateTime();
            }
            for (i = 0; i < seqs.length; ++i) {
                if (seqs[i] == data.getItemId()) {
                    Group group = last.groups.get(i);
                    group.time = data.getItemTime();
                    group.val = "" + data.getItemValue();
                    break;
                }
            }
        }
        Collections.sort(aidRows, new Comparator<AidRow>() {
            @Override
            public int compare(AidRow o1, AidRow o2) {
                return -o1.time.compareTo(o2.time);
            }
        });
        XSSFWorkbook workbook = new XSSFWorkbook();
        Sheet sheet = workbook.createSheet();

        XSSFDataFormat dataFormat = workbook.createDataFormat();

        //单元格格式1：居中
        XSSFCellStyle center = workbook.createCellStyle();
        center.setAlignment(HorizontalAlignment.CENTER);
        center.setDataFormat(dataFormat.getFormat("@"));


        //单元格格式2：边框
        XSSFCellStyle centerAndBorder = workbook.createCellStyle();
        centerAndBorder.setDataFormat(dataFormat.getFormat("@"));
        centerAndBorder.setBorderBottom(BorderStyle.THIN);
        centerAndBorder.setBorderTop(BorderStyle.THIN);
        centerAndBorder.setBorderLeft(BorderStyle.THIN);
        centerAndBorder.setBorderRight(BorderStyle.THIN);

        //标题行
        Row row = sheet.createRow(0);
        Cell title = row.createCell(0, CellType.STRING);
        title.setCellValue(generalTime(datas.get(datas.size() - 1).getCreateTime()) + "-" + generalTime(datas.get(0).getCreateTime()));
        title.setCellStyle(center);
        CellRangeAddress cellRangeAddressTitle = new CellRangeAddress(0, 0, 0, transducers.size() * 2 + 1);
        this.mergedSetBorder(cellRangeAddressTitle, sheet);

        //入库时间
        Row row1 = sheet.createRow(1);
        sheet.setColumnWidth(0, timeWidth);
        Cell row1Cell1 = row1.createCell(0, CellType.STRING);
        row1Cell1.setCellValue("入库时间");
        row1Cell1.setCellStyle(center);
        CellRangeAddress createTimeColumn = new CellRangeAddress(1, 2, 0, 0);
        this.mergedSetBorder(createTimeColumn, sheet);

        //栏目行
        Row row2 = sheet.createRow(2);
        for (int j = 0; j < transducers.size(); j++) {
            EnvTransducer transducer = transducers.get(j);
            Cell name = row1.createCell(j * 2 + 1, CellType.STRING);
            sheet.setColumnWidth(j * 2 + 1, timeWidth);
            sheet.setColumnWidth(j * 2 + 2, valueWidth);
            name.setCellValue(transducer.getName() + "(" + transducer.getSeq() + ")");
            name.setCellStyle(center);
            CellRangeAddress itemName = new CellRangeAddress(1, 1, j * 2 + 1, j * 2 + 2);
            this.mergedSetBorder(itemName, sheet);
            Cell valueTime = row2.createCell(j * 2 + 1, CellType.STRING);
            valueTime.setCellValue("采集时间");
            valueTime.setCellStyle(centerAndBorder);
            Cell value = row2.createCell(j * 2 + 2, CellType.STRING);
            value.setCellValue("值(" + transducer.getUnit() + ")");
            value.setCellStyle(centerAndBorder);
        }

        //填充内容数据
        for (int m = 0; m < aidRows.size(); m++) {
            Row dataRow = sheet.createRow(m + 3);
            Cell createTime = dataRow.createCell(0, CellType.STRING);
            createTime.setCellValue(this.generalTime(aidRows.get(m).time));
            createTime.setCellStyle(centerAndBorder);
            List<Group> groups = aidRows.get(m).groups;
            for (int n = 0; n < groups.size(); n++) {
                Cell itemTime = dataRow.createCell(n * 2 + 1, CellType.STRING);
                itemTime.setCellValue(this.generalTime(groups.get(n).time));
                itemTime.setCellStyle(centerAndBorder);
                Cell itemValue = dataRow.createCell(n * 2 + 2, CellType.STRING);
                itemValue.setCellValue(groups.get(n).val);
                itemValue.setCellStyle(centerAndBorder);
            }
        }


        //输出
        //FileOutputStream fileOutputStream = new FileOutputStream("E:\\XMTT\\xmtt.xlsx");
        //workbook.write(fileOutputStream);
        //fileOutputStream.close();
        //workbook.close();
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        workbook.write(baos);
        workbook.close();
        baos.flush();
        return baos.toByteArray();
    }

    private String generalTime(String time) {
        if (time != null) {
            return time.substring(0, 4) + "/" + time.substring(4, 6) + "/" + time.substring(6, 8) + " " + time.substring(8, 10) + ":" + time.substring(10, 12) + ":" + time.substring(12, 14);
        } else {
            return "";
        }
    }

    private void mergedSetBorder(CellRangeAddress cellRangeAddress, Sheet sheet) {
        sheet.addMergedRegion(cellRangeAddress);
        RegionUtil.setBorderBottom(BorderStyle.THIN, cellRangeAddress, sheet);
        RegionUtil.setBorderTop(BorderStyle.THIN, cellRangeAddress, sheet);
        RegionUtil.setBorderLeft(BorderStyle.THIN, cellRangeAddress, sheet);
        RegionUtil.setBorderRight(BorderStyle.THIN, cellRangeAddress, sheet);
    }

    @Override
    public void init() throws ServletException {
        this.dataSource = JfwAppContext.getDataSource();
        if (this.dataSource == null) {
            throw new ServletException("not found datasource");
        }
        this.envDataDao = (EnvDataDao) JfwAppContext.getBeanFactory().getBean("com_ekexiu_project_bridge_envdata_EnvDataDao");
        if (this.envDataDao == null) {
            throw new ServletException("not found evuDataDao");
        }
    }
}
