package com.ekexiu.project.bridge.servlet;

import com.ekexiu.project.bridge.envdata.EnvData;
import com.ekexiu.project.bridge.envdata.EnvDataService;
import org.jfw.util.DateUtil;
import org.jfw.util.context.JfwAppContext;

import static java.lang.Thread.sleep;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.sql.DataSource;
import java.io.IOException;
import java.io.InputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * Created by TT on 2019/3/6.
 */
public class EnvDataServlet extends HttpServlet {
    private static final long serialVersionUID = 749116393196185727L;
    private DataSource dataSource;
    private EnvDataService envDataService;
    private List<EnvThread> envThreadList = new CopyOnWriteArrayList<EnvThread>();
    private ListenThread listenThread;

    private AtomicBoolean running = new AtomicBoolean(false);

    @Override
    public void init(ServletConfig config) throws ServletException {
        dataSource = JfwAppContext.getDataSource();
        envDataService = (EnvDataService) JfwAppContext.getBeanFactory().getBean("com_ekexiu_project_bridge_envdata_EnvDataService");
        if (envDataService == null) {
            throw new ServletException("not found EnvDataService");
        }
        running.set(true);
        try {
            ServerSocket serverSocket = new ServerSocket(4000);
            listenThread = new ListenThread(serverSocket);
            listenThread.start();
        } catch (IOException e) {
            e.printStackTrace();
        }


    }

    @Override
    public void destroy() {
        this.running.set(false);
        if (listenThread != null) {
            listenThread.interrupt();

            for (; ; ) {
                if (this.envThreadList.size() > 0) {
                    for (Thread thread : this.envThreadList) {
                        thread.interrupt();
                    }
                    try {
                        sleep(500);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                } else {
                    break;
                }
            }

        }
    }

    class ListenThread extends Thread {
        private ServerSocket serverSocket;


        public ListenThread(ServerSocket serverSocket) {
            this.serverSocket = serverSocket;
        }

        @Override
        public void run() {
            while (running.get()) {
                try {
                    Socket socket = serverSocket.accept();
                    EnvThread envThread = new EnvThread(socket);
                    envThreadList.add(envThread);
                    envThread.start();
                } catch (Exception e) {
                }
            }
        }
    }

    class EnvThread extends Thread {
        private Socket socket;
        private char timeArray[] = new char[14];

        public EnvThread(Socket socket) {
            this.socket = socket;
        }

        @Override
        public void run() {
            int idx, len, ret, offset;
            InputStream inputStream = null;
            while (running.get()) {
                try {
                    if (inputStream == null) {
                        inputStream = socket.getInputStream();
                    }
                    byte[] head = new byte[52];
                    idx = 0;
                    len = 52;
                    while (len > 0) {
                        ret = inputStream.read(head, idx, len);
                        if (ret >= 0) {
                            idx += ret;
                            len -= ret;
                        } else {
                            throw new IOException("a");
                        }
                    }
                    if (EnvDataService.dolog.get()) {
                        System.out.print("head:");
                        headtoString(head, 0, head.length);
                    }
                    len = head[51] & 0xFF;
                    len <<= 8;
                    len += head[50] & 0xFF;
                    len <<= 8;
                    len += head[49] & 0xFF;
                    len <<= 8;
                    len += head[48] & 0xFF;
                    len = len * 32;
                    byte[] body = new byte[len];
                    idx = 0;
                    while (len > 0) {
                        ret = inputStream.read(body, idx, len);
                        if (ret >= 0) {
                            idx += ret;
                            len -= ret;
                        } else {
                            throw new IOException("a");
                        }
                    }
                    if (EnvDataService.dolog.get()) {
                        System.out.print("body:");
                        headtoString(body, 0, body.length);
                    }
                    String createTime = DateUtil.formatDateTime(System.currentTimeMillis());
                    len = body.length;
                    offset = 0;
                    List<EnvData> list = new ArrayList<EnvData>();
                    while (len > offset) {
                        int id = body[offset + 3] & 0xFF;
                        id <<= 8;
                        id += body[offset + 2] & 0xFF;
                        id <<= 8;
                        id += body[offset + 1] & 0xFF;
                        id <<= 8;
                        id += body[offset] & 0xFF;

                        double value = bytes2Double(body, offset + 4);

                        timeArray[0] = (char) body[offset + 12];
                        timeArray[1] = (char) body[offset + 13];
                        timeArray[2] = (char) body[offset + 14];
                        timeArray[3] = (char) body[offset + 15];
                        timeArray[4] = (char) body[offset + 17];
                        timeArray[5] = (char) body[offset + 18];
                        timeArray[6] = (char) body[offset + 20];
                        timeArray[7] = (char) body[offset + 21];
                        timeArray[8] = (char) body[offset + 23];
                        timeArray[9] = (char) body[offset + 24];
                        timeArray[10] = (char) body[offset + 26];
                        timeArray[11] = (char) body[offset + 27];
                        timeArray[12] = (char) body[offset + 29];
                        timeArray[13] = (char) body[offset + 30];
                        offset += 32;
                        EnvData envData = new EnvData();
                        envData.setItemId(id);
                        envData.setItemValue(value);
                        envData.setItemTime(new String(timeArray));
                        envData.setCreateTime(createTime);
                        list.add(envData);
                    }

                    try {
                        Connection con = dataSource.getConnection();
                        try {
                            for (EnvData envData : list) {
                                PreparedStatement ps = con.prepareStatement("INSERT INTO ENV_DATA (ITEM_ID,ITEM_VALUE,ITEM_TIME,CREATE_TIME) VALUES (?,?,?,?)");
                                ps.setInt(1, envData.getItemId());
                                ps.setDouble(2, envData.getItemValue());
                                ps.setString(3, envData.getItemTime());
                                ps.setString(4, envData.getCreateTime());
                                ps.executeUpdate();
                            }
                            con.commit();
                            envDataService.setLastData(list);
                        } catch (SQLException e) {
                            con.rollback();
                        } finally {
                            con.close();
                        }
                    } catch (Throwable thr) {

                    }
                } catch (Exception e) {
                    break;
                }
            }
            try {
                socket.close();
            } catch (IOException e1) {
                e1.printStackTrace();
            }
            envThreadList.remove(this);
        }

    }

    public static void headtoString(byte[] head,int idx,int len) {
        int rlen = idx +len;
        System.out.print("[");
        for(int i = idx; i < rlen;++i){
            System.out.print(head[i]);
            System.out.print(",");
        }
        System.out.println("]");
    }

    public static double bytes2Double(byte[] arr, int index) {
        long value = 0;
        for (int i = 0; i < 8; i++) {
            value |= ((long) (arr[index + i] & 0xff)) << (8 * i);
        }
        return Double.longBitsToDouble(value);
    }


}

