package com.ekexiu.project.bridge.servlet;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Type;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.jfw.apt.util.Base64;
import org.jfw.util.ConstData;
import org.jfw.util.context.JfwAppContext;
import org.jfw.util.io.IoUtil;
import org.jfw.util.json.JsonService;
import org.jfw.util.reflect.TypeReference;

import com.ekexiu.project.bridge.collect.CollectDao;
import com.ekexiu.project.bridge.collect.CollectService;
import com.ekexiu.project.bridge.collect.WaveDataError;
import com.ekexiu.project.bridge.collect.WaveDataItem;

public class WaveDataServlet extends HttpServlet {
	private static final long serialVersionUID = 5218822832173871831L;

	private static final Map<String, List<WaveDataItem>> DATA_CACHE = new HashMap<String, List<WaveDataItem>>();

	@SuppressWarnings("unused")
	private static final LinkedList<List<WaveDataItem>> EMPTY_LIST = new LinkedList<List<WaveDataItem>>();

	public static Type DATA_TYPE = new TypeReference<List<Object>>() {
	}.getType();

	private DataSource dataSource;
	private CollectDao collectDao;

	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		response.sendError(400);
	}

	@SuppressWarnings({ "rawtypes" })
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		InputStream in = request.getInputStream();
		final List<WaveDataItem> items = new LinkedList<WaveDataItem>();
		final Map<String, String> errorData = new HashMap<String, String>();
		String cnt = null;
		List<Object> param = null;
		try {
			byte[] buf = new byte[8192];
			IoUtil.copy(in, baos, buf);
			byte[] ba = baos.toByteArray();
			if (ba.length == 0) {
				errorData.put(" ", "EMPTY");
			} else {
				try {
					cnt = new String(baos.toByteArray(), ConstData.UTF8);
				} catch (Exception e) {
					errorData.put(new String(Base64.encodeBase64(ba), ConstData.UTF8), "CHARSET");
				}
				if (cnt != null) {
					try {
						param = JsonService.fromJson(cnt, DATA_TYPE);
					} catch (Exception e) {
						errorData.put(cnt, "JSON_PARSE");
					}
					if (param != null) {
						for (Object obj : param) {
							if (obj instanceof List && ((List)obj).size()>5) {
								WaveDataItem item = new WaveDataItem();
								Object tmpObj = ((List) obj).get(0);
								if (tmpObj instanceof String) {
									item.setSeq((String) tmpObj);
								} else {
									items.clear();
									break;
								}
								tmpObj = ((List) obj).get(3);
								if (tmpObj instanceof String) {
									item.setCtime((String) tmpObj);
								} else {
									items.clear();
									break;
								}
								tmpObj = ((List) obj).get(1);
								if (tmpObj instanceof String) {
									try {
										item.setDid(Integer.parseInt((String) tmpObj));
									} catch (Exception e) {
										items.clear();
										break;
									}
								} else {
									items.clear();
									break;
								}
								tmpObj = ((List) obj).get(2);
								if (tmpObj instanceof String) {
									try {
										item.setCid(Integer.parseInt((String) tmpObj));
									} catch (Exception e) {
										items.clear();
										break;
									}
								} else {
									items.clear();
									break;
								}
								tmpObj = ((List) obj).get(4);
								if (tmpObj instanceof String) {
									try {
										item.setLvalue(Integer.parseInt((String) tmpObj));
									} catch (Exception e) {
										items.clear();
										break;
									}
								} else {
									items.clear();
									break;
								}
								tmpObj = ((List) obj).get(5);
								if (tmpObj instanceof String) {
									try {
										item.setHvalue(Integer.parseInt((String) tmpObj));
									} catch (Exception e) {
										items.clear();
										break;
									}
								} else {
									items.clear();
									break;
								}
								items.add(item);
							} else {
								items.clear();
								break;
							}
						}
					
						if(items.isEmpty()){
							errorData.put(cnt,"WAVEDATA_PARSE");
						}
					}
				}
			}
		} finally {
			in.close();
		}
		response.setDateHeader("Expires", 0);
		response.setHeader("Cache-Control", "no-store, no-cache, must-revalidate");
		response.addHeader("Cache-Control", "post-check=0, pre-check=0");
		response.setHeader("Pragma", "no-cache");
		if (null == request.getCharacterEncoding()) {
			request.setCharacterEncoding("UTF-8");
			request.setCharacterEncoding("UTF-8");
		}
		response.setContentType("application/json");
		java.io.PrintWriter out = response.getWriter();
		if (items.isEmpty()) {
			out.write("{\"status\":0}");
			JfwAppContext.getScheduledExecutorService().execute(new Runnable() {
				@Override
				public void run() {
					try {
						Connection con = dataSource.getConnection();
						try {
							for(Map.Entry<String,String> entry:errorData.entrySet()){
								WaveDataError wde = new WaveDataError();
								wde.setCnt(entry.getKey());
								wde.setReason(entry.getValue());
								wde.setRtime(System.currentTimeMillis());
								collectDao.insertWaveDataError(con, wde);
							}
							con.commit();
						} catch (SQLException e) {
							con.rollback();
						} finally {
							con.close();
						}
					} catch (Throwable thr) {
					}

				}
			});
			
			return;
		}
		out.write("{\"status\":1}");
		synchronized (DATA_CACHE) {
			DATA_CACHE.put(items.get(0).getSeq(), items);
		}
		JfwAppContext.getScheduledExecutorService().execute(new Runnable() {
			@Override
			public void run() {
				try {
					Connection con = dataSource.getConnection();
					try {
						collectDao.insertWaveDataItems(con, items);
						con.commit();
					} catch (SQLException e) {
						con.rollback();
					} finally {
						con.close();
					}
				} catch (Throwable thr) {
				}

			}
		});
	}

	@Override
	public void init() throws ServletException {
		this.dataSource = JfwAppContext.getDataSource();
		if (this.dataSource == null) {
			throw new ServletException("not found datasource");
		}
		this.collectDao = (CollectDao) JfwAppContext.getBeanFactory().getBean("com_ekexiu_project_bridge_collect_CollectDao");
		if (this.dataSource == null) {
			throw new ServletException("not found collectDao");
		}
		CollectService s = (CollectService) JfwAppContext.getBeanFactory().getBean("com_ekexiu_project_bridge_collect_CollectService");
		if (s == null) {
			throw new ServletException("not found CollectService");
		}
		s.setWaveCache(DATA_CACHE);
	}
}