package com.ekexiu.project.bridge.system.service;

import com.ekexiu.project.bridge.mobile.MobilePhoneService;
import com.ekexiu.project.bridge.right.po.UserBridge;
import com.ekexiu.project.bridge.system.dao.UserDao;
import com.ekexiu.project.bridge.system.po.User;
import com.ekexiu.project.bridge.system.vo.SessionUser;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.method.InvalidSession;
import org.jfw.apt.web.annotation.method.ResultToNull;
import org.jfw.apt.web.annotation.method.SetSession;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.SessionVal;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.jdbc.JdbcUtil;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 * Created by TT on 2018/8/7.
 */
@Path("/sys")
public class SysService {

	public static final String DEFAULT_PW_STR = StringUtil.md5("123456");

	private String regMobilePhoneReplaceKey;
	private String regMobilePhoneContentTemplate;
	private long timeLimitWithRegMobilePhone = 3 * 60 * 1000;

	@Autowrie
	private UserDao userDao;
	@Autowrie
	private MobilePhoneService mobilePhoneService;

	public UserDao getUserDao() {
		return userDao;
	}

	public void setUserDao(UserDao userDao) {
		this.userDao = userDao;
	}

	public MobilePhoneService getMobilePhoneService() {
		return mobilePhoneService;
	}

	public void setMobilePhoneService(MobilePhoneService mobilePhoneService) {
		this.mobilePhoneService = mobilePhoneService;
	}

	public String getRegMobilePhoneReplaceKey() {
		return regMobilePhoneReplaceKey;
	}

	public void setRegMobilePhoneReplaceKey(String regMobilePhoneReplaceKey) {
		this.regMobilePhoneReplaceKey = regMobilePhoneReplaceKey;
	}

	public String getRegMobilePhoneContentTemplate() {
		return regMobilePhoneContentTemplate;
	}

	public void setRegMobilePhoneContentTemplate(String regMobilePhoneContentTemplate) {
		this.regMobilePhoneContentTemplate = regMobilePhoneContentTemplate;
	}

	@SetSession("JFW_SESSION_LOGIN_USER=result")
	@Path("/login")
	@Post
    public SessionUser login(@JdbcConn Connection con, String account, String pw, String vc, @Nullable @SessionVal(value = "PIC_LOGIN", remove = true) String code,
                             @Nullable @SessionVal(value = "TIMEOUT_PIC_LOGIN", remove = true) Long timeout) throws SQLException, JfwBaseException {
        if (code == null || timeout == null) {
            throw new JfwBaseException(-60001, "not found sessionVal");
        }
        if (!vc.equals(code)) {
            throw new JfwBaseException(-60003, "vc not right");
        }
        if (System.currentTimeMillis() > timeout) {
            throw new JfwBaseException(-60002, "timeout");
        }
        User user = userDao.login(con, account, StringUtil.md5(pw));
        if (user != null) {
			if(!user.isActive()){
				throw new JfwBaseException(-60005,"user is disabled");
			}
            return makeSessionUser(user);
        }
        return null;
	}

	@Get
	@Path("/user")
	public SessionUser get(@LoginUser SessionUser user) {
		return user;
	}

	@Get
	@Path("/logout")
	@InvalidSession
	public void logout() {
	}

	@Post
	@Path("/insert")
	public String insert(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, User user) throws SQLException {
		String id = StringUtil.buildUUID();
		user.setId(id);
		user.setActive(true);
		user.setPasswd(DEFAULT_PW_STR);
		user.setCreator(sessionUser.getId());
		user.setModifier(sessionUser.getId());
		this.userDao.insert(con, user);
		return id;
	}

	@Get
	@Path("/check")
	public Boolean check(@JdbcConn Connection con,String account,@Nullable String id)throws SQLException {
		return this.userDao.queryByAccount(con, account,id)==null;
	}

	@Get
	@Path("/qo")
	public SessionUser query(@JdbcConn Connection con, String id) throws SQLException {
		return makeSessionUser(this.userDao.query(con, id));
	}

	@Post
	@Path("/update")
	public void update(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, User user) throws SQLException {
		user.setModifier(sessionUser.getId());
		this.userDao.update(con, user);
	}

	@Get
	@Path("/ban")
	public void ban(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, String id) throws SQLException {
		this.userDao.ban(con, sessionUser.getId(), id);
	}

	@Get
	@Path("/pq")
	public PageQueryResult<SessionUser> pageQuery(@JdbcConn Connection con,@Nullable Boolean active, @Nullable String account, @Nullable String name, @Nullable String comp,
			int pageSize, int pageNo) throws SQLException {
		PageQueryResult<User> pageQueryResult = this.userDao.pageQuery(con,active, account, name == null ? null : "%" + name + "%",
				comp == null ? null : "%" + comp + "%", pageSize, pageNo);
		List<User> users = pageQueryResult.getData();
		List<SessionUser> sessionUsers = new ArrayList<>();
		if (!users.isEmpty()) {
			for (User user : users) {
				sessionUsers.add(makeSessionUser(user));
			}
			PageQueryResult<SessionUser> ret = new PageQueryResult<>();
			ret.setPageNo(pageQueryResult.getPageNo());
			ret.setPageSize(pageQueryResult.getPageSize());
			ret.setTotal(pageQueryResult.getTotal());
			ret.setData(sessionUsers);
			return ret;
		}
		return null;
	}

	@Post
	@Path("/changePw")
	public void changePw(@JdbcConn(true) Connection con, @LoginUser SessionUser u, String newPw) throws SQLException, JfwBaseException {
		this.userDao.changePw(con, StringUtil.md5(newPw), u.getId());
	}

	@Post
	@Path("/notice")
	public void updateNotice(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, String cnt) throws SQLException {
		this.userDao.updateNotice(con, cnt, sessionUser.getId());
	}

	@Get
    @Path("/notice")
    public String queryNotice(@JdbcConn Connection con)throws SQLException {
        return JdbcUtil.queryString(con, "SELECT CNT FROM NOTICE", "欢迎使用本系统");
    }

    @SetSession({"RESET_PW_MOBILE=result[0]", "RESET_PW_VC=result[1]", "RESET_PW_TO=result[2]"})
    @ResultToNull
    @Get
    @Path("/resetPwMobile")
    public Object[] resetpwMobile(@JdbcConn(false) Connection con, String account, @DefaultValue("false") boolean checkExists) throws Exception {
		if (checkExists) {
			User user = this.userDao.queryByAccount(con, account,null);
			if (null == user) {
				throw new JfwBaseException(-60000, "not found account");
			}
		}
		Random rd = new Random();
		int vi = rd.nextInt(10000);
		String vcode = String.format("%04d", vi);

		this.mobilePhoneService.sendMessage(account, this.regMobilePhoneContentTemplate, this.regMobilePhoneReplaceKey, vcode);
		Object[] result = new Object[3];
		result[0] = account;
		result[1] = vcode;
		result[2] = System.currentTimeMillis() + 5000 + (3 * 60 * 1000);
		return result;
	}

	@Post
	@Path("/resetPasswordWithMobilePhone")
	public boolean resetPassword(@JdbcConn(true) Connection con, String pw, String vc,
			@Nullable @SessionVal(value = "RESET_PW_MOBILE", remove = true) String account,
			@Nullable @SessionVal(value = "RESET_PW_VC", remove = true) String vcode, @Nullable @SessionVal(value = "RESET_PW_TO", remove = true) Long timeout)
			throws SQLException,JfwBaseException {
		if (vcode == null || account == null || timeout == null) {
			throw  new JfwBaseException(-60001,"not found sessionVal");
		}
		if (System.currentTimeMillis() > timeout) {
			throw new JfwBaseException(-60002,"timeout");
		}
		if(!vc.equals(vcode)){
			throw new JfwBaseException(-60004, "Verification code error");
		}
		this.userDao.updatePasswdWithMobile(con, StringUtil.md5(pw), account);
		return true;
	}

	@Post
	@Path("/userBridge/refresh")
	public void refresh(@JdbcConn(true)Connection con,String id,@Nullable String[] bridges, @LoginUser SessionUser sessionUser)throws SQLException {
		this.userDao.delete(con, id);
		if (bridges != null && bridges.length > 0) {
			UserBridge[] userBridges = new UserBridge[bridges.length];
			for (int i = 0; i < bridges.length; ++i) {
				UserBridge ub = new UserBridge();
				ub.setUid(id);
				ub.setBridge(bridges[i]);
				userBridges[i] = ub;
			}
			this.userDao.insert(con, userBridges);
		}
	}

	@Get
	@Path("/serviceTime")
	public long serviceTime() {
		return System.currentTimeMillis();
	}

	private static SessionUser makeSessionUser(User user) {
		SessionUser sessionUser = new SessionUser();
		sessionUser.setAccount(user.getAccount());
		sessionUser.setComp(user.getComp());
		sessionUser.setEmail(user.getEmail());
		sessionUser.setId(user.getId());
		sessionUser.setJob(user.getJob());
		sessionUser.setName(user.getName());
		sessionUser.setPhone(user.getPhone());
		sessionUser.setRemark(user.getRemark());
		sessionUser.setType(user.getType());
		sessionUser.setActive(user.isActive());
		return sessionUser;
	}
}
