CREATE TABLE SYS_DICT (
        CODE TEXT NOT NULL,
        BCODE TEXT,
        DICT_CODE TEXT NOT NULL,
        CAPTION TEXT NOT NULL,
        FULL_CAPTION TEXT,
        SHORT_CUT TEXT,
        REMARK TEXT,
        READONLY CHAR(1) NOT NULL,
        ACTIVE CHAR(1) NOT NULL);
ALTER TABLE SYS_DICT ADD PRIMARY KEY (DICT_CODE,CODE);
COMMENT ON TABLE SYS_DICT IS '数据字典表';
COMMENT ON COLUMN SYS_DICT.CODE IS '字典项编码';
COMMENT ON COLUMN SYS_DICT.BCODE IS '字典项编码(业务编码)';
COMMENT ON COLUMN SYS_DICT.DICT_CODE IS '字典编码';
COMMENT ON COLUMN SYS_DICT.CAPTION IS '字典项显示';
COMMENT ON COLUMN SYS_DICT.FULL_CAPTION IS '字典项显示(全)';
COMMENT ON COLUMN SYS_DICT.SHORT_CUT IS '助记码';
COMMENT ON COLUMN SYS_DICT.REMARK IS '备注';
COMMENT ON COLUMN SYS_DICT.READONLY IS '是否只读';
COMMENT ON COLUMN SYS_DICT.ACTIVE IS '是否有效';

CREATE TABLE NOTICE (
        CNT TEXT,
        MODIFIER CHAR(32) NOT NULL,
        MODIFY_TIME CHAR(14) NOT NULL);
COMMENT ON TABLE NOTICE IS '系统公告';
COMMENT ON COLUMN NOTICE.CNT IS '公告内容';
COMMENT ON COLUMN NOTICE.MODIFIER IS '最后修改人ID';
COMMENT ON COLUMN NOTICE.MODIFY_TIME IS '最后修改时间（YYYYMMDDHH24MISS）,初始值:20000101010101';

CREATE TABLE LUSER (
        ACTIVE CHAR(1) NOT NULL,
        PASSWD TEXT NOT NULL,
        ID CHAR(32) NOT NULL,
        NAME TEXT NOT NULL,
        ACCOUNT TEXT NOT NULL,
        COMP TEXT NOT NULL,
        JOB TEXT NOT NULL,
        PHONE TEXT,
        EMAIL TEXT,
        REMARK TEXT,
        TYPE INTEGER NOT NULL,
        CREATE_TIME CHAR(14) NOT NULL,
        MODIFY_TIME CHAR(14) NOT NULL,
        CREATOR CHAR(32) NOT NULL,
        MODIFIER CHAR(32) NOT NULL);
ALTER TABLE LUSER ADD PRIMARY KEY (ID);
ALTER TABLE LUSER ADD UNIQUE (ACCOUNT);
COMMENT ON TABLE LUSER IS '用户表';
COMMENT ON COLUMN LUSER.ACTIVE IS '是否可用';
COMMENT ON COLUMN LUSER.PASSWD IS '密码（MD5）';
COMMENT ON COLUMN LUSER.ID IS '用户ID';
COMMENT ON COLUMN LUSER.NAME IS '用户姓名';
COMMENT ON COLUMN LUSER.ACCOUNT IS '用户帐号';
COMMENT ON COLUMN LUSER.COMP IS '用户所属单位';
COMMENT ON COLUMN LUSER.JOB IS '用户职位';
COMMENT ON COLUMN LUSER.PHONE IS '联系电话';
COMMENT ON COLUMN LUSER.EMAIL IS '联系邮件';
COMMENT ON COLUMN LUSER.REMARK IS '备注';
COMMENT ON COLUMN LUSER.TYPE IS '账户类型';
COMMENT ON COLUMN LUSER.CREATE_TIME IS '创建时间（YYYYMMDDHH24MISS）不可修改';
COMMENT ON COLUMN LUSER.MODIFY_TIME IS '最后修改时间（YYYYMMDDHH24MISS）,初始值:20000101010101';
COMMENT ON COLUMN LUSER.CREATOR IS '创建人ID';
COMMENT ON COLUMN LUSER.MODIFIER IS '最后修改人ID';


CREATE TABLE BRIDGE (
        ID CHAR(32) NOT NULL,
        ACTIVE CHAR(1) NOT NULL,
        CODE TEXT NOT NULL,
        SHORT_NAME TEXT NOT NULL,
        NAME TEXT NOT NULL,
        ADDR_CODE TEXT NOT NULL,
        SERVICE_DAY TEXT NOT NULL,
        LOCATION TEXT NOT NULL,
        DESCP TEXT,
        REMARK TEXT,
        IMG TEXT NOT NULL,
        ORG TEXT NOT NULL,
        PHONE TEXT NOT NULL,
        EMAIL TEXT NOT NULL,
        CREATE_TIME CHAR(14) NOT NULL,
        MODIFY_TIME CHAR(14) NOT NULL,
        CREATOR CHAR(32) NOT NULL,
        MODIFIER CHAR(32) NOT NULL);
ALTER TABLE BRIDGE ADD PRIMARY KEY (ID);
ALTER TABLE BRIDGE ADD UNIQUE (SHORT_NAME);
ALTER TABLE BRIDGE ADD UNIQUE (NAME);
COMMENT ON TABLE BRIDGE IS '桥梁信息';
COMMENT ON COLUMN BRIDGE.ID IS 'id';
COMMENT ON COLUMN BRIDGE.ACTIVE IS '是否可用';
COMMENT ON COLUMN BRIDGE.CODE IS '桥梁编号';
COMMENT ON COLUMN BRIDGE.SHORT_NAME IS '桥梁简称';
COMMENT ON COLUMN BRIDGE.NAME IS '桥梁名称（全称）';
COMMENT ON COLUMN BRIDGE.ADDR_CODE IS '桥梁政区划编码';
COMMENT ON COLUMN BRIDGE.SERVICE_DAY IS '桥梁开通日期（YYYYMMDD）';
COMMENT ON COLUMN BRIDGE.LOCATION IS '桥梁定位';
COMMENT ON COLUMN BRIDGE.DESCP IS '桥梁详细描述';
COMMENT ON COLUMN BRIDGE.REMARK IS '桥梁备注';
COMMENT ON COLUMN BRIDGE.ORG IS '桥梁维护机构名称';
COMMENT ON COLUMN BRIDGE.PHONE IS '桥梁维护机构电话';
COMMENT ON COLUMN BRIDGE.EMAIL IS '桥梁维护机构电邮';
COMMENT ON COLUMN BRIDGE.CREATE_TIME IS '创建时间（YYYYMMDDHH24MISS）不可修改';
COMMENT ON COLUMN BRIDGE.MODIFY_TIME IS '最后修改时间（YYYYMMDDHH24MISS）,初始值:20000101010101';
COMMENT ON COLUMN BRIDGE.CREATOR IS '创建人ID';
COMMENT ON COLUMN BRIDGE.MODIFIER IS '最后修改人ID';


CREATE TABLE USER_BRIDGE (
        UID TEXT NOT NULL,
        BRIDGE TEXT NOT NULL);
COMMENT ON TABLE USER_BRIDGE IS '用户桥梁';
COMMENT ON COLUMN USER_BRIDGE.UID IS '用户ID';
COMMENT ON COLUMN USER_BRIDGE.BRIDGE IS '桥梁ID';

CREATE TABLE BRIDGE_SERVER (
        ID CHAR(32) NOT NULL,
        SEQ TEXT NOT NULL,
        ACTIVE CHAR(1) NOT NULL,
        CODE TEXT NOT NULL,
        BRIDGE_ID TEXT NOT NULL,
        DEVICES INTEGER NOT NULL,
        REMARK TEXT,
        CREATE_TIME CHAR(14) NOT NULL,
        MODIFY_TIME CHAR(14) NOT NULL,
        CREATOR CHAR(32) NOT NULL,
        MODIFIER CHAR(32) NOT NULL);
ALTER TABLE BRIDGE_SERVER ADD PRIMARY KEY (ID);
COMMENT ON TABLE BRIDGE_SERVER IS '采集服务器';
COMMENT ON COLUMN BRIDGE_SERVER.ID IS 'id';
COMMENT ON COLUMN BRIDGE_SERVER.SEQ IS '内部编号';
COMMENT ON COLUMN BRIDGE_SERVER.ACTIVE IS '是否可用';
COMMENT ON COLUMN BRIDGE_SERVER.CODE IS '采集服务器编号';
COMMENT ON COLUMN BRIDGE_SERVER.BRIDGE_ID IS '采集服务器所属桥梁Id';
COMMENT ON COLUMN BRIDGE_SERVER.DEVICES IS '采集盒数量';
COMMENT ON COLUMN BRIDGE_SERVER.REMARK IS '备注';
COMMENT ON COLUMN BRIDGE_SERVER.CREATE_TIME IS '创建时间（YYYYMMDDHH24MISS）不可修改';
COMMENT ON COLUMN BRIDGE_SERVER.MODIFY_TIME IS '最后修改时间（YYYYMMDDHH24MISS）,初始值:20000101010101';
COMMENT ON COLUMN BRIDGE_SERVER.CREATOR IS '创建人ID';
COMMENT ON COLUMN BRIDGE_SERVER.MODIFIER IS '最后修改人ID';

CREATE TABLE COLLECT_DEVICE (
        ID CHAR(32) NOT NULL,
        SEQ INTEGER NOT NULL,
        CODE TEXT NOT NULL,
        SERVER_ID TEXT NOT NULL,
        CHANNELS INTEGER NOT NULL,
        REMARK TEXT,
        ACTIVE CHAR(1) NOT NULL,
        CREATE_TIME CHAR(14) NOT NULL,
        MODIFY_TIME CHAR(14) NOT NULL,
        CREATOR CHAR(32) NOT NULL,
        MODIFIER CHAR(32) NOT NULL);
ALTER TABLE COLLECT_DEVICE ADD PRIMARY KEY (ID);
COMMENT ON TABLE COLLECT_DEVICE IS '采集盒';
COMMENT ON COLUMN COLLECT_DEVICE.ID IS 'id';
COMMENT ON COLUMN COLLECT_DEVICE.SEQ IS '内部编号';
COMMENT ON COLUMN COLLECT_DEVICE.CODE IS '采集盒编号';
COMMENT ON COLUMN COLLECT_DEVICE.SERVER_ID IS '采集盒所属服务器ID';
COMMENT ON COLUMN COLLECT_DEVICE.CHANNELS IS '采集盒信道数量';
COMMENT ON COLUMN COLLECT_DEVICE.REMARK IS '备注';
COMMENT ON COLUMN COLLECT_DEVICE.ACTIVE IS '是否可用';
COMMENT ON COLUMN COLLECT_DEVICE.CREATE_TIME IS '创建时间（YYYYMMDDHH24MISS）不可修改';
COMMENT ON COLUMN COLLECT_DEVICE.MODIFY_TIME IS '最后修改时间（YYYYMMDDHH24MISS）,初始值:20000101010101';
COMMENT ON COLUMN COLLECT_DEVICE.CREATOR IS '创建人ID';
COMMENT ON COLUMN COLLECT_DEVICE.MODIFIER IS '最后修改人ID';


CREATE TABLE TRANSDUCER (
        ID CHAR(32) NOT NULL,
        SEQ INTEGER NOT NULL,
        CODE TEXT NOT NULL,
        DEVICE_ID TEXT NOT NULL,
        CABLE_TYPE TEXT NOT NULL,
        LOC_TYPE TEXT NOT NULL,
        REMARK TEXT,
        ACTIVE CHAR(1) NOT NULL,
        CREATE_TIME CHAR(14) NOT NULL,
        MODIFY_TIME CHAR(14) NOT NULL,
        CREATOR CHAR(32) NOT NULL,
        MODIFIER CHAR(32) NOT NULL);
ALTER TABLE TRANSDUCER ADD PRIMARY KEY (ID);
COMMENT ON TABLE TRANSDUCER IS '传感器';
COMMENT ON COLUMN TRANSDUCER.ID IS 'id';
COMMENT ON COLUMN TRANSDUCER.SEQ IS '内部编号';
COMMENT ON COLUMN TRANSDUCER.CODE IS '采集盒编号';
COMMENT ON COLUMN TRANSDUCER.DEVICE_ID IS '传感器编号';
COMMENT ON COLUMN TRANSDUCER.CABLE_TYPE IS '传感器所在主缆(dict)';
COMMENT ON COLUMN TRANSDUCER.LOC_TYPE IS '传感器所在位置(dict)';
COMMENT ON COLUMN TRANSDUCER.REMARK IS '备注';
COMMENT ON COLUMN TRANSDUCER.ACTIVE IS '是否可用';
COMMENT ON COLUMN TRANSDUCER.CREATE_TIME IS '创建时间（YYYYMMDDHH24MISS）不可修改';
COMMENT ON COLUMN TRANSDUCER.MODIFY_TIME IS '最后修改时间（YYYYMMDDHH24MISS）,初始值:20000101010101';
COMMENT ON COLUMN TRANSDUCER.CREATOR IS '创建人ID';
COMMENT ON COLUMN TRANSDUCER.MODIFIER IS '最后修改人ID';

CREATE TABLE ALARM_DATA_ITEM (
        ID TEXT NOT NULL,
        SEQ INTEGER NOT NULL,
        STIME TEXT NOT NULL,
        ATIME TEXT NOT NULL,
        DATA TEXT NOT NULL);
ALTER TABLE ALARM_DATA_ITEM ADD PRIMARY KEY (ID,SEQ);
COMMENT ON TABLE ALARM_DATA_ITEM IS '报警数据';
COMMENT ON COLUMN ALARM_DATA_ITEM.ID IS '报警编号';
COMMENT ON COLUMN ALARM_DATA_ITEM.SEQ IS '传感内部编号';
COMMENT ON COLUMN ALARM_DATA_ITEM.STIME IS '起始时间';
COMMENT ON COLUMN ALARM_DATA_ITEM.ATIME IS '报警时间';
COMMENT ON COLUMN ALARM_DATA_ITEM.DATA IS '报警数据，共300个数据点，以英文逗号分隔';


CREATE TABLE ALARM_MSG (
        UID CHAR(32) NOT NULL,
        AID TEXT NOT NULL,
        SERVER TEXT NOT NULL,
        DEVICE INTEGER NOT NULL,
        READED CHAR(1) NOT NULL,
        READ_TIME TEXT,
        CREATE_TIME CHAR(14) NOT NULL,
        ALARM_TIME TEXT NOT NULL);
ALTER TABLE ALARM_MSG ADD PRIMARY KEY (UID,AID);
COMMENT ON TABLE ALARM_MSG IS '报警消息';
COMMENT ON COLUMN ALARM_MSG.UID IS '用户ID';
COMMENT ON COLUMN ALARM_MSG.AID IS '报警ID';
COMMENT ON COLUMN ALARM_MSG.SERVER IS '采集服务器内部编号';
COMMENT ON COLUMN ALARM_MSG.DEVICE IS '采集盒内部编号';
COMMENT ON COLUMN ALARM_MSG.READED IS '是否已读';
COMMENT ON COLUMN ALARM_MSG.READ_TIME IS '读取时间';
COMMENT ON COLUMN ALARM_MSG.CREATE_TIME IS '创建时间（YYYYMMDDHH24MISS）不可修改';
COMMENT ON COLUMN ALARM_MSG.ALARM_TIME IS '报警时间';


CREATE TABLE WAVE_DATA_ITEM (
        SEQ TEXT NOT NULL,
        CTIME TEXT NOT NULL,
        DID INTEGER NOT NULL,
        CID INTEGER NOT NULL,
        HVALUE INTEGER NOT NULL,
        LVALUE INTEGER NOT NULL);
ALTER TABLE WAVE_DATA_ITEM ADD PRIMARY KEY (CTIME,SEQ,DID,CID);
COMMENT ON TABLE WAVE_DATA_ITEM IS '监控历史数据';
COMMENT ON COLUMN WAVE_DATA_ITEM.SEQ IS '采集服务器内部编号';
COMMENT ON COLUMN WAVE_DATA_ITEM.CTIME IS '上传时间（YYYYMMDDHH24MISS';
COMMENT ON COLUMN WAVE_DATA_ITEM.DID IS '采集盒内部编号';
COMMENT ON COLUMN WAVE_DATA_ITEM.CID IS '传感器内部编号';
COMMENT ON COLUMN WAVE_DATA_ITEM.HVALUE IS '一秒内最大幅度值';
COMMENT ON COLUMN WAVE_DATA_ITEM.LVALUE IS '一秒内最小幅度值';



INSERT INTO LUSER (active, passwd, id, name,account, comp,job,phone,email,remark,type, create_time,modify_time, creator, modifier) 
VALUES 
('1', 'E10ADC3949BA59ABBE56E057F20F883E', '77005D20EAB44360918AFD2EA5B867AE', 'admin', '18888888888', 'kexiu', 'a', '18888888888', 
'admin@ekexiu.com', NULL, '1', '20180801123001', '20000101010101', '77005D20EAB44360918AFD2EA5B867AE', '77005D20EAB44360918AFD2EA5B867AE');

INSERT INTO sys_dict (code, bcode, dict_code, caption, full_caption, short_cut, remark, readonly, active) VALUES ('01', NULL, 'ZLLX', '上游主缆', NULL, NULL, NULL, '0', '1');
INSERT INTO sys_dict (code, bcode, dict_code, caption, full_caption, short_cut, remark, readonly, active) VALUES ('02', NULL, 'ZLLX', '下游主缆', NULL, NULL, NULL, '0', '1');
INSERT INTO sys_dict (code, bcode, dict_code, caption, full_caption, short_cut, remark, readonly, active) VALUES ('01', NULL, 'ZLWZ', '东跨', NULL, NULL, NULL, '0', '1');
INSERT INTO sys_dict (code, bcode, dict_code, caption, full_caption, short_cut, remark, readonly, active) VALUES ('02', NULL, 'ZLWZ', '西跨', NULL, NULL, NULL, '0', '1');

CREATE TABLE ALARM_DATA_ERROR (
        CNT TEXT NOT NULL,
        RTIME BIGINT NOT NULL,
        REASON TEXT NOT NULL);
COMMENT ON TABLE ALARM_DATA_ERROR IS '报警数据错误表';
COMMENT ON COLUMN ALARM_DATA_ERROR.CNT IS '接口数据';
COMMENT ON COLUMN ALARM_DATA_ERROR.RTIME IS '接口数据接收时间';
COMMENT ON COLUMN ALARM_DATA_ERROR.REASON IS '接口数据错误原因';

CREATE TABLE WAVE_DATA_ERROR (
        CNT TEXT NOT NULL,
        RTIME BIGINT NOT NULL,
        REASON TEXT NOT NULL);
COMMENT ON TABLE WAVE_DATA_ERROR IS '实时数据错误表';
COMMENT ON COLUMN WAVE_DATA_ERROR.CNT IS '接口数据';
COMMENT ON COLUMN WAVE_DATA_ERROR.RTIME IS '接口数据接收时间';
COMMENT ON COLUMN WAVE_DATA_ERROR.REASON IS '接口数据错误原因';

INSERT INTO notice(cnt,modifier,modify_time) VALUES ('欢迎使用本系统！', '111                             ', '20180101010101');


INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621227', NULL, 'XZQH', '徽县','甘肃省陇南市徽县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621228', NULL, 'XZQH', '两当县','甘肃省陇南市两当县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621225', NULL, 'XZQH', '西和县','甘肃省陇南市西和县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621226', NULL, 'XZQH', '礼县','甘肃省陇南市礼县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431230', NULL, 'XZQH', '通道侗族自治县','湖南省怀化市通道侗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320281', NULL, 'XZQH', '江阴市','江苏省无锡市江阴市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320282', NULL, 'XZQH', '宜兴市','江苏省无锡市宜兴市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431222', NULL, 'XZQH', '沅陵县','湖南省怀化市沅陵县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431223', NULL, 'XZQH', '辰溪县','湖南省怀化市辰溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431224', NULL, 'XZQH', '溆浦县','湖南省怀化市溆浦县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431225', NULL, 'XZQH', '会同县','湖南省怀化市会同县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431226', NULL, 'XZQH', '麻阳苗族自治县','湖南省怀化市麻阳苗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431227', NULL, 'XZQH', '新晃侗族自治县','湖南省怀化市新晃侗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431228', NULL, 'XZQH', '芷江侗族自治县','湖南省怀化市芷江侗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431229', NULL, 'XZQH', '靖州苗族侗族自治县','湖南省怀化市靖州苗族侗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341126', NULL, 'XZQH', '凤阳县','安徽省滁州市凤阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341125', NULL, 'XZQH', '定远县','安徽省滁州市定远县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341124', NULL, 'XZQH', '全椒县','安徽省滁州市全椒县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431221', NULL, 'XZQH', '中方县','湖南省怀化市中方县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341122', NULL, 'XZQH', '来安县','安徽省滁州市来安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511703', NULL, 'XZQH', '达川区','四川省达州市达川区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511702', NULL, 'XZQH', '通川区','四川省达州市通川区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511700', NULL, 'XZQH', '达州市','四川省达州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350825', NULL, 'XZQH', '连城县','福建省龙岩市连城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350824', NULL, 'XZQH', '武平县','福建省龙岩市武平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350823', NULL, 'XZQH', '上杭县','福建省龙岩市上杭县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621222', NULL, 'XZQH', '文县','甘肃省陇南市文县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621221', NULL, 'XZQH', '成县','甘肃省陇南市成县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350821', NULL, 'XZQH', '长汀县','福建省龙岩市长汀县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621224', NULL, 'XZQH', '康县','甘肃省陇南市康县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621223', NULL, 'XZQH', '宕昌县','甘肃省陇南市宕昌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430121', NULL, 'XZQH', '长沙县','湖南省长沙市长沙县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230621', NULL, 'XZQH', '肇州县','黑龙江省大庆市肇州县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230622', NULL, 'XZQH', '肇源县','黑龙江省大庆市肇源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230623', NULL, 'XZQH', '林甸县','黑龙江省大庆市林甸县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230624', NULL, 'XZQH', '杜尔伯特蒙古族自治县','黑龙江省大庆市杜尔伯特蒙古族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371581', NULL, 'XZQH', '临清市','山东省聊城市临清市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430112', NULL, 'XZQH', '望城区','湖南省长沙市望城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430111', NULL, 'XZQH', '雨花区','湖南省长沙市雨花区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341102', NULL, 'XZQH', '琅琊区','安徽省滁州市琅琊区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341100', NULL, 'XZQH', '滁州市','安徽省滁州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140581', NULL, 'XZQH', '高平市','山西省晋城市高平市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341103', NULL, 'XZQH', '南谯区','安徽省滁州市南谯区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('232700', NULL, 'XZQH', '大兴安岭地区','黑龙江省大兴安岭地区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430105', NULL, 'XZQH', '开福区','湖南省长沙市开福区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621200', NULL, 'XZQH', '陇南市','甘肃省陇南市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430104', NULL, 'XZQH', '岳麓区','湖南省长沙市岳麓区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621202', NULL, 'XZQH', '武都区','甘肃省陇南市武都区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430100', NULL, 'XZQH', '长沙市','湖南省长沙市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430103', NULL, 'XZQH', '天心区','湖南省长沙市天心区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430102', NULL, 'XZQH', '芙蓉区','湖南省长沙市芙蓉区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131023', NULL, 'XZQH', '永清县','河北省廊坊市永清县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131024', NULL, 'XZQH', '香河县','河北省廊坊市香河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540126', NULL, 'XZQH', '达孜县','西藏自治区拉萨市达孜县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131022', NULL, 'XZQH', '固安县','河北省廊坊市固安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540127', NULL, 'XZQH', '墨竹工卡县','西藏自治区拉萨市墨竹工卡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540124', NULL, 'XZQH', '曲水县','西藏自治区拉萨市曲水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340711', NULL, 'XZQH', '郊区','安徽省铜陵市郊区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150105', NULL, 'XZQH', '赛罕区','内蒙古自治区呼和浩特市赛罕区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131028', NULL, 'XZQH', '大厂回族自治县','河北省廊坊市大厂回族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150104', NULL, 'XZQH', '玉泉区','内蒙古自治区呼和浩特市玉泉区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131025', NULL, 'XZQH', '大城县','河北省廊坊市大城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540122', NULL, 'XZQH', '当雄县','西藏自治区拉萨市当雄县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150103', NULL, 'XZQH', '回民区','内蒙古自治区呼和浩特市回民区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131026', NULL, 'XZQH', '文安县','河北省廊坊市文安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540123', NULL, 'XZQH', '尼木县','西藏自治区拉萨市尼木县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150102', NULL, 'XZQH', '新城区','内蒙古自治区呼和浩特市新城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150100', NULL, 'XZQH', '呼和浩特市','内蒙古自治区呼和浩特市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230606', NULL, 'XZQH', '大同区','黑龙江省大庆市大同区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230605', NULL, 'XZQH', '红岗区','黑龙江省大庆市红岗区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230604', NULL, 'XZQH', '让胡路区','黑龙江省大庆市让胡路区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230603', NULL, 'XZQH', '龙凤区','黑龙江省大庆市龙凤区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341182', NULL, 'XZQH', '明光市','安徽省滁州市明光市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230602', NULL, 'XZQH', '萨尔图区','黑龙江省大庆市萨尔图区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341181', NULL, 'XZQH', '天长市','安徽省滁州市天长市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230600', NULL, 'XZQH', '大庆市','黑龙江省大庆市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231202', NULL, 'XZQH', '北林区','黑龙江省绥化市北林区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231200', NULL, 'XZQH', '绥化市','黑龙江省绥化市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632522', NULL, 'XZQH', '同德县','青海省海南藏族自治州同德县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632523', NULL, 'XZQH', '贵德县','青海省海南藏族自治州贵德县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632524', NULL, 'XZQH', '兴海县','青海省海南藏族自治州兴海县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632525', NULL, 'XZQH', '贵南县','青海省海南藏族自治州贵南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340722', NULL, 'XZQH', '枞阳县','安徽省铜陵市枞阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('533400', NULL, 'XZQH', '迪庆藏族自治州','云南省迪庆藏族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540121', NULL, 'XZQH', '林周县','西藏自治区拉萨市林周县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131003', NULL, 'XZQH', '广阳区','河北省廊坊市广阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131002', NULL, 'XZQH', '安次区','河北省廊坊市安次区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131000', NULL, 'XZQH', '廊坊市','河北省廊坊市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('533401', NULL, 'XZQH', '香格里拉市','云南省迪庆藏族自治州香格里拉市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231222', NULL, 'XZQH', '兰西县','黑龙江省绥化市兰西县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231223', NULL, 'XZQH', '青冈县','黑龙江省绥化市青冈县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231221', NULL, 'XZQH', '望奎县','黑龙江省绥化市望奎县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231226', NULL, 'XZQH', '绥棱县','黑龙江省绥化市绥棱县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231224', NULL, 'XZQH', '庆安县','黑龙江省绥化市庆安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231225', NULL, 'XZQH', '明水县','黑龙江省绥化市明水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511722', NULL, 'XZQH', '宣汉县','四川省达州市宣汉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340700', NULL, 'XZQH', '铜陵市','安徽省铜陵市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511723', NULL, 'XZQH', '开江县','四川省达州市开江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511724', NULL, 'XZQH', '大竹县','四川省达州市大竹县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340706', NULL, 'XZQH', '义安区','安徽省铜陵市义安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511725', NULL, 'XZQH', '渠县','四川省达州市渠县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340705', NULL, 'XZQH', '铜官区','安徽省铜陵市铜官区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431281', NULL, 'XZQH', '洪江市','湖南省怀化市洪江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350800', NULL, 'XZQH', '龙岩市','福建省龙岩市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210782', NULL, 'XZQH', '北镇市','辽宁省锦州市北镇市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350802', NULL, 'XZQH', '新罗区','福建省龙岩市新罗区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('533422', NULL, 'XZQH', '德钦县','云南省迪庆藏族自治州德钦县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350803', NULL, 'XZQH', '永定区','福建省龙岩市永定区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430802', NULL, 'XZQH', '永定区','湖南省张家界市永定区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430800', NULL, 'XZQH', '张家界市','湖南省张家界市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430181', NULL, 'XZQH', '浏阳市','湖南省长沙市浏阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451381', NULL, 'XZQH', '合山市','广西壮族自治区来宾市合山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370523', NULL, 'XZQH', '广饶县','山东省东营市广饶县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370522', NULL, 'XZQH', '利津县','山东省东营市利津县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450921', NULL, 'XZQH', '容县','广西壮族自治区玉林市容县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320206', NULL, 'XZQH', '惠山区','江苏省无锡市惠山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450923', NULL, 'XZQH', '博白县','广西壮族自治区玉林市博白县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320205', NULL, 'XZQH', '锡山区','江苏省无锡市锡山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450922', NULL, 'XZQH', '陆川县','广西壮族自治区玉林市陆川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320200', NULL, 'XZQH', '无锡市','江苏省无锡市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430811', NULL, 'XZQH', '武陵源区','湖南省张家界市武陵源区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450924', NULL, 'XZQH', '兴业县','广西壮族自治区玉林市兴业县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430821', NULL, 'XZQH', '慈利县','湖南省张家界市慈利县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430822', NULL, 'XZQH', '桑植县','湖南省张家界市桑植县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640000', NULL, 'XZQH', '宁夏回族自治区','宁夏回族自治区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410306', NULL, 'XZQH', '吉利区','河南省洛阳市吉利区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410304', NULL, 'XZQH', '瀍河回族区','河南省洛阳市瀍河回族区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410305', NULL, 'XZQH', '涧西区','河南省洛阳市涧西区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410302', NULL, 'XZQH', '老城区','河南省洛阳市老城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410303', NULL, 'XZQH', '西工区','河南省洛阳市西工区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410300', NULL, 'XZQH', '洛阳市','河南省洛阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540100', NULL, 'XZQH', '拉萨市','西藏自治区拉萨市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540103', NULL, 'XZQH', '堆龙德庆区','西藏自治区拉萨市堆龙德庆区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540102', NULL, 'XZQH', '城关区','西藏自治区拉萨市城关区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370502', NULL, 'XZQH', '东营区','山东省东营市东营区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370500', NULL, 'XZQH', '东营市','山东省东营市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370505', NULL, 'XZQH', '垦利区','山东省东营市垦利区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370503', NULL, 'XZQH', '河口区','山东省东营市河口区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410311', NULL, 'XZQH', '洛龙区','河南省洛阳市洛龙区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410325', NULL, 'XZQH', '嵩县','河南省洛阳市嵩县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410324', NULL, 'XZQH', '栾川县','河南省洛阳市栾川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410323', NULL, 'XZQH', '新安县','河南省洛阳市新安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410322', NULL, 'XZQH', '孟津县','河南省洛阳市孟津县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410329', NULL, 'XZQH', '伊川县','河南省洛阳市伊川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410328', NULL, 'XZQH', '洛宁县','河南省洛阳市洛宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410327', NULL, 'XZQH', '宜阳县','河南省洛阳市宜阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410326', NULL, 'XZQH', '汝阳县','河南省洛阳市汝阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430124', NULL, 'XZQH', '宁乡市','湖南省长沙市宁乡市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('232721', NULL, 'XZQH', '呼玛县','黑龙江省大兴安岭地区呼玛县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('232723', NULL, 'XZQH', '漠河县','黑龙江省大兴安岭地区漠河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('232722', NULL, 'XZQH', '塔河县','黑龙江省大兴安岭地区塔河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320213', NULL, 'XZQH', '梁溪区','江苏省无锡市梁溪区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320211', NULL, 'XZQH', '滨湖区','江苏省无锡市滨湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431202', NULL, 'XZQH', '鹤城区','湖南省怀化市鹤城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320214', NULL, 'XZQH', '新吴区','江苏省无锡市新吴区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431200', NULL, 'XZQH', '怀化市','湖南省怀化市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450981', NULL, 'XZQH', '北流市','广西壮族自治区玉林市北流市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522622', NULL, 'XZQH', '黄平县','贵州省黔东南苗族侗族自治州黄平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522624', NULL, 'XZQH', '三穗县','贵州省黔东南苗族侗族自治州三穗县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522623', NULL, 'XZQH', '施秉县','贵州省黔东南苗族侗族自治州施秉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522626', NULL, 'XZQH', '岑巩县','贵州省黔东南苗族侗族自治州岑巩县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522625', NULL, 'XZQH', '镇远县','贵州省黔东南苗族侗族自治州镇远县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522628', NULL, 'XZQH', '锦屏县','贵州省黔东南苗族侗族自治州锦屏县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522627', NULL, 'XZQH', '天柱县','贵州省黔东南苗族侗族自治州天柱县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522629', NULL, 'XZQH', '剑河县','贵州省黔东南苗族侗族自治州剑河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451302', NULL, 'XZQH', '兴宾区','广西壮族自治区来宾市兴宾区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440507', NULL, 'XZQH', '龙湖区','广东省汕头市龙湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451300', NULL, 'XZQH', '来宾市','广西壮族自治区来宾市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610481', NULL, 'XZQH', '兴平市','陕西省咸阳市兴平市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441224', NULL, 'XZQH', '怀集县','广东省肇庆市怀集县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370481', NULL, 'XZQH', '滕州市','山东省枣庄市滕州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441223', NULL, 'XZQH', '广宁县','广东省肇庆市广宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440500', NULL, 'XZQH', '汕头市','广东省汕头市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441226', NULL, 'XZQH', '德庆县','广东省肇庆市德庆县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522630', NULL, 'XZQH', '台江县','贵州省黔东南苗族侗族自治州台江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441225', NULL, 'XZQH', '封开县','广东省肇庆市封开县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522631', NULL, 'XZQH', '黎平县','贵州省黔东南苗族侗族自治州黎平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340000', NULL, 'XZQH', '安徽省','安徽省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211481', NULL, 'XZQH', '兴城市','辽宁省葫芦岛市兴城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341800', NULL, 'XZQH', '宣城市','安徽省宣城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341802', NULL, 'XZQH', '宣州区','安徽省宣城市宣州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440514', NULL, 'XZQH', '潮南区','广东省汕头市潮南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440515', NULL, 'XZQH', '澄海区','广东省汕头市澄海区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('820103', NULL, 'XZQH', '大堂区','澳门特别行政区null大堂区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440512', NULL, 'XZQH', '濠江区','广东省汕头市濠江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('820104', NULL, 'XZQH', '望德堂区','澳门特别行政区null望德堂区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440513', NULL, 'XZQH', '潮阳区','广东省汕头市潮阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('820101', NULL, 'XZQH', '花地玛堂区','澳门特别行政区null花地玛堂区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('820102', NULL, 'XZQH', '圣安多尼堂区','澳门特别行政区null圣安多尼堂区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440511', NULL, 'XZQH', '金平区','广东省汕头市金平区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('820109', NULL, 'XZQH', '澳门新城','澳门特别行政区null澳门新城', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('820106', NULL, 'XZQH', '嘉模堂区','澳门特别行政区null嘉模堂区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152202', NULL, 'XZQH', '阿尔山市','内蒙古自治区兴安盟阿尔山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('820105', NULL, 'XZQH', '风顺堂区','澳门特别行政区null风顺堂区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510302', NULL, 'XZQH', '自流井区','四川省自贡市自流井区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('820108', NULL, 'XZQH', '路氹城','澳门特别行政区null路氹城', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152200', NULL, 'XZQH', '兴安盟','内蒙古自治区兴安盟', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510303', NULL, 'XZQH', '贡井区','四川省自贡市贡井区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('820107', NULL, 'XZQH', '圣方济各堂区','澳门特别行政区null圣方济各堂区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152201', NULL, 'XZQH', '乌兰浩特市','内蒙古自治区兴安盟乌兰浩特市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510304', NULL, 'XZQH', '大安区','四川省自贡市大安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451321', NULL, 'XZQH', '忻城县','广西壮族自治区来宾市忻城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451322', NULL, 'XZQH', '象州县','广西壮族自治区来宾市象州县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451323', NULL, 'XZQH', '武宣县','广西壮族自治区来宾市武宣县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451324', NULL, 'XZQH', '金秀瑶族自治县','广西壮族自治区来宾市金秀瑶族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510311', NULL, 'XZQH', '沿滩区','四川省自贡市沿滩区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530630', NULL, 'XZQH', '水富县','云南省昭通市水富县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522636', NULL, 'XZQH', '丹寨县','贵州省黔东南苗族侗族自治州丹寨县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522635', NULL, 'XZQH', '麻江县','贵州省黔东南苗族侗族自治州麻江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522634', NULL, 'XZQH', '雷山县','贵州省黔东南苗族侗族自治州雷山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522633', NULL, 'XZQH', '从江县','贵州省黔东南苗族侗族自治州从江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522632', NULL, 'XZQH', '榕江县','贵州省黔东南苗族侗族自治州榕江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510300', NULL, 'XZQH', '自贡市','四川省自贡市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530629', NULL, 'XZQH', '威信县','云南省昭通市威信县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530628', NULL, 'XZQH', '彝良县','云南省昭通市彝良县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530627', NULL, 'XZQH', '镇雄县','云南省昭通市镇雄县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530626', NULL, 'XZQH', '绥江县','云南省昭通市绥江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530625', NULL, 'XZQH', '永善县','云南省昭通市永善县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152224', NULL, 'XZQH', '突泉县','内蒙古自治区兴安盟突泉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530624', NULL, 'XZQH', '大关县','云南省昭通市大关县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152223', NULL, 'XZQH', '扎赉特旗','内蒙古自治区兴安盟扎赉特旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530623', NULL, 'XZQH', '盐津县','云南省昭通市盐津县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152222', NULL, 'XZQH', '科尔沁右翼中旗','内蒙古自治区兴安盟科尔沁右翼中旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530622', NULL, 'XZQH', '巧家县','云南省昭通市巧家县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152221', NULL, 'XZQH', '科尔沁右翼前旗','内蒙古自治区兴安盟科尔沁右翼前旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530621', NULL, 'XZQH', '鲁甸县','云南省昭通市鲁甸县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('611000', NULL, 'XZQH', '商洛市','陕西省商洛市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('611002', NULL, 'XZQH', '商州区','陕西省商洛市商州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522600', NULL, 'XZQH', '黔东南苗族侗族自治州','贵州省黔东南苗族侗族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530602', NULL, 'XZQH', '昭阳区','云南省昭通市昭阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522601', NULL, 'XZQH', '凯里市','贵州省黔东南苗族侗族自治州凯里市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530600', NULL, 'XZQH', '昭通市','云南省昭通市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441203', NULL, 'XZQH', '鼎湖区','广东省肇庆市鼎湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440523', NULL, 'XZQH', '南澳县','广东省汕头市南澳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441204', NULL, 'XZQH', '高要区','广东省肇庆市高要区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441202', NULL, 'XZQH', '端州区','广东省肇庆市端州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441200', NULL, 'XZQH', '肇庆市','广东省肇庆市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('611024', NULL, 'XZQH', '山阳县','陕西省商洛市山阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('611025', NULL, 'XZQH', '镇安县','陕西省商洛市镇安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('611026', NULL, 'XZQH', '柞水县','陕西省商洛市柞水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('611021', NULL, 'XZQH', '洛南县','陕西省商洛市洛南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('611022', NULL, 'XZQH', '丹凤县','陕西省商洛市丹凤县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('611023', NULL, 'XZQH', '商南县','陕西省商洛市商南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341822', NULL, 'XZQH', '广德县','安徽省宣城市广德县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341823', NULL, 'XZQH', '泾县','安徽省宣城市泾县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341824', NULL, 'XZQH', '绩溪县','安徽省宣城市绩溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341825', NULL, 'XZQH', '旌德县','安徽省宣城市旌德县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360281', NULL, 'XZQH', '乐平市','江西省景德镇市乐平市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341821', NULL, 'XZQH', '郎溪县','安徽省宣城市郎溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420684', NULL, 'XZQH', '宜城市','湖北省襄阳市宜城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330523', NULL, 'XZQH', '安吉县','浙江省湖州市安吉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610400', NULL, 'XZQH', '咸阳市','陕西省咸阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360222', NULL, 'XZQH', '浮梁县','江西省景德镇市浮梁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350782', NULL, 'XZQH', '武夷山市','福建省南平市武夷山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330521', NULL, 'XZQH', '德清县','浙江省湖州市德清县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350781', NULL, 'XZQH', '邵武市','福建省南平市邵武市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330522', NULL, 'XZQH', '长兴县','浙江省湖州市长兴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131081', NULL, 'XZQH', '霸州市','河北省廊坊市霸州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350783', NULL, 'XZQH', '建瓯市','福建省南平市建瓯市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131082', NULL, 'XZQH', '三河市','河北省廊坊市三河市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321003', NULL, 'XZQH', '邗江区','江苏省扬州市邗江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321002', NULL, 'XZQH', '广陵区','江苏省扬州市广陵区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211400', NULL, 'XZQH', '葫芦岛市','辽宁省葫芦岛市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211403', NULL, 'XZQH', '龙港区','辽宁省葫芦岛市龙港区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211402', NULL, 'XZQH', '连山区','辽宁省葫芦岛市连山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('533323', NULL, 'XZQH', '福贡县','云南省怒江傈僳族自治州福贡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211404', NULL, 'XZQH', '南票区','辽宁省葫芦岛市南票区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321012', NULL, 'XZQH', '江都区','江苏省扬州市江都区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340800', NULL, 'XZQH', '安庆市','安徽省安庆市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140603', NULL, 'XZQH', '平鲁区','山西省朔州市平鲁区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340802', NULL, 'XZQH', '迎江区','安徽省安庆市迎江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340803', NULL, 'XZQH', '大观区','安徽省安庆市大观区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140602', NULL, 'XZQH', '朔城区','山西省朔州市朔城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420683', NULL, 'XZQH', '枣阳市','湖北省襄阳市枣阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632600', NULL, 'XZQH', '果洛藏族自治州','青海省果洛藏族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140600', NULL, 'XZQH', '朔州市','山西省朔州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420682', NULL, 'XZQH', '老河口市','湖北省襄阳市老河口市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511681', NULL, 'XZQH', '华蓥市','四川省广安市华蓥市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532822', NULL, 'XZQH', '勐海县','云南省西双版纳傣族自治州勐海县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532823', NULL, 'XZQH', '勐腊县','云南省西双版纳傣族自治州勐腊县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341881', NULL, 'XZQH', '宁国市','安徽省宣城市宁国市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210781', NULL, 'XZQH', '凌海市','辽宁省锦州市凌海市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321023', NULL, 'XZQH', '宝应县','江苏省扬州市宝应县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120102', NULL, 'XZQH', '河东区','天津市天津市河东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120103', NULL, 'XZQH', '河西区','天津市天津市河西区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120104', NULL, 'XZQH', '南开区','天津市天津市南开区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120105', NULL, 'XZQH', '河北区','天津市天津市河北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610422', NULL, 'XZQH', '三原县','陕西省咸阳市三原县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330502', NULL, 'XZQH', '吴兴区','浙江省湖州市吴兴区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120100', NULL, 'XZQH', '天津市','天津市天津市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330500', NULL, 'XZQH', '湖州市','浙江省湖州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120101', NULL, 'XZQH', '和平区','天津市天津市和平区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330503', NULL, 'XZQH', '南浔区','浙江省湖州市南浔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120106', NULL, 'XZQH', '红桥区','天津市天津市红桥区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211421', NULL, 'XZQH', '绥中县','辽宁省葫芦岛市绥中县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211422', NULL, 'XZQH', '建昌县','辽宁省葫芦岛市建昌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710203', NULL, 'XZQH', '新庄区','台湾省新北市新庄区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140623', NULL, 'XZQH', '右玉县','山西省朔州市右玉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632624', NULL, 'XZQH', '达日县','青海省果洛藏族自治州达日县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710202', NULL, 'XZQH', '土城区','台湾省新北市土城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140624', NULL, 'XZQH', '怀仁县','山西省朔州市怀仁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130324', NULL, 'XZQH', '卢龙县','河北省秦皇岛市卢龙县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632623', NULL, 'XZQH', '甘德县','青海省果洛藏族自治州甘德县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710201', NULL, 'XZQH', '板桥区','台湾省新北市板桥区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140621', NULL, 'XZQH', '山阴县','山西省朔州市山阴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130321', NULL, 'XZQH', '青龙满族自治县','河北省秦皇岛市青龙满族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632622', NULL, 'XZQH', '班玛县','青海省果洛藏族自治州班玛县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710200', NULL, 'XZQH', '新北市','台湾省新北市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140622', NULL, 'XZQH', '应县','山西省朔州市应县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130322', NULL, 'XZQH', '昌黎县','河北省秦皇岛市昌黎县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632621', NULL, 'XZQH', '玛沁县','青海省果洛藏族自治州玛沁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632626', NULL, 'XZQH', '玛多县','青海省果洛藏族自治州玛多县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632625', NULL, 'XZQH', '久治县','青海省果洛藏族自治州久治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710208', NULL, 'XZQH', '乌来区','台湾省新北市乌来区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371622', NULL, 'XZQH', '阳信县','山东省滨州市阳信县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710209', NULL, 'XZQH', '五股区','台湾省新北市五股区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371621', NULL, 'XZQH', '惠民县','山东省滨州市惠民县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371623', NULL, 'XZQH', '无棣县','山东省滨州市无棣县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710204', NULL, 'XZQH', '新店区','台湾省新北市新店区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371626', NULL, 'XZQH', '邹平县','山东省滨州市邹平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710205', NULL, 'XZQH', '深坑区','台湾省新北市深坑区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371625', NULL, 'XZQH', '博兴县','山东省滨州市博兴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710206', NULL, 'XZQH', '石碇区','台湾省新北市石碇区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710207', NULL, 'XZQH', '坪林区','台湾省新北市坪林区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150000', NULL, 'XZQH', '内蒙古自治区','内蒙古自治区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620602', NULL, 'XZQH', '凉州区','甘肃省武威市凉州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321000', NULL, 'XZQH', '扬州市','江苏省扬州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('533325', NULL, 'XZQH', '兰坪白族普米族自治县','云南省怒江傈僳族自治州兰坪白族普米族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('533324', NULL, 'XZQH', '贡山独龙族怒族自治县','云南省怒江傈僳族自治州贡山独龙族怒族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620600', NULL, 'XZQH', '武威市','甘肃省武威市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710212', NULL, 'XZQH', '淡水区','台湾省新北市淡水区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710211', NULL, 'XZQH', '林口区','台湾省新北市林口区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610402', NULL, 'XZQH', '秦都区','陕西省咸阳市秦都区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710214', NULL, 'XZQH', '永和区','台湾省新北市永和区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610403', NULL, 'XZQH', '杨陵区','陕西省咸阳市杨陵区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710213', NULL, 'XZQH', '中和区','台湾省新北市中和区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610404', NULL, 'XZQH', '渭城区','陕西省咸阳市渭城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710210', NULL, 'XZQH', '八里区','台湾省新北市八里区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710218', NULL, 'XZQH', '树林区','台湾省新北市树林区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210727', NULL, 'XZQH', '义县','辽宁省锦州市义县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710217', NULL, 'XZQH', '泰山区','台湾省新北市泰山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210726', NULL, 'XZQH', '黑山县','辽宁省锦州市黑山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710216', NULL, 'XZQH', '芦洲区','台湾省新北市芦洲区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710215', NULL, 'XZQH', '三重区','台湾省新北市三重区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710219', NULL, 'XZQH', '莺歌区','台湾省新北市莺歌区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710220', NULL, 'XZQH', '三峡区','台湾省新北市三峡区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231281', NULL, 'XZQH', '安达市','黑龙江省绥化市安达市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710221', NULL, 'XZQH', '汐止区','台湾省新北市汐止区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710224', NULL, 'XZQH', '三芝区','台湾省新北市三芝区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130302', NULL, 'XZQH', '海港区','河北省秦皇岛市海港区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710225', NULL, 'XZQH', '石门区','台湾省新北市石门区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710222', NULL, 'XZQH', '金山区','台湾省新北市金山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231283', NULL, 'XZQH', '海伦市','黑龙江省绥化市海伦市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130300', NULL, 'XZQH', '秦皇岛市','河北省秦皇岛市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710223', NULL, 'XZQH', '万里区','台湾省新北市万里区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231282', NULL, 'XZQH', '肇东市','黑龙江省绥化市肇东市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130306', NULL, 'XZQH', '抚宁区','河北省秦皇岛市抚宁区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130304', NULL, 'XZQH', '北戴河区','河北省秦皇岛市北戴河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130303', NULL, 'XZQH', '山海关区','河北省秦皇岛市山海关区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120119', NULL, 'XZQH', '蓟州区','天津市天津市蓟州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120118', NULL, 'XZQH', '静海区','天津市天津市静海区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120117', NULL, 'XZQH', '宁河区','天津市天津市宁河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420626', NULL, 'XZQH', '保康县','湖北省襄阳市保康县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710227', NULL, 'XZQH', '贡寮区','台湾省新北市贡寮区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371603', NULL, 'XZQH', '沾化区','山东省滨州市沾化区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120112', NULL, 'XZQH', '津南区','天津市天津市津南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610430', NULL, 'XZQH', '淳化县','陕西省咸阳市淳化县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710226', NULL, 'XZQH', '瑞芳区','台湾省新北市瑞芳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120111', NULL, 'XZQH', '西青区','天津市天津市西青区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610431', NULL, 'XZQH', '武功县','陕西省咸阳市武功县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710229', NULL, 'XZQH', '平溪区','台湾省新北市平溪区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120110', NULL, 'XZQH', '东丽区','天津市天津市东丽区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710228', NULL, 'XZQH', '双溪区','台湾省新北市双溪区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120116', NULL, 'XZQH', '滨海新区','天津市天津市滨海新区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420625', NULL, 'XZQH', '谷城县','湖北省襄阳市谷城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371600', NULL, 'XZQH', '滨州市','山东省滨州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120115', NULL, 'XZQH', '宝坻区','天津市天津市宝坻区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420624', NULL, 'XZQH', '南漳县','湖北省襄阳市南漳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120114', NULL, 'XZQH', '武清区','天津市天津市武清区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371602', NULL, 'XZQH', '滨城区','山东省滨州市滨城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120113', NULL, 'XZQH', '北辰区','天津市天津市北辰区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610428', NULL, 'XZQH', '长武县','陕西省咸阳市长武县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610427', NULL, 'XZQH', '彬县','陕西省咸阳市彬县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610429', NULL, 'XZQH', '旬邑县','陕西省咸阳市旬邑县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610424', NULL, 'XZQH', '乾县','陕西省咸阳市乾县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610423', NULL, 'XZQH', '泾阳县','陕西省咸阳市泾阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610426', NULL, 'XZQH', '永寿县','陕西省咸阳市永寿县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610425', NULL, 'XZQH', '礼泉县','陕西省咸阳市礼泉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620621', NULL, 'XZQH', '民勤县','甘肃省武威市民勤县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620622', NULL, 'XZQH', '古浪县','甘肃省武威市古浪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620623', NULL, 'XZQH', '天祝藏族自治县','甘肃省武威市天祝藏族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360203', NULL, 'XZQH', '珠山区','江西省景德镇市珠山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210700', NULL, 'XZQH', '锦州市','辽宁省锦州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360200', NULL, 'XZQH', '景德镇市','江西省景德镇市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360202', NULL, 'XZQH', '昌江区','江西省景德镇市昌江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210703', NULL, 'XZQH', '凌河区','辽宁省锦州市凌河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210702', NULL, 'XZQH', '古塔区','辽宁省锦州市古塔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650109', NULL, 'XZQH', '米东区','新疆维吾尔自治区乌鲁木齐市米东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650102', NULL, 'XZQH', '天山区','新疆维吾尔自治区乌鲁木齐市天山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650103', NULL, 'XZQH', '沙依巴克区','新疆维吾尔自治区乌鲁木齐市沙依巴克区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650100', NULL, 'XZQH', '乌鲁木齐市','新疆维吾尔自治区乌鲁木齐市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650106', NULL, 'XZQH', '头屯河区','新疆维吾尔自治区乌鲁木齐市头屯河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650107', NULL, 'XZQH', '达坂城区','新疆维吾尔自治区乌鲁木齐市达坂城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650104', NULL, 'XZQH', '新市区','新疆维吾尔自治区乌鲁木齐市新市区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650105', NULL, 'XZQH', '水磨沟区','新疆维吾尔自治区乌鲁木齐市水磨沟区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210711', NULL, 'XZQH', '太和区','辽宁省锦州市太和区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420607', NULL, 'XZQH', '襄州区','湖北省襄阳市襄州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420606', NULL, 'XZQH', '樊城区','湖北省襄阳市樊城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650121', NULL, 'XZQH', '乌鲁木齐县','新疆维吾尔自治区乌鲁木齐市乌鲁木齐县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420602', NULL, 'XZQH', '襄城区','湖北省襄阳市襄城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220000', NULL, 'XZQH', '吉林省','吉林省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420600', NULL, 'XZQH', '襄阳市','湖北省襄阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460105', NULL, 'XZQH', '秀英区','海南省海口市秀英区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460106', NULL, 'XZQH', '龙华区','海南省海口市龙华区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460107', NULL, 'XZQH', '琼山区','海南省海口市琼山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460108', NULL, 'XZQH', '美兰区','海南省海口市美兰区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532801', NULL, 'XZQH', '景洪市','云南省西双版纳傣族自治州景洪市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532800', NULL, 'XZQH', '西双版纳傣族自治州','云南省西双版纳傣族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441284', NULL, 'XZQH', '四会市','广东省肇庆市四会市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460100', NULL, 'XZQH', '海口市','海南省海口市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520628', NULL, 'XZQH', '松桃苗族自治县','贵州省铜仁市松桃苗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520627', NULL, 'XZQH', '沿河土家族自治县','贵州省铜仁市沿河土家族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430000', NULL, 'XZQH', '湖南省','湖南省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520622', NULL, 'XZQH', '玉屏侗族自治县','贵州省铜仁市玉屏侗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320381', NULL, 'XZQH', '新沂市','江苏省徐州市新沂市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520621', NULL, 'XZQH', '江口县','贵州省铜仁市江口县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320382', NULL, 'XZQH', '邳州市','江苏省徐州市邳州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520624', NULL, 'XZQH', '思南县','贵州省铜仁市思南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520623', NULL, 'XZQH', '石阡县','贵州省铜仁市石阡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520626', NULL, 'XZQH', '德江县','贵州省铜仁市德江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520625', NULL, 'XZQH', '印江土家族苗族自治县','贵州省铜仁市印江土家族苗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431127', NULL, 'XZQH', '蓝山县','湖南省永州市蓝山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431128', NULL, 'XZQH', '新田县','湖南省永州市新田县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431129', NULL, 'XZQH', '江华瑶族自治县','湖南省永州市江华瑶族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431123', NULL, 'XZQH', '双牌县','湖南省永州市双牌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431124', NULL, 'XZQH', '道县','湖南省永州市道县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431125', NULL, 'XZQH', '江永县','湖南省永州市江永县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431126', NULL, 'XZQH', '宁远县','湖南省永州市宁远县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511800', NULL, 'XZQH', '雅安市','四川省雅安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511802', NULL, 'XZQH', '雨城区','四川省雅安市雨城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511803', NULL, 'XZQH', '名山区','四川省雅安市名山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431122', NULL, 'XZQH', '东安县','湖南省永州市东安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341222', NULL, 'XZQH', '太和县','安徽省阜阳市太和县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431121', NULL, 'XZQH', '祁阳县','湖南省永州市祁阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341221', NULL, 'XZQH', '临泉县','安徽省阜阳市临泉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340621', NULL, 'XZQH', '濉溪县','安徽省淮北市濉溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341226', NULL, 'XZQH', '颍上县','安徽省阜阳市颍上县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341225', NULL, 'XZQH', '阜南县','安徽省阜阳市阜南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350724', NULL, 'XZQH', '松溪县','福建省南平市松溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350723', NULL, 'XZQH', '光泽县','福建省南平市光泽县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350722', NULL, 'XZQH', '浦城县','福建省南平市浦城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350721', NULL, 'XZQH', '顺昌县','福建省南平市顺昌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360403', NULL, 'XZQH', '浔阳区','江西省九江市浔阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360402', NULL, 'XZQH', '濂溪区','江西省九江市濂溪区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350725', NULL, 'XZQH', '政和县','福建省南平市政和县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360400', NULL, 'XZQH', '九江市','江西省九江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330483', NULL, 'XZQH', '桐乡市','浙江省嘉兴市桐乡市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330482', NULL, 'XZQH', '平湖市','浙江省嘉兴市平湖市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330481', NULL, 'XZQH', '海宁市','浙江省嘉兴市海宁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520602', NULL, 'XZQH', '碧江区','贵州省铜仁市碧江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520603', NULL, 'XZQH', '万山区','贵州省铜仁市万山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140430', NULL, 'XZQH', '沁县','山西省长治市沁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520600', NULL, 'XZQH', '铜仁市','贵州省铜仁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140431', NULL, 'XZQH', '沁源县','山西省长治市沁源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431102', NULL, 'XZQH', '零陵区','湖南省永州市零陵区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431103', NULL, 'XZQH', '冷水滩区','湖南省永州市冷水滩区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420703', NULL, 'XZQH', '华容区','湖北省鄂州市华容区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420704', NULL, 'XZQH', '鄂城区','湖北省鄂州市鄂城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420700', NULL, 'XZQH', '鄂州市','湖北省鄂州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420702', NULL, 'XZQH', '梁子湖区','湖北省鄂州市梁子湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511824', NULL, 'XZQH', '石棉县','四川省雅安市石棉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511823', NULL, 'XZQH', '汉源县','四川省雅安市汉源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511826', NULL, 'XZQH', '芦山县','四川省雅安市芦山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511825', NULL, 'XZQH', '天全县','四川省雅安市天全县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511822', NULL, 'XZQH', '荥经县','四川省雅安市荥经县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431100', NULL, 'XZQH', '永州市','湖南省永州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511827', NULL, 'XZQH', '宝兴县','四川省雅安市宝兴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350703', NULL, 'XZQH', '建阳区','福建省南平市建阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350702', NULL, 'XZQH', '延平区','福建省南平市延平区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350700', NULL, 'XZQH', '南平市','福建省南平市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450881', NULL, 'XZQH', '桂平市','广西壮族自治区贵港市桂平市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230708', NULL, 'XZQH', '美溪区','黑龙江省伊春市美溪区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230709', NULL, 'XZQH', '金山屯区','黑龙江省伊春市金山屯区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210681', NULL, 'XZQH', '东港市','辽宁省丹东市东港市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150200', NULL, 'XZQH', '包头市','内蒙古自治区包头市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210682', NULL, 'XZQH', '凤城市','辽宁省丹东市凤城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150205', NULL, 'XZQH', '石拐区','内蒙古自治区包头市石拐区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150204', NULL, 'XZQH', '青山区','内蒙古自治区包头市青山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150203', NULL, 'XZQH', '昆都仑区','内蒙古自治区包头市昆都仑区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150202', NULL, 'XZQH', '东河区','内蒙古自治区包头市东河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150207', NULL, 'XZQH', '九原区','内蒙古自治区包头市九原区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150206', NULL, 'XZQH', '白云鄂博矿区','内蒙古自治区包头市白云鄂博矿区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231102', NULL, 'XZQH', '爱辉区','黑龙江省黑河市爱辉区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231100', NULL, 'XZQH', '黑河市','黑龙江省黑河市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430903', NULL, 'XZQH', '赫山区','湖南省益阳市赫山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510181', NULL, 'XZQH', '都江堰市','四川省成都市都江堰市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430902', NULL, 'XZQH', '资阳区','湖南省益阳市资阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510182', NULL, 'XZQH', '彭州市','四川省成都市彭州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430900', NULL, 'XZQH', '益阳市','湖南省益阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230703', NULL, 'XZQH', '南岔区','黑龙江省伊春市南岔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510185', NULL, 'XZQH', '简阳市','四川省成都市简阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230702', NULL, 'XZQH', '伊春区','黑龙江省伊春市伊春区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510183', NULL, 'XZQH', '邛崃市','四川省成都市邛崃市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230700', NULL, 'XZQH', '伊春市','黑龙江省伊春市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510184', NULL, 'XZQH', '崇州市','四川省成都市崇州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230707', NULL, 'XZQH', '新青区','黑龙江省伊春市新青区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341282', NULL, 'XZQH', '界首市','安徽省阜阳市界首市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230706', NULL, 'XZQH', '翠峦区','黑龙江省伊春市翠峦区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230705', NULL, 'XZQH', '西林区','黑龙江省伊春市西林区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230704', NULL, 'XZQH', '友好区','黑龙江省伊春市友好区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230710', NULL, 'XZQH', '五营区','黑龙江省伊春市五营区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230712', NULL, 'XZQH', '汤旺河区','黑龙江省伊春市汤旺河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230711', NULL, 'XZQH', '乌马河区','黑龙江省伊春市乌马河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230714', NULL, 'XZQH', '乌伊岭区','黑龙江省伊春市乌伊岭区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230713', NULL, 'XZQH', '带岭区','黑龙江省伊春市带岭区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230716', NULL, 'XZQH', '上甘岭区','黑龙江省伊春市上甘岭区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230715', NULL, 'XZQH', '红星区','黑龙江省伊春市红星区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('533301', NULL, 'XZQH', '泸水市','云南省怒江傈僳族自治州泸水市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('533300', NULL, 'XZQH', '怒江傈僳族自治州','云南省怒江傈僳族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150223', NULL, 'XZQH', '达尔罕茂明安联合旗','内蒙古自治区包头市达尔罕茂明安联合旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150222', NULL, 'XZQH', '固阳县','内蒙古自治区包头市固阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150221', NULL, 'XZQH', '土默特右旗','内蒙古自治区包头市土默特右旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230722', NULL, 'XZQH', '嘉荫县','黑龙江省伊春市嘉荫县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340600', NULL, 'XZQH', '淮北市','安徽省淮北市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140481', NULL, 'XZQH', '潞城市','山西省长治市潞城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340604', NULL, 'XZQH', '烈山区','安徽省淮北市烈山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340603', NULL, 'XZQH', '相山区','安徽省淮北市相山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340602', NULL, 'XZQH', '杜集区','安徽省淮北市杜集区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370602', NULL, 'XZQH', '芝罘区','山东省烟台市芝罘区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540000', NULL, 'XZQH', '西藏自治区','西藏自治区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370600', NULL, 'XZQH', '烟台市','山东省烟台市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640122', NULL, 'XZQH', '贺兰县','宁夏回族自治区银川市贺兰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640121', NULL, 'XZQH', '永宁县','宁夏回族自治区银川市永宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370611', NULL, 'XZQH', '福山区','山东省烟台市福山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370612', NULL, 'XZQH', '牟平区','山东省烟台市牟平区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370613', NULL, 'XZQH', '莱山区','山东省烟台市莱山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340181', NULL, 'XZQH', '巢湖市','安徽省合肥市巢湖市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410200', NULL, 'XZQH', '开封市','河南省开封市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320300', NULL, 'XZQH', '徐州市','江苏省徐州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320303', NULL, 'XZQH', '云龙区','江苏省徐州市云龙区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320302', NULL, 'XZQH', '鼓楼区','江苏省徐州市鼓楼区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450821', NULL, 'XZQH', '平南县','广西壮族自治区贵港市平南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320305', NULL, 'XZQH', '贾汪区','江苏省徐州市贾汪区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411729', NULL, 'XZQH', '新蔡县','河南省驻马店市新蔡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411728', NULL, 'XZQH', '遂平县','河南省驻马店市遂平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411727', NULL, 'XZQH', '汝南县','河南省驻马店市汝南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411726', NULL, 'XZQH', '泌阳县','河南省驻马店市泌阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411725', NULL, 'XZQH', '确山县','河南省驻马店市确山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411724', NULL, 'XZQH', '正阳县','河南省驻马店市正阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640104', NULL, 'XZQH', '兴庆区','宁夏回族自治区银川市兴庆区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411723', NULL, 'XZQH', '平舆县','河南省驻马店市平舆县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640105', NULL, 'XZQH', '西夏区','宁夏回族自治区银川市西夏区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411722', NULL, 'XZQH', '上蔡县','河南省驻马店市上蔡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640106', NULL, 'XZQH', '金凤区','宁夏回族自治区银川市金凤区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411721', NULL, 'XZQH', '西平县','河南省驻马店市西平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510121', NULL, 'XZQH', '金堂县','四川省成都市金堂县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320312', NULL, 'XZQH', '铜山区','江苏省徐州市铜山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320311', NULL, 'XZQH', '泉山区','江苏省徐州市泉山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430921', NULL, 'XZQH', '南县','湖南省益阳市南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430922', NULL, 'XZQH', '桃江县','湖南省益阳市桃江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430923', NULL, 'XZQH', '安化县','湖南省益阳市安化县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370634', NULL, 'XZQH', '长岛县','山东省烟台市长岛县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510114', NULL, 'XZQH', '新都区','四川省成都市新都区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510115', NULL, 'XZQH', '温江区','四川省成都市温江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510116', NULL, 'XZQH', '双流区','四川省成都市双流区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510117', NULL, 'XZQH', '郫都区','四川省成都市郫都区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450800', NULL, 'XZQH', '贵港市','广西壮族自治区贵港市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320324', NULL, 'XZQH', '睢宁县','江苏省徐州市睢宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450802', NULL, 'XZQH', '港北区','广西壮族自治区贵港市港北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510132', NULL, 'XZQH', '新津县','四川省成都市新津县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320321', NULL, 'XZQH', '丰县','江苏省徐州市丰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450804', NULL, 'XZQH', '覃塘区','广西壮族自治区贵港市覃塘区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510131', NULL, 'XZQH', '蒲江县','四川省成都市蒲江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450803', NULL, 'XZQH', '港南区','广西壮族自治区贵港市港南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320322', NULL, 'XZQH', '沛县','江苏省徐州市沛县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411700', NULL, 'XZQH', '驻马店市','河南省驻马店市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510129', NULL, 'XZQH', '大邑县','四川省成都市大邑县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530700', NULL, 'XZQH', '丽江市','云南省丽江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411702', NULL, 'XZQH', '驿城区','河南省驻马店市驿城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530702', NULL, 'XZQH', '古城区','云南省丽江市古城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230781', NULL, 'XZQH', '铁力市','黑龙江省伊春市铁力市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421003', NULL, 'XZQH', '荆州区','湖北省荆州市荆州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430981', NULL, 'XZQH', '沅江市','湖南省益阳市沅江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510100', NULL, 'XZQH', '成都市','四川省成都市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410225', NULL, 'XZQH', '兰考县','河南省开封市兰考县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410223', NULL, 'XZQH', '尉氏县','河南省开封市尉氏县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451481', NULL, 'XZQH', '凭祥市','广西壮族自治区崇左市凭祥市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654323', NULL, 'XZQH', '福海县','新疆维吾尔自治区阿勒泰地区福海县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654322', NULL, 'XZQH', '富蕴县','新疆维吾尔自治区阿勒泰地区富蕴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654321', NULL, 'XZQH', '布尔津县','新疆维吾尔自治区阿勒泰地区布尔津县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510112', NULL, 'XZQH', '龙泉驿区','四川省成都市龙泉驿区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510113', NULL, 'XZQH', '青白江区','四川省成都市青白江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510106', NULL, 'XZQH', '金牛区','四川省成都市金牛区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510105', NULL, 'XZQH', '青羊区','四川省成都市青羊区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510104', NULL, 'XZQH', '锦江区','四川省成都市锦江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510108', NULL, 'XZQH', '成华区','四川省成都市成华区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510107', NULL, 'XZQH', '武侯区','四川省成都市武侯区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421002', NULL, 'XZQH', '沙市区','湖北省荆州市沙市区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421000', NULL, 'XZQH', '荆州市','湖北省荆州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640100', NULL, 'XZQH', '银川市','宁夏回族自治区银川市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654324', NULL, 'XZQH', '哈巴河县','新疆维吾尔自治区阿勒泰地区哈巴河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410204', NULL, 'XZQH', '鼓楼区','河南省开封市鼓楼区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654325', NULL, 'XZQH', '青河县','新疆维吾尔自治区阿勒泰地区青河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410203', NULL, 'XZQH', '顺河回族区','河南省开封市顺河回族区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654326', NULL, 'XZQH', '吉木乃县','新疆维吾尔自治区阿勒泰地区吉木乃县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410202', NULL, 'XZQH', '龙亭区','河南省开封市龙亭区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410205', NULL, 'XZQH', '禹王台区','河南省开封市禹王台区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370687', NULL, 'XZQH', '海阳市','山东省烟台市海阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370686', NULL, 'XZQH', '栖霞市','山东省烟台市栖霞市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370685', NULL, 'XZQH', '招远市','山东省烟台市招远市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370684', NULL, 'XZQH', '蓬莱市','山东省烟台市蓬莱市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370683', NULL, 'XZQH', '莱州市','山东省烟台市莱州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370682', NULL, 'XZQH', '莱阳市','山东省烟台市莱阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370681', NULL, 'XZQH', '龙口市','山东省烟台市龙口市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410221', NULL, 'XZQH', '杞县','河南省开封市杞县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410222', NULL, 'XZQH', '通许县','河南省开封市通许县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410212', NULL, 'XZQH', '祥符区','河南省开封市祥符区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340104', NULL, 'XZQH', '蜀山区','安徽省合肥市蜀山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340102', NULL, 'XZQH', '瑶海区','安徽省合肥市瑶海区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141102', NULL, 'XZQH', '离石区','山西省吕梁市离石区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340103', NULL, 'XZQH', '庐阳区','安徽省合肥市庐阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340100', NULL, 'XZQH', '合肥市','安徽省合肥市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141100', NULL, 'XZQH', '吕梁市','山西省吕梁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211381', NULL, 'XZQH', '北票市','辽宁省朝阳市北票市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211382', NULL, 'XZQH', '凌源市','辽宁省朝阳市凌源市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130283', NULL, 'XZQH', '迁安市','河北省唐山市迁安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130281', NULL, 'XZQH', '遵化市','河北省唐山市遵化市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441900', NULL, 'XZQH', '东莞市','广东省东莞市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340111', NULL, 'XZQH', '包河区','安徽省合肥市包河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('622901', NULL, 'XZQH', '临夏市','甘肃省临夏回族自治州临夏市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('622900', NULL, 'XZQH', '临夏回族自治州','甘肃省临夏回族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451424', NULL, 'XZQH', '大新县','广西壮族自治区崇左市大新县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451425', NULL, 'XZQH', '天等县','广西壮族自治区崇左市天等县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451422', NULL, 'XZQH', '宁明县','广西壮族自治区崇左市宁明县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451423', NULL, 'XZQH', '龙州县','广西壮族自治区崇左市龙州县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451421', NULL, 'XZQH', '扶绥县','广西壮族自治区崇左市扶绥县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340122', NULL, 'XZQH', '肥东县','安徽省合肥市肥东县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340123', NULL, 'XZQH', '肥西县','安徽省合肥市肥西县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522732', NULL, 'XZQH', '三都水族自治县','贵州省黔南布依族苗族自治州三都水族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340121', NULL, 'XZQH', '长丰县','安徽省合肥市长丰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522731', NULL, 'XZQH', '惠水县','贵州省黔南布依族苗族自治州惠水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340124', NULL, 'XZQH', '庐江县','安徽省合肥市庐江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('622921', NULL, 'XZQH', '临夏县','甘肃省临夏回族自治州临夏县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522725', NULL, 'XZQH', '瓮安县','贵州省黔南布依族苗族自治州瓮安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('622922', NULL, 'XZQH', '康乐县','甘肃省临夏回族自治州康乐县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522726', NULL, 'XZQH', '独山县','贵州省黔南布依族苗族自治州独山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522727', NULL, 'XZQH', '平塘县','贵州省黔南布依族苗族自治州平塘县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530724', NULL, 'XZQH', '宁蒗彝族自治县','云南省丽江市宁蒗彝族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('622925', NULL, 'XZQH', '和政县','甘肃省临夏回族自治州和政县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530723', NULL, 'XZQH', '华坪县','云南省丽江市华坪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('622926', NULL, 'XZQH', '东乡族自治县','甘肃省临夏回族自治州东乡族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522722', NULL, 'XZQH', '荔波县','贵州省黔南布依族苗族自治州荔波县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530722', NULL, 'XZQH', '永胜县','云南省丽江市永胜县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('622923', NULL, 'XZQH', '永靖县','甘肃省临夏回族自治州永靖县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522723', NULL, 'XZQH', '贵定县','贵州省黔南布依族苗族自治州贵定县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530721', NULL, 'XZQH', '玉龙纳西族自治县','云南省丽江市玉龙纳西族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('622924', NULL, 'XZQH', '广河县','甘肃省临夏回族自治州广河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('622927', NULL, 'XZQH', '积石山保安族东乡族撒拉族自治县','甘肃省临夏回族自治州积石山保安族东乡族撒拉族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522728', NULL, 'XZQH', '罗甸县','贵州省黔南布依族苗族自治州罗甸县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522729', NULL, 'XZQH', '长顺县','贵州省黔南布依族苗族自治州长顺县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440404', NULL, 'XZQH', '金湾区','广东省珠海市金湾区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451402', NULL, 'XZQH', '江州区','广西壮族自治区崇左市江州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451400', NULL, 'XZQH', '崇左市','广西壮族自治区崇左市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450900', NULL, 'XZQH', '玉林市','广西壮族自治区玉林市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440400', NULL, 'XZQH', '珠海市','广东省珠海市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440403', NULL, 'XZQH', '斗门区','广东省珠海市斗门区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522730', NULL, 'XZQH', '龙里县','贵州省黔南布依族苗族自治州龙里县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440402', NULL, 'XZQH', '香洲区','广东省珠海市香洲区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450902', NULL, 'XZQH', '玉州区','广西壮族自治区玉林市玉州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450903', NULL, 'XZQH', '福绵区','广西壮族自治区玉林市福绵区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341700', NULL, 'XZQH', '池州市','安徽省池州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331181', NULL, 'XZQH', '龙泉市','浙江省丽水市龙泉市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341702', NULL, 'XZQH', '贵池区','安徽省池州市贵池区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522700', NULL, 'XZQH', '黔南布依族苗族自治州','贵州省黔南布依族苗族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522701', NULL, 'XZQH', '都匀市','贵州省黔南布依族苗族自治州都匀市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522702', NULL, 'XZQH', '福泉市','贵州省黔南布依族苗族自治州福泉市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410181', NULL, 'XZQH', '巩义市','河南省郑州市巩义市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410184', NULL, 'XZQH', '新郑市','河南省郑州市新郑市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410185', NULL, 'XZQH', '登封市','河南省郑州市登封市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410182', NULL, 'XZQH', '荥阳市','河南省郑州市荥阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410183', NULL, 'XZQH', '新密市','河南省郑州市新密市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341721', NULL, 'XZQH', '东至县','安徽省池州市东至县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341722', NULL, 'XZQH', '石台县','安徽省池州市石台县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341723', NULL, 'XZQH', '青阳县','安徽省池州市青阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210624', NULL, 'XZQH', '宽甸满族自治县','辽宁省丹东市宽甸满族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211303', NULL, 'XZQH', '龙城区','辽宁省朝阳市龙城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211300', NULL, 'XZQH', '朝阳市','辽宁省朝阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211302', NULL, 'XZQH', '双塔区','辽宁省朝阳市双塔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331123', NULL, 'XZQH', '遂昌县','浙江省丽水市遂昌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331122', NULL, 'XZQH', '缙云县','浙江省丽水市缙云县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331125', NULL, 'XZQH', '云和县','浙江省丽水市云和县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331124', NULL, 'XZQH', '松阳县','浙江省丽水市松阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350681', NULL, 'XZQH', '龙海市','福建省漳州市龙海市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331127', NULL, 'XZQH', '景宁畲族自治县','浙江省丽水市景宁畲族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331126', NULL, 'XZQH', '庆元县','浙江省丽水市庆元县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610300', NULL, 'XZQH', '宝鸡市','陕西省宝鸡市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140502', NULL, 'XZQH', '城区','山西省晋城市城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130202', NULL, 'XZQH', '路南区','河北省唐山市路南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130203', NULL, 'XZQH', '路北区','河北省唐山市路北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140500', NULL, 'XZQH', '晋城市','山西省晋城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130200', NULL, 'XZQH', '唐山市','河北省唐山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220182', NULL, 'XZQH', '榆树市','吉林省长春市榆树市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220183', NULL, 'XZQH', '德惠市','吉林省长春市德惠市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431081', NULL, 'XZQH', '资兴市','湖南省郴州市资兴市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130208', NULL, 'XZQH', '丰润区','河北省唐山市丰润区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130209', NULL, 'XZQH', '曹妃甸区','河北省唐山市曹妃甸区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130207', NULL, 'XZQH', '丰南区','河北省唐山市丰南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130204', NULL, 'XZQH', '古冶区','河北省唐山市古冶区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130205', NULL, 'XZQH', '开平区','河北省唐山市开平区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511781', NULL, 'XZQH', '万源市','四川省达州市万源市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150125', NULL, 'XZQH', '武川县','内蒙古自治区呼和浩特市武川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150121', NULL, 'XZQH', '土默特左旗','内蒙古自治区呼和浩特市土默特左旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620724', NULL, 'XZQH', '高台县','甘肃省张掖市高台县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150122', NULL, 'XZQH', '托克托县','内蒙古自治区呼和浩特市托克托县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620723', NULL, 'XZQH', '临泽县','甘肃省张掖市临泽县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150123', NULL, 'XZQH', '和林格尔县','内蒙古自治区呼和浩特市和林格尔县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150124', NULL, 'XZQH', '清水河县','内蒙古自治区呼和浩特市清水河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620725', NULL, 'XZQH', '山丹县','甘肃省张掖市山丹县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620722', NULL, 'XZQH', '民乐县','甘肃省张掖市民乐县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620721', NULL, 'XZQH', '肃南裕固族自治县','甘肃省张掖市肃南裕固族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230811', NULL, 'XZQH', '郊区','黑龙江省佳木斯市郊区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141181', NULL, 'XZQH', '孝义市','山西省吕梁市孝义市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141182', NULL, 'XZQH', '汾阳市','山西省吕梁市汾阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330402', NULL, 'XZQH', '南湖区','浙江省嘉兴市南湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330400', NULL, 'XZQH', '嘉兴市','浙江省嘉兴市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331100', NULL, 'XZQH', '丽水市','浙江省丽水市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331102', NULL, 'XZQH', '莲都区','浙江省丽水市莲都区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632726', NULL, 'XZQH', '曲麻莱县','青海省玉树藏族自治州曲麻莱县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632725', NULL, 'XZQH', '囊谦县','青海省玉树藏族自治州囊谦县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632724', NULL, 'XZQH', '治多县','青海省玉树藏族自治州治多县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130224', NULL, 'XZQH', '滦南县','河北省唐山市滦南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632723', NULL, 'XZQH', '称多县','青海省玉树藏族自治州称多县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130225', NULL, 'XZQH', '乐亭县','河北省唐山市乐亭县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632722', NULL, 'XZQH', '杂多县','青海省玉树藏族自治州杂多县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130223', NULL, 'XZQH', '滦县','河北省唐山市滦县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532930', NULL, 'XZQH', '洱源县','云南省大理白族自治州洱源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130229', NULL, 'XZQH', '玉田县','河北省唐山市玉田县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532931', NULL, 'XZQH', '剑川县','云南省大理白族自治州剑川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130227', NULL, 'XZQH', '迁西县','河北省唐山市迁西县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532932', NULL, 'XZQH', '鹤庆县','云南省大理白族自治州鹤庆县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230800', NULL, 'XZQH', '佳木斯市','黑龙江省佳木斯市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230803', NULL, 'XZQH', '向阳区','黑龙江省佳木斯市向阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230804', NULL, 'XZQH', '前进区','黑龙江省佳木斯市前进区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230805', NULL, 'XZQH', '东风区','黑龙江省佳木斯市东风区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231121', NULL, 'XZQH', '嫩江县','黑龙江省黑河市嫩江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231124', NULL, 'XZQH', '孙吴县','黑龙江省黑河市孙吴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231123', NULL, 'XZQH', '逊克县','黑龙江省黑河市逊克县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331121', NULL, 'XZQH', '青田县','浙江省丽水市青田县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620700', NULL, 'XZQH', '张掖市','甘肃省张掖市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620702', NULL, 'XZQH', '甘州区','甘肃省张掖市甘州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460000', NULL, 'XZQH', '海南省','海南省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330421', NULL, 'XZQH', '嘉善县','浙江省嘉兴市嘉善县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360321', NULL, 'XZQH', '莲花县','江西省萍乡市莲花县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330424', NULL, 'XZQH', '海盐县','浙江省嘉兴市海盐县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360323', NULL, 'XZQH', '芦溪县','江西省萍乡市芦溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360322', NULL, 'XZQH', '上栗县','江西省萍乡市上栗县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632700', NULL, 'XZQH', '玉树藏族自治州','青海省玉树藏族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632701', NULL, 'XZQH', '玉树市','青海省玉树藏族自治州玉树市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231182', NULL, 'XZQH', '五大连池市','黑龙江省黑河市五大连池市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231181', NULL, 'XZQH', '北安市','黑龙江省黑河市北安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371700', NULL, 'XZQH', '菏泽市','山东省菏泽市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220122', NULL, 'XZQH', '农安县','吉林省长春市农安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610330', NULL, 'XZQH', '凤县','陕西省宝鸡市凤县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371702', NULL, 'XZQH', '牡丹区','山东省菏泽市牡丹区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610331', NULL, 'XZQH', '太白县','陕西省宝鸡市太白县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371703', NULL, 'XZQH', '定陶区','山东省菏泽市定陶区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330411', NULL, 'XZQH', '秀洲区','浙江省嘉兴市秀洲区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360313', NULL, 'XZQH', '湘东区','江西省萍乡市湘东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650000', NULL, 'XZQH', '新疆维吾尔自治区','新疆维吾尔自治区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532922', NULL, 'XZQH', '漾濞彝族自治县','云南省大理白族自治州漾濞彝族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532924', NULL, 'XZQH', '宾川县','云南省大理白族自治州宾川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532923', NULL, 'XZQH', '祥云县','云南省大理白族自治州祥云县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532929', NULL, 'XZQH', '云龙县','云南省大理白族自治州云龙县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610324', NULL, 'XZQH', '扶风县','陕西省宝鸡市扶风县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610327', NULL, 'XZQH', '陇县','陕西省宝鸡市陇县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610326', NULL, 'XZQH', '眉县','陕西省宝鸡市眉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532926', NULL, 'XZQH', '南涧彝族自治县','云南省大理白族自治州南涧彝族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610329', NULL, 'XZQH', '麟游县','陕西省宝鸡市麟游县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532925', NULL, 'XZQH', '弥渡县','云南省大理白族自治州弥渡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610328', NULL, 'XZQH', '千阳县','陕西省宝鸡市千阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532928', NULL, 'XZQH', '永平县','云南省大理白族自治州永平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532927', NULL, 'XZQH', '巍山彝族回族自治县','云南省大理白族自治州巍山彝族回族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('120000', NULL, 'XZQH', '天津市','天津市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610322', NULL, 'XZQH', '凤翔县','陕西省宝鸡市凤翔县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610323', NULL, 'XZQH', '岐山县','陕西省宝鸡市岐山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210604', NULL, 'XZQH', '振安区','辽宁省丹东市振安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360300', NULL, 'XZQH', '萍乡市','江西省萍乡市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210603', NULL, 'XZQH', '振兴区','辽宁省丹东市振兴区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220112', NULL, 'XZQH', '双阳区','吉林省长春市双阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220113', NULL, 'XZQH', '九台区','吉林省长春市九台区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350622', NULL, 'XZQH', '云霄县','福建省漳州市云霄县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211321', NULL, 'XZQH', '朝阳县','辽宁省朝阳市朝阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350623', NULL, 'XZQH', '漳浦县','福建省漳州市漳浦县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211322', NULL, 'XZQH', '建平县','辽宁省朝阳市建平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350624', NULL, 'XZQH', '诏安县','福建省漳州市诏安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350625', NULL, 'XZQH', '长泰县','福建省漳州市长泰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211324', NULL, 'XZQH', '喀喇沁左翼蒙古族自治县','辽宁省朝阳市喀喇沁左翼蒙古族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350626', NULL, 'XZQH', '东山县','福建省漳州市东山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210602', NULL, 'XZQH', '元宝区','辽宁省丹东市元宝区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360302', NULL, 'XZQH', '安源区','江西省萍乡市安源区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350627', NULL, 'XZQH', '南靖县','福建省漳州市南靖县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350628', NULL, 'XZQH', '平和县','福建省漳州市平和县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210600', NULL, 'XZQH', '丹东市','辽宁省丹东市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350629', NULL, 'XZQH', '华安县','福建省漳州市华安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140521', NULL, 'XZQH', '沁水县','山西省晋城市沁水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341200', NULL, 'XZQH', '阜阳市','安徽省阜阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141130', NULL, 'XZQH', '交口县','山西省吕梁市交口县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341202', NULL, 'XZQH', '颍州区','安徽省阜阳市颍州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341203', NULL, 'XZQH', '颍东区','安徽省阜阳市颍东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341204', NULL, 'XZQH', '颍泉区','安徽省阜阳市颍泉区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710100', NULL, 'XZQH', '台北市','台湾省台北市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710103', NULL, 'XZQH', '中山区','台湾省台北市中山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140525', NULL, 'XZQH', '泽州县','山西省晋城市泽州县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710104', NULL, 'XZQH', '万华区','台湾省台北市万华区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140524', NULL, 'XZQH', '陵川县','山西省晋城市陵川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710101', NULL, 'XZQH', '中正区','台湾省台北市中正区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710102', NULL, 'XZQH', '大同区','台湾省台北市大同区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140522', NULL, 'XZQH', '阳城县','山西省晋城市阳城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710106', NULL, 'XZQH', '松山区','台湾省台北市松山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371724', NULL, 'XZQH', '巨野县','山东省菏泽市巨野县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220105', NULL, 'XZQH', '二道区','吉林省长春市二道区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710105', NULL, 'XZQH', '信义区','台湾省台北市信义区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371725', NULL, 'XZQH', '郓城县','山东省菏泽市郓城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220106', NULL, 'XZQH', '绿园区','吉林省长春市绿园区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710108', NULL, 'XZQH', '南港区','台湾省台北市南港区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371726', NULL, 'XZQH', '鄄城县','山东省菏泽市鄄城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220103', NULL, 'XZQH', '宽城区','吉林省长春市宽城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710107', NULL, 'XZQH', '大安区','台湾省台北市大安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220104', NULL, 'XZQH', '朝阳区','吉林省长春市朝阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710109', NULL, 'XZQH', '北投区','台湾省台北市北投区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371721', NULL, 'XZQH', '曹县','山东省菏泽市曹县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220102', NULL, 'XZQH', '南关区','吉林省长春市南关区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371722', NULL, 'XZQH', '单县','山东省菏泽市单县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500000', NULL, 'XZQH', '重庆市','重庆市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371723', NULL, 'XZQH', '成武县','山东省菏泽市成武县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220100', NULL, 'XZQH', '长春市','吉林省长春市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141121', NULL, 'XZQH', '文水县','山西省吕梁市文水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141122', NULL, 'XZQH', '交城县','山西省吕梁市交城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141123', NULL, 'XZQH', '兴县','山西省吕梁市兴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532900', NULL, 'XZQH', '大理白族自治州','云南省大理白族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141124', NULL, 'XZQH', '临县','山西省吕梁市临县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141125', NULL, 'XZQH', '柳林县','山西省吕梁市柳林县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141126', NULL, 'XZQH', '石楼县','山西省吕梁市石楼县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532901', NULL, 'XZQH', '大理市','云南省大理白族自治州大理市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141127', NULL, 'XZQH', '岚县','山西省吕梁市岚县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141128', NULL, 'XZQH', '方山县','山西省吕梁市方山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710110', NULL, 'XZQH', '内湖区','台湾省台北市内湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141129', NULL, 'XZQH', '中阳县','山西省吕梁市中阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710111', NULL, 'XZQH', '士林区','台湾省台北市士林区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710112', NULL, 'XZQH', '文山区','台湾省台北市文山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610303', NULL, 'XZQH', '金台区','陕西省宝鸡市金台区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371728', NULL, 'XZQH', '东明县','山东省菏泽市东明县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610302', NULL, 'XZQH', '渭滨区','陕西省宝鸡市渭滨区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610304', NULL, 'XZQH', '陈仓区','陕西省宝鸡市陈仓区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371322', NULL, 'XZQH', '郯城县','山东省临沂市郯城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371323', NULL, 'XZQH', '沂水县','山东省临沂市沂水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371321', NULL, 'XZQH', '沂南县','山东省临沂市沂南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371326', NULL, 'XZQH', '平邑县','山东省临沂市平邑县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371327', NULL, 'XZQH', '莒南县','山东省临沂市莒南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371324', NULL, 'XZQH', '兰陵县','山东省临沂市兰陵县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371325', NULL, 'XZQH', '费县','山东省临沂市费县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632300', NULL, 'XZQH', '黄南藏族自治州','青海省黄南藏族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371328', NULL, 'XZQH', '蒙阴县','山东省临沂市蒙阴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371329', NULL, 'XZQH', '临沭县','山东省临沂市临沭县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810101', NULL, 'XZQH', '中西区','香港特别行政区null中西区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810102', NULL, 'XZQH', '湾仔区','香港特别行政区null湾仔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810103', NULL, 'XZQH', '东区','香港特别行政区null东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810104', NULL, 'XZQH', '南区','香港特别行政区null南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810105', NULL, 'XZQH', '油尖旺区','香港特别行政区null油尖旺区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370724', NULL, 'XZQH', '临朐县','山东省潍坊市临朐县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370725', NULL, 'XZQH', '昌乐县','山东省潍坊市昌乐县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411628', NULL, 'XZQH', '鹿邑县','河南省周口市鹿邑县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411621', NULL, 'XZQH', '扶沟县','河南省周口市扶沟县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411622', NULL, 'XZQH', '西华县','河南省周口市西华县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411623', NULL, 'XZQH', '商水县','河南省周口市商水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411624', NULL, 'XZQH', '沈丘县','河南省周口市沈丘县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411625', NULL, 'XZQH', '郸城县','河南省周口市郸城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411626', NULL, 'XZQH', '淮阳县','河南省周口市淮阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411627', NULL, 'XZQH', '太康县','河南省周口市太康县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810115', NULL, 'XZQH', '屯门区','香港特别行政区null屯门区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371300', NULL, 'XZQH', '临沂市','山东省临沂市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810116', NULL, 'XZQH', '元朗区','香港特别行政区null元朗区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810113', NULL, 'XZQH', '西贡区','香港特别行政区null西贡区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230822', NULL, 'XZQH', '桦南县','黑龙江省佳木斯市桦南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810114', NULL, 'XZQH', '荃湾区','香港特别行政区null荃湾区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810111', NULL, 'XZQH', '大埔区','香港特别行政区null大埔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230828', NULL, 'XZQH', '汤原县','黑龙江省佳木斯市汤原县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810112', NULL, 'XZQH', '沙田区','香港特别行政区null沙田区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230826', NULL, 'XZQH', '桦川县','黑龙江省佳木斯市桦川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810110', NULL, 'XZQH', '北区','香港特别行政区null北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632323', NULL, 'XZQH', '泽库县','青海省黄南藏族自治州泽库县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632322', NULL, 'XZQH', '尖扎县','青海省黄南藏族自治州尖扎县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632321', NULL, 'XZQH', '同仁县','青海省黄南藏族自治州同仁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371302', NULL, 'XZQH', '兰山区','山东省临沂市兰山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410506', NULL, 'XZQH', '龙安区','河南省安阳市龙安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410503', NULL, 'XZQH', '北关区','河南省安阳市北关区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410502', NULL, 'XZQH', '文峰区','河南省安阳市文峰区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410505', NULL, 'XZQH', '殷都区','河南省安阳市殷都区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810109', NULL, 'XZQH', '观塘区','香港特别行政区null观塘区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810108', NULL, 'XZQH', '黄大仙区','香港特别行政区null黄大仙区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810107', NULL, 'XZQH', '九龙城区','香港特别行政区null九龙城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810106', NULL, 'XZQH', '深水埗区','香港特别行政区null深水埗区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410500', NULL, 'XZQH', '安阳市','河南省安阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371311', NULL, 'XZQH', '罗庄区','山东省临沂市罗庄区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371312', NULL, 'XZQH', '河东区','山东省临沂市河东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511083', NULL, 'XZQH', '隆昌市','四川省内江市隆昌市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370700', NULL, 'XZQH', '潍坊市','山东省潍坊市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370702', NULL, 'XZQH', '潍城区','山东省潍坊市潍城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370704', NULL, 'XZQH', '坊子区','山东省潍坊市坊子区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370703', NULL, 'XZQH', '寒亭区','山东省潍坊市寒亭区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370705', NULL, 'XZQH', '奎文区','山东省潍坊市奎文区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361181', NULL, 'XZQH', '德兴市','江西省上饶市德兴市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810118', NULL, 'XZQH', '离岛区','香港特别行政区null离岛区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810117', NULL, 'XZQH', '葵青区','香港特别行政区null葵青区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361102', NULL, 'XZQH', '信州区','江西省上饶市信州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230883', NULL, 'XZQH', '抚远市','黑龙江省佳木斯市抚远市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450000', NULL, 'XZQH', '广西壮族自治区','广西壮族自治区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361103', NULL, 'XZQH', '广丰区','江西省上饶市广丰区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('630200', NULL, 'XZQH', '海东市','青海省海东市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230881', NULL, 'XZQH', '同江市','黑龙江省佳木斯市同江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230882', NULL, 'XZQH', '富锦市','黑龙江省佳木斯市富锦市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511011', NULL, 'XZQH', '东兴区','四川省内江市东兴区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361100', NULL, 'XZQH', '上饶市','江西省上饶市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653122', NULL, 'XZQH', '疏勒县','新疆维吾尔自治区喀什地区疏勒县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653121', NULL, 'XZQH', '疏附县','新疆维吾尔自治区喀什地区疏附县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653124', NULL, 'XZQH', '泽普县','新疆维吾尔自治区喀什地区泽普县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653123', NULL, 'XZQH', '英吉沙县','新疆维吾尔自治区喀什地区英吉沙县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150300', NULL, 'XZQH', '乌海市','内蒙古自治区乌海市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140311', NULL, 'XZQH', '郊区','山西省阳泉市郊区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('445122', NULL, 'XZQH', '饶平县','广东省潮州市饶平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620302', NULL, 'XZQH', '金川区','甘肃省金昌市金川区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150303', NULL, 'XZQH', '海南区','内蒙古自治区乌海市海南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150304', NULL, 'XZQH', '乌达区','内蒙古自治区乌海市乌达区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150302', NULL, 'XZQH', '海勃湾区','内蒙古自治区乌海市海勃湾区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511024', NULL, 'XZQH', '威远县','四川省内江市威远县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511025', NULL, 'XZQH', '资中县','四川省内江市资中县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653131', NULL, 'XZQH', '塔什库尔干塔吉克自治县','新疆维吾尔自治区喀什地区塔什库尔干塔吉克自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360502', NULL, 'XZQH', '渝水区','江西省新余市渝水区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653130', NULL, 'XZQH', '巴楚县','新疆维吾尔自治区喀什地区巴楚县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360500', NULL, 'XZQH', '新余市','江西省新余市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620321', NULL, 'XZQH', '永昌县','甘肃省金昌市永昌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653127', NULL, 'XZQH', '麦盖提县','新疆维吾尔自治区喀什地区麦盖提县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653128', NULL, 'XZQH', '岳普湖县','新疆维吾尔自治区喀什地区岳普湖县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653125', NULL, 'XZQH', '莎车县','新疆维吾尔自治区喀什地区莎车县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653126', NULL, 'XZQH', '叶城县','新疆维吾尔自治区喀什地区叶城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140321', NULL, 'XZQH', '平定县','山西省阳泉市平定县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653129', NULL, 'XZQH', '伽师县','新疆维吾尔自治区喀什地区伽师县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140322', NULL, 'XZQH', '盂县','山西省阳泉市盂县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632324', NULL, 'XZQH', '河南蒙古族自治县','青海省黄南藏族自治州河南蒙古族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220203', NULL, 'XZQH', '龙潭区','吉林省吉林市龙潭区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220202', NULL, 'XZQH', '昌邑区','吉林省吉林市昌邑区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361121', NULL, 'XZQH', '上饶县','江西省上饶市上饶县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220204', NULL, 'XZQH', '船营区','吉林省吉林市船营区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361124', NULL, 'XZQH', '铅山县','江西省上饶市铅山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361123', NULL, 'XZQH', '玉山县','江西省上饶市玉山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361126', NULL, 'XZQH', '弋阳县','江西省上饶市弋阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361125', NULL, 'XZQH', '横峰县','江西省上饶市横峰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220200', NULL, 'XZQH', '吉林市','吉林省吉林市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('630222', NULL, 'XZQH', '民和回族土族自治县','青海省海东市民和回族土族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653100', NULL, 'XZQH', '喀什地区','新疆维吾尔自治区喀什地区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361128', NULL, 'XZQH', '鄱阳县','江西省上饶市鄱阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361127', NULL, 'XZQH', '余干县','江西省上饶市余干县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653101', NULL, 'XZQH', '喀什市','新疆维吾尔自治区喀什地区喀什市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361129', NULL, 'XZQH', '万年县','江西省上饶市万年县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411681', NULL, 'XZQH', '项城市','河南省周口市项城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520100', NULL, 'XZQH', '贵阳市','贵州省贵阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('445100', NULL, 'XZQH', '潮州市','广东省潮州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('445102', NULL, 'XZQH', '湘桥区','广东省潮州市湘桥区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('445103', NULL, 'XZQH', '潮安区','广东省潮州市潮安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360521', NULL, 'XZQH', '分宜县','江西省新余市分宜县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361130', NULL, 'XZQH', '婺源县','江西省上饶市婺源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220211', NULL, 'XZQH', '丰满区','吉林省吉林市丰满区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350603', NULL, 'XZQH', '龙文区','福建省漳州市龙文区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350602', NULL, 'XZQH', '芗城区','福建省漳州市芗城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350600', NULL, 'XZQH', '漳州市','福建省漳州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520111', NULL, 'XZQH', '花溪区','贵州省贵阳市花溪区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520112', NULL, 'XZQH', '乌当区','贵州省贵阳市乌当区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520103', NULL, 'XZQH', '云岩区','贵州省贵阳市云岩区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520102', NULL, 'XZQH', '南明区','贵州省贵阳市南明区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('630203', NULL, 'XZQH', '平安区','青海省海东市平安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511921', NULL, 'XZQH', '通江县','四川省巴中市通江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('630202', NULL, 'XZQH', '乐都区','青海省海东市乐都区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511923', NULL, 'XZQH', '平昌县','四川省巴中市平昌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511922', NULL, 'XZQH', '南江县','四川省巴中市南江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320481', NULL, 'XZQH', '溧阳市','江苏省常州市溧阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640221', NULL, 'XZQH', '平罗县','宁夏回族自治区石嘴山市平罗县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522301', NULL, 'XZQH', '兴义市','贵州省黔西南布依族苗族自治州兴义市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522300', NULL, 'XZQH', '黔西南布依族苗族自治州','贵州省黔西南布依族苗族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430600', NULL, 'XZQH', '岳阳市','湖南省岳阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430602', NULL, 'XZQH', '岳阳楼区','湖南省岳阳市岳阳楼区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430603', NULL, 'XZQH', '云溪区','湖南省岳阳市云溪区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421102', NULL, 'XZQH', '黄州区','湖北省黄冈市黄州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152502', NULL, 'XZQH', '锡林浩特市','内蒙古自治区锡林郭勒盟锡林浩特市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511902', NULL, 'XZQH', '巴州区','四川省巴中市巴州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152501', NULL, 'XZQH', '二连浩特市','内蒙古自治区锡林郭勒盟二连浩特市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421100', NULL, 'XZQH', '黄冈市','湖北省黄冈市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511903', NULL, 'XZQH', '恩阳区','四川省巴中市恩阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511900', NULL, 'XZQH', '巴中市','四川省巴中市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640205', NULL, 'XZQH', '惠农区','宁夏回族自治区石嘴山市惠农区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640200', NULL, 'XZQH', '石嘴山市','宁夏回族自治区石嘴山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152500', NULL, 'XZQH', '锡林郭勒盟','内蒙古自治区锡林郭勒盟', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640202', NULL, 'XZQH', '大武口区','宁夏回族自治区石嘴山市大武口区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450722', NULL, 'XZQH', '浦北县','广西壮族自治区钦州市浦北县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410581', NULL, 'XZQH', '林州市','河南省安阳市林州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654224', NULL, 'XZQH', '托里县','新疆维吾尔自治区塔城地区托里县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654223', NULL, 'XZQH', '沙湾县','新疆维吾尔自治区塔城地区沙湾县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430611', NULL, 'XZQH', '君山区','湖南省岳阳市君山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654221', NULL, 'XZQH', '额敏县','新疆维吾尔自治区塔城地区额敏县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451100', NULL, 'XZQH', '贺州市','广西壮族自治区贺州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451102', NULL, 'XZQH', '八步区','广西壮族自治区贺州市八步区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451103', NULL, 'XZQH', '平桂区','广西壮族自治区贺州市平桂区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450721', NULL, 'XZQH', '灵山县','广西壮族自治区钦州市灵山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430621', NULL, 'XZQH', '岳阳县','湖南省岳阳市岳阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430624', NULL, 'XZQH', '湘阴县','湖南省岳阳市湘阴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430623', NULL, 'XZQH', '华容县','湖南省岳阳市华容县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522323', NULL, 'XZQH', '普安县','贵州省黔西南布依族苗族自治州普安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522322', NULL, 'XZQH', '兴仁县','贵州省黔西南布依族苗族自治州兴仁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421124', NULL, 'XZQH', '英山县','湖北省黄冈市英山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421125', NULL, 'XZQH', '浠水县','湖北省黄冈市浠水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421126', NULL, 'XZQH', '蕲春县','湖北省黄冈市蕲春县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421127', NULL, 'XZQH', '黄梅县','湖北省黄冈市黄梅县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430626', NULL, 'XZQH', '平江县','湖南省岳阳市平江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152528', NULL, 'XZQH', '镶黄旗','内蒙古自治区锡林郭勒盟镶黄旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152527', NULL, 'XZQH', '太仆寺旗','内蒙古自治区锡林郭勒盟太仆寺旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152529', NULL, 'XZQH', '正镶白旗','内蒙古自治区锡林郭勒盟正镶白旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152524', NULL, 'XZQH', '苏尼特右旗','内蒙古自治区锡林郭勒盟苏尼特右旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421123', NULL, 'XZQH', '罗田县','湖北省黄冈市罗田县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152523', NULL, 'XZQH', '苏尼特左旗','内蒙古自治区锡林郭勒盟苏尼特左旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421122', NULL, 'XZQH', '红安县','湖北省黄冈市红安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152526', NULL, 'XZQH', '西乌珠穆沁旗','内蒙古自治区锡林郭勒盟西乌珠穆沁旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421121', NULL, 'XZQH', '团风县','湖北省黄冈市团风县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152525', NULL, 'XZQH', '东乌珠穆沁旗','内蒙古自治区锡林郭勒盟东乌珠穆沁旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522326', NULL, 'XZQH', '望谟县','贵州省黔西南布依族苗族自治州望谟县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522327', NULL, 'XZQH', '册亨县','贵州省黔西南布依族苗族自治州册亨县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152522', NULL, 'XZQH', '阿巴嘎旗','内蒙古自治区锡林郭勒盟阿巴嘎旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522324', NULL, 'XZQH', '晴隆县','贵州省黔西南布依族苗族自治州晴隆县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522325', NULL, 'XZQH', '贞丰县','贵州省黔西南布依族苗族自治州贞丰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('522328', NULL, 'XZQH', '安龙县','贵州省黔西南布依族苗族自治州安龙县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654202', NULL, 'XZQH', '乌苏市','新疆维吾尔自治区塔城地区乌苏市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654201', NULL, 'XZQH', '塔城市','新疆维吾尔自治区塔城地区塔城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654200', NULL, 'XZQH', '塔城地区','新疆维吾尔自治区塔城地区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370784', NULL, 'XZQH', '安丘市','山东省潍坊市安丘市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370783', NULL, 'XZQH', '寿光市','山东省潍坊市寿光市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370782', NULL, 'XZQH', '诸城市','山东省潍坊市诸城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370781', NULL, 'XZQH', '青州市','山东省潍坊市青州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511000', NULL, 'XZQH', '内江市','四川省内江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370786', NULL, 'XZQH', '昌邑市','山东省潍坊市昌邑市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370785', NULL, 'XZQH', '高密市','山东省潍坊市高密市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451122', NULL, 'XZQH', '钟山县','广西壮族自治区贺州市钟山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451121', NULL, 'XZQH', '昭平县','广西壮族自治区贺州市昭平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511002', NULL, 'XZQH', '市中区','四川省内江市市中区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451123', NULL, 'XZQH', '富川瑶族自治县','广西壮族自治区贺州市富川瑶族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410522', NULL, 'XZQH', '安阳县','河南省安阳市安阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410523', NULL, 'XZQH', '汤阴县','河南省安阳市汤阴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410526', NULL, 'XZQH', '滑县','河南省安阳市滑县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410527', NULL, 'XZQH', '内黄县','河南省安阳市内黄县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510000', NULL, 'XZQH', '四川省','四川省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654226', NULL, 'XZQH', '和布克赛尔蒙古自治县','新疆维吾尔自治区塔城地区和布克赛尔蒙古自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654225', NULL, 'XZQH', '裕民县','新疆维吾尔自治区塔城地区裕民县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130881', NULL, 'XZQH', '平泉市','河北省承德市平泉市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320411', NULL, 'XZQH', '新北区','江苏省常州市新北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320413', NULL, 'XZQH', '金坛区','江苏省常州市金坛区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320412', NULL, 'XZQH', '武进区','江苏省常州市武进区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411602', NULL, 'XZQH', '川汇区','河南省周口市川汇区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411600', NULL, 'XZQH', '周口市','河南省周口市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450700', NULL, 'XZQH', '钦州市','广西壮族自治区钦州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450703', NULL, 'XZQH', '钦北区','广西壮族自治区钦州市钦北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450702', NULL, 'XZQH', '钦南区','广西壮族自治区钦州市钦南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330727', NULL, 'XZQH', '磐安县','浙江省金华市磐安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320400', NULL, 'XZQH', '常州市','江苏省常州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440781', NULL, 'XZQH', '台山市','广东省江门市台山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440784', NULL, 'XZQH', '鹤山市','广东省江门市鹤山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440783', NULL, 'XZQH', '开平市','广东省江门市开平市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330723', NULL, 'XZQH', '武义县','浙江省金华市武义县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320404', NULL, 'XZQH', '钟楼区','江苏省常州市钟楼区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440785', NULL, 'XZQH', '恩平市','广东省江门市恩平市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330726', NULL, 'XZQH', '浦江县','浙江省金华市浦江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320402', NULL, 'XZQH', '天宁区','江苏省常州市天宁区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350583', NULL, 'XZQH', '南安市','福建省泉州市南安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350581', NULL, 'XZQH', '石狮市','福建省泉州市石狮市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350582', NULL, 'XZQH', '晋江市','福建省泉州市晋江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331023', NULL, 'XZQH', '天台县','浙江省台州市天台县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331024', NULL, 'XZQH', '仙居县','浙江省台州市仙居县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640181', NULL, 'XZQH', '灵武市','宁夏回族自治区银川市灵武市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130133', NULL, 'XZQH', '赵县','河北省石家庄市赵县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130132', NULL, 'XZQH', '元氏县','河北省石家庄市元氏县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130131', NULL, 'XZQH', '平山县','河北省石家庄市平山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130130', NULL, 'XZQH', '无极县','河北省石家庄市无极县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532628', NULL, 'XZQH', '富宁县','云南省文山壮族苗族自治州富宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341602', NULL, 'XZQH', '谯城区','安徽省亳州市谯城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130983', NULL, 'XZQH', '黄骅市','河北省沧州市黄骅市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130984', NULL, 'XZQH', '河间市','河北省沧州市河间市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130981', NULL, 'XZQH', '泊头市','河北省沧州市泊头市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130982', NULL, 'XZQH', '任丘市','河北省沧州市任丘市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211281', NULL, 'XZQH', '调兵山市','辽宁省铁岭市调兵山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532623', NULL, 'XZQH', '西畴县','云南省文山壮族苗族自治州西畴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532622', NULL, 'XZQH', '砚山县','云南省文山壮族苗族自治州砚山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341600', NULL, 'XZQH', '亳州市','安徽省亳州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211282', NULL, 'XZQH', '开原市','辽宁省铁岭市开原市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532627', NULL, 'XZQH', '广南县','云南省文山壮族苗族自治州广南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532626', NULL, 'XZQH', '丘北县','云南省文山壮族苗族自治州丘北县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532625', NULL, 'XZQH', '马关县','云南省文山壮族苗族自治州马关县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532624', NULL, 'XZQH', '麻栗坡县','云南省文山壮族苗族自治州麻栗坡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330702', NULL, 'XZQH', '婺城区','浙江省金华市婺城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330703', NULL, 'XZQH', '金东区','浙江省金华市金东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330700', NULL, 'XZQH', '金华市','浙江省金华市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130123', NULL, 'XZQH', '正定县','河北省石家庄市正定县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421087', NULL, 'XZQH', '松滋市','湖北省荆州市松滋市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130126', NULL, 'XZQH', '灵寿县','河北省石家庄市灵寿县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130125', NULL, 'XZQH', '行唐县','河北省石家庄市行唐县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421083', NULL, 'XZQH', '洪湖市','湖北省荆州市洪湖市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130121', NULL, 'XZQH', '井陉县','河北省石家庄市井陉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421081', NULL, 'XZQH', '石首市','湖北省荆州市石首市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130128', NULL, 'XZQH', '深泽县','河北省石家庄市深泽县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130127', NULL, 'XZQH', '高邑县','河北省石家庄市高邑县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130129', NULL, 'XZQH', '赞皇县','河北省石家庄市赞皇县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130111', NULL, 'XZQH', '栾城区','河北省石家庄市栾城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130110', NULL, 'XZQH', '鹿泉区','河北省石家庄市鹿泉区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210922', NULL, 'XZQH', '彰武县','辽宁省阜新市彰武县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210111', NULL, 'XZQH', '苏家屯区','辽宁省沈阳市苏家屯区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210112', NULL, 'XZQH', '浑南区','辽宁省沈阳市浑南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654300', NULL, 'XZQH', '阿勒泰地区','新疆维吾尔自治区阿勒泰地区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210113', NULL, 'XZQH', '沈北新区','辽宁省沈阳市沈北新区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654301', NULL, 'XZQH', '阿勒泰市','新疆维吾尔自治区阿勒泰地区阿勒泰市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210114', NULL, 'XZQH', '于洪区','辽宁省沈阳市于洪区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210115', NULL, 'XZQH', '辽中区','辽宁省沈阳市辽中区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210921', NULL, 'XZQH', '阜新蒙古族自治县','辽宁省阜新市阜新蒙古族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530426', NULL, 'XZQH', '峨山彝族自治县','云南省玉溪市峨山彝族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130930', NULL, 'XZQH', '孟村回族自治县','河北省沧州市孟村回族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530427', NULL, 'XZQH', '新平彝族傣族自治县','云南省玉溪市新平彝族傣族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530424', NULL, 'XZQH', '华宁县','云南省玉溪市华宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530425', NULL, 'XZQH', '易门县','云南省玉溪市易门县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530422', NULL, 'XZQH', '澄江县','云南省玉溪市澄江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530423', NULL, 'XZQH', '通海县','云南省玉溪市通海县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421022', NULL, 'XZQH', '公安县','湖北省荆州市公安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421023', NULL, 'XZQH', '监利县','湖北省荆州市监利县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530428', NULL, 'XZQH', '元江哈尼族彝族傣族自治县','云南省玉溪市元江哈尼族彝族傣族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421024', NULL, 'XZQH', '江陵县','湖北省荆州市江陵县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441802', NULL, 'XZQH', '清城区','广东省清远市清城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441803', NULL, 'XZQH', '清新区','广东省清远市清新区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130184', NULL, 'XZQH', '新乐市','河北省石家庄市新乐市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130183', NULL, 'XZQH', '晋州市','河北省石家庄市晋州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130181', NULL, 'XZQH', '辛集市','河北省石家庄市辛集市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210102', NULL, 'XZQH', '和平区','辽宁省沈阳市和平区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210103', NULL, 'XZQH', '沈河区','辽宁省沈阳市沈河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210100', NULL, 'XZQH', '沈阳市','辽宁省沈阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210106', NULL, 'XZQH', '铁西区','辽宁省沈阳市铁西区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('429021', NULL, 'XZQH', '神农架林区','湖北省null神农架林区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210104', NULL, 'XZQH', '大东区','辽宁省沈阳市大东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441800', NULL, 'XZQH', '清远市','广东省清远市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210105', NULL, 'XZQH', '皇姑区','辽宁省沈阳市皇姑区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331081', NULL, 'XZQH', '温岭市','浙江省台州市温岭市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331082', NULL, 'XZQH', '临海市','浙江省台州市临海市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331083', NULL, 'XZQH', '玉环市','浙江省台州市玉环市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210904', NULL, 'XZQH', '太平区','辽宁省阜新市太平区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210905', NULL, 'XZQH', '清河门区','辽宁省阜新市清河门区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210900', NULL, 'XZQH', '阜新市','辽宁省阜新市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210902', NULL, 'XZQH', '海州区','辽宁省阜新市海州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210903', NULL, 'XZQH', '新邱区','辽宁省阜新市新邱区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530400', NULL, 'XZQH', '玉溪市','云南省玉溪市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430682', NULL, 'XZQH', '临湘市','湖南省岳阳市临湘市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530402', NULL, 'XZQH', '红塔区','云南省玉溪市红塔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430681', NULL, 'XZQH', '汨罗市','湖南省岳阳市汨罗市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530403', NULL, 'XZQH', '江川区','云南省玉溪市江川区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210911', NULL, 'XZQH', '细河区','辽宁省阜新市细河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130929', NULL, 'XZQH', '献县','河北省沧州市献县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210124', NULL, 'XZQH', '法库县','辽宁省沈阳市法库县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130927', NULL, 'XZQH', '南皮县','河北省沧州市南皮县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130928', NULL, 'XZQH', '吴桥县','河北省沧州市吴桥县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210123', NULL, 'XZQH', '康平县','辽宁省沈阳市康平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130926', NULL, 'XZQH', '肃宁县','河北省沧州市肃宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130925', NULL, 'XZQH', '盐山县','河北省沧州市盐山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130924', NULL, 'XZQH', '海兴县','河北省沧州市海兴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130923', NULL, 'XZQH', '东光县','河北省沧州市东光县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130922', NULL, 'XZQH', '青县','河北省沧州市青县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130921', NULL, 'XZQH', '沧县','河北省沧州市沧县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460300', NULL, 'XZQH', '三沙市','海南省三沙市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('630102', NULL, 'XZQH', '城东区','青海省西宁市城东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('630103', NULL, 'XZQH', '城中区','青海省西宁市城中区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('630104', NULL, 'XZQH', '城西区','青海省西宁市城西区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('630105', NULL, 'XZQH', '城北区','青海省西宁市城北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140402', NULL, 'XZQH', '城区','山西省长治市城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140400', NULL, 'XZQH', '长治市','山西省长治市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341321', NULL, 'XZQH', '砀山县','安徽省宿州市砀山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341322', NULL, 'XZQH', '萧县','安徽省宿州市萧县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341323', NULL, 'XZQH', '灵璧县','安徽省宿州市灵璧县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341324', NULL, 'XZQH', '泗县','安徽省宿州市泗县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360428', NULL, 'XZQH', '都昌县','江西省九江市都昌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211200', NULL, 'XZQH', '铁岭市','辽宁省铁岭市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350502', NULL, 'XZQH', '鲤城区','福建省泉州市鲤城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350503', NULL, 'XZQH', '丰泽区','福建省泉州市丰泽区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211202', NULL, 'XZQH', '银州区','辽宁省铁岭市银州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360429', NULL, 'XZQH', '湖口县','江西省九江市湖口县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350504', NULL, 'XZQH', '洛江区','福建省泉州市洛江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360424', NULL, 'XZQH', '修水县','江西省九江市修水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350505', NULL, 'XZQH', '泉港区','福建省泉州市泉港区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211204', NULL, 'XZQH', '清河区','辽宁省铁岭市清河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440700', NULL, 'XZQH', '江门市','广东省江门市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('445203', NULL, 'XZQH', '揭东区','广东省揭阳市揭东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360423', NULL, 'XZQH', '武宁县','江西省九江市武宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('533124', NULL, 'XZQH', '陇川县','云南省德宏傣族景颇族自治州陇川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360426', NULL, 'XZQH', '德安县','江西省九江市德安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('533123', NULL, 'XZQH', '盈江县','云南省德宏傣族景颇族自治州盈江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360425', NULL, 'XZQH', '永修县','江西省九江市永修县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('533122', NULL, 'XZQH', '梁河县','云南省德宏傣族景颇族自治州梁河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440704', NULL, 'XZQH', '江海区','广东省江门市江海区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440703', NULL, 'XZQH', '蓬江区','广东省江门市蓬江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610202', NULL, 'XZQH', '王益区','陕西省铜川市王益区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360421', NULL, 'XZQH', '柴桑区','江西省九江市柴桑区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610200', NULL, 'XZQH', '铜川市','陕西省铜川市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350500', NULL, 'XZQH', '泉州市','福建省泉州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440705', NULL, 'XZQH', '新会区','广东省江门市新会区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513227', NULL, 'XZQH', '小金县','四川省阿坝藏族羌族自治州小金县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513228', NULL, 'XZQH', '黑水县','四川省阿坝藏族羌族自治州黑水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('445202', NULL, 'XZQH', '榕城区','广东省揭阳市榕城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460321', NULL, 'XZQH', '西沙群岛','海南省三沙市西沙群岛', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460322', NULL, 'XZQH', '南沙群岛','海南省三沙市南沙群岛', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('445200', NULL, 'XZQH', '揭阳市','广东省揭阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460323', NULL, 'XZQH', '中沙群岛','海南省三沙市中沙群岛', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130900', NULL, 'XZQH', '沧州市','河北省沧州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460324', NULL, 'XZQH', '永乐群岛','海南省三沙市永乐群岛', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130902', NULL, 'XZQH', '新华区','河北省沧州市新华区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130903', NULL, 'XZQH', '运河区','河北省沧州市运河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653200', NULL, 'XZQH', '和田地区','新疆维吾尔自治区和田地区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653201', NULL, 'XZQH', '和田市','新疆维吾尔自治区和田地区和田市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520000', NULL, 'XZQH', '贵州省','贵州省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513233', NULL, 'XZQH', '红原县','四川省阿坝藏族羌族自治州红原县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('630123', NULL, 'XZQH', '湟源县','青海省西宁市湟源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513232', NULL, 'XZQH', '若尔盖县','四川省阿坝藏族羌族自治州若尔盖县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('630122', NULL, 'XZQH', '湟中县','青海省西宁市湟中县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513231', NULL, 'XZQH', '阿坝县','四川省阿坝藏族羌族自治州阿坝县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('630121', NULL, 'XZQH', '大通回族土族自治县','青海省西宁市大通回族土族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220221', NULL, 'XZQH', '永吉县','吉林省吉林市永吉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513230', NULL, 'XZQH', '壤塘县','四川省阿坝藏族羌族自治州壤塘县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140426', NULL, 'XZQH', '黎城县','山西省长治市黎城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140425', NULL, 'XZQH', '平顺县','山西省长治市平顺县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140424', NULL, 'XZQH', '屯留县','山西省长治市屯留县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140423', NULL, 'XZQH', '襄垣县','山西省长治市襄垣县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710000', NULL, 'XZQH', '台湾省','台湾省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140429', NULL, 'XZQH', '武乡县','山西省长治市武乡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140428', NULL, 'XZQH', '长子县','山西省长治市长子县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140427', NULL, 'XZQH', '壶关县','山西省长治市壶关县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341302', NULL, 'XZQH', '埇桥区','安徽省宿州市埇桥区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140421', NULL, 'XZQH', '长治县','山西省长治市长治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341300', NULL, 'XZQH', '宿州市','安徽省宿州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210181', NULL, 'XZQH', '新民市','辽宁省沈阳市新民市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350527', NULL, 'XZQH', '金门县','福建省泉州市金门县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620421', NULL, 'XZQH', '靖远县','甘肃省白银市靖远县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350524', NULL, 'XZQH', '安溪县','福建省泉州市安溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211223', NULL, 'XZQH', '西丰县','辽宁省铁岭市西丰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350525', NULL, 'XZQH', '永春县','福建省泉州市永春县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211224', NULL, 'XZQH', '昌图县','辽宁省铁岭市昌图县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350526', NULL, 'XZQH', '德化县','福建省泉州市德化县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330784', NULL, 'XZQH', '永康市','浙江省金华市永康市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330783', NULL, 'XZQH', '东阳市','浙江省金华市东阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710902', NULL, 'XZQH', '西区','台湾省嘉义市西区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350521', NULL, 'XZQH', '惠安县','福建省泉州市惠安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330782', NULL, 'XZQH', '义乌市','浙江省金华市义乌市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710901', NULL, 'XZQH', '东区','台湾省嘉义市东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330781', NULL, 'XZQH', '兰溪市','浙江省金华市兰溪市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710900', NULL, 'XZQH', '嘉义市','台湾省嘉义市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610222', NULL, 'XZQH', '宜君县','陕西省铜川市宜君县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610204', NULL, 'XZQH', '耀州区','陕西省铜川市耀州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610203', NULL, 'XZQH', '印台区','陕西省铜川市印台区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211221', NULL, 'XZQH', '铁岭县','辽宁省铁岭市铁岭县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620400', NULL, 'XZQH', '白银市','甘肃省白银市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140411', NULL, 'XZQH', '郊区','山西省长治市郊区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620402', NULL, 'XZQH', '白银区','甘肃省白银市白银区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620403', NULL, 'XZQH', '平川区','甘肃省白银市平川区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441821', NULL, 'XZQH', '佛冈县','广东省清远市佛冈县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('429006', NULL, 'XZQH', '天门市','湖北省null天门市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441825', NULL, 'XZQH', '连山壮族瑶族自治县','广东省清远市连山壮族瑶族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('429005', NULL, 'XZQH', '潜江市','湖北省null潜江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441826', NULL, 'XZQH', '连南瑶族自治县','广东省清远市连南瑶族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('429004', NULL, 'XZQH', '仙桃市','湖北省null仙桃市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441823', NULL, 'XZQH', '阳山县','广东省清远市阳山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360430', NULL, 'XZQH', '彭泽县','江西省九江市彭泽县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130107', NULL, 'XZQH', '井陉矿区','河北省石家庄市井陉矿区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130108', NULL, 'XZQH', '裕华区','河北省石家庄市裕华区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130105', NULL, 'XZQH', '新华区','河北省石家庄市新华区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130109', NULL, 'XZQH', '藁城区','河北省石家庄市藁城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220282', NULL, 'XZQH', '桦甸市','吉林省吉林市桦甸市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130100', NULL, 'XZQH', '石家庄市','河北省石家庄市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220283', NULL, 'XZQH', '舒兰市','吉林省吉林市舒兰市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220284', NULL, 'XZQH', '磐石市','吉林省吉林市磐石市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130104', NULL, 'XZQH', '桥西区','河北省石家庄市桥西区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130102', NULL, 'XZQH', '长安区','河北省石家庄市长安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220281', NULL, 'XZQH', '蛟河市','吉林省吉林市蛟河市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513201', NULL, 'XZQH', '马尔康市','四川省阿坝藏族羌族自治州马尔康市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513200', NULL, 'XZQH', '阿坝藏族羌族自治州','四川省阿坝藏族羌族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331000', NULL, 'XZQH', '台州市','浙江省台州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371402', NULL, 'XZQH', '德城区','山东省德州市德城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371403', NULL, 'XZQH', '陵城区','山东省德州市陵城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230921', NULL, 'XZQH', '勃利县','黑龙江省七台河市勃利县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371400', NULL, 'XZQH', '德州市','山东省德州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441882', NULL, 'XZQH', '连州市','广东省清远市连州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441881', NULL, 'XZQH', '英德市','广东省清远市英德市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360481', NULL, 'XZQH', '瑞昌市','江西省九江市瑞昌市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331002', NULL, 'XZQH', '椒江区','浙江省台州市椒江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360482', NULL, 'XZQH', '共青城市','江西省九江市共青城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360483', NULL, 'XZQH', '庐山市','江西省九江市庐山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331004', NULL, 'XZQH', '路桥区','浙江省台州市路桥区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513221', NULL, 'XZQH', '汶川县','四川省阿坝藏族羌族自治州汶川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331003', NULL, 'XZQH', '黄岩区','浙江省台州市黄岩区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513222', NULL, 'XZQH', '理县','四川省阿坝藏族羌族自治州理县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513223', NULL, 'XZQH', '茂县','四川省阿坝藏族羌族自治州茂县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513224', NULL, 'XZQH', '松潘县','四川省阿坝藏族羌族自治州松潘县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513225', NULL, 'XZQH', '九寨沟县','四川省阿坝藏族羌族自治州九寨沟县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230000', NULL, 'XZQH', '黑龙江省','黑龙江省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513226', NULL, 'XZQH', '金川县','四川省阿坝藏族羌族自治州金川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('533102', NULL, 'XZQH', '瑞丽市','云南省德宏傣族景颇族自治州瑞丽市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('533103', NULL, 'XZQH', '芒市','云南省德宏傣族景颇族自治州芒市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('533100', NULL, 'XZQH', '德宏傣族景颇族自治州','云南省德宏傣族景颇族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341623', NULL, 'XZQH', '利辛县','安徽省亳州市利辛县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341622', NULL, 'XZQH', '蒙城县','安徽省亳州市蒙城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341621', NULL, 'XZQH', '涡阳县','安徽省亳州市涡阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532600', NULL, 'XZQH', '文山壮族苗族自治州','云南省文山壮族苗族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532601', NULL, 'XZQH', '文山市','云南省文山壮族苗族自治州文山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('331022', NULL, 'XZQH', '三门县','浙江省台州市三门县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230902', NULL, 'XZQH', '新兴区','黑龙江省七台河市新兴区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230903', NULL, 'XZQH', '桃山区','黑龙江省七台河市桃山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230904', NULL, 'XZQH', '茄子河区','黑龙江省七台河市茄子河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230900', NULL, 'XZQH', '七台河市','黑龙江省七台河市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371427', NULL, 'XZQH', '夏津县','山东省德州市夏津县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371428', NULL, 'XZQH', '武城县','山东省德州市武城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371425', NULL, 'XZQH', '齐河县','山东省德州市齐河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371426', NULL, 'XZQH', '平原县','山东省德州市平原县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371423', NULL, 'XZQH', '庆云县','山东省德州市庆云县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370812', NULL, 'XZQH', '兖州区','山东省济宁市兖州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371424', NULL, 'XZQH', '临邑县','山东省德州市临邑县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370811', NULL, 'XZQH', '任城区','山东省济宁市任城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653022', NULL, 'XZQH', '阿克陶县','新疆维吾尔自治区克孜勒苏柯尔克孜自治州阿克陶县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653023', NULL, 'XZQH', '阿合奇县','新疆维吾尔自治区克孜勒苏柯尔克孜自治州阿合奇县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371422', NULL, 'XZQH', '宁津县','山东省德州市宁津县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653024', NULL, 'XZQH', '乌恰县','新疆维吾尔自治区克孜勒苏柯尔克孜自治州乌恰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131128', NULL, 'XZQH', '阜城县','河北省衡水市阜城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340811', NULL, 'XZQH', '宜秀区','安徽省安庆市宜秀区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131126', NULL, 'XZQH', '故城县','河北省衡水市故城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131127', NULL, 'XZQH', '景县','河北省衡水市景县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131124', NULL, 'XZQH', '饶阳县','河北省衡水市饶阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131125', NULL, 'XZQH', '安平县','河北省衡水市安平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131122', NULL, 'XZQH', '武邑县','河北省衡水市武邑县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230184', NULL, 'XZQH', '五常市','黑龙江省哈尔滨市五常市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131123', NULL, 'XZQH', '武强县','河北省衡水市武强县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131121', NULL, 'XZQH', '枣强县','河北省衡水市枣强县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230183', NULL, 'XZQH', '尚志市','黑龙江省哈尔滨市尚志市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('222406', NULL, 'XZQH', '和龙市','吉林省延边朝鲜族自治州和龙市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('222402', NULL, 'XZQH', '图们市','吉林省延边朝鲜族自治州图们市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('222403', NULL, 'XZQH', '敦化市','吉林省延边朝鲜族自治州敦化市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('222404', NULL, 'XZQH', '珲春市','吉林省延边朝鲜族自治州珲春市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('222405', NULL, 'XZQH', '龙井市','吉林省延边朝鲜族自治州龙井市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360602', NULL, 'XZQH', '月湖区','江西省鹰潭市月湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360600', NULL, 'XZQH', '鹰潭市','江西省鹰潭市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('222400', NULL, 'XZQH', '延边朝鲜族自治州','吉林省延边朝鲜族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('222401', NULL, 'XZQH', '延吉市','吉林省延边朝鲜族自治州延吉市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411525', NULL, 'XZQH', '固始县','河南省信阳市固始县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411526', NULL, 'XZQH', '潢川县','河南省信阳市潢川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411527', NULL, 'XZQH', '淮滨县','河南省信阳市淮滨县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411528', NULL, 'XZQH', '息县','河南省信阳市息县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150430', NULL, 'XZQH', '敖汉旗','内蒙古自治区赤峰市敖汉旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411521', NULL, 'XZQH', '罗山县','河南省信阳市罗山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('422822', NULL, 'XZQH', '建始县','湖北省恩施土家族苗族自治州建始县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441781', NULL, 'XZQH', '阳春市','广东省阳江市阳春市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411522', NULL, 'XZQH', '光山县','河南省信阳市光山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('422823', NULL, 'XZQH', '巴东县','湖北省恩施土家族苗族自治州巴东县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411523', NULL, 'XZQH', '新县','河南省信阳市新县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411524', NULL, 'XZQH', '商城县','河南省信阳市商城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('422825', NULL, 'XZQH', '宣恩县','湖北省恩施土家族苗族自治州宣恩县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370826', NULL, 'XZQH', '微山县','山东省济宁市微山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340824', NULL, 'XZQH', '潜山县','安徽省安庆市潜山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('422826', NULL, 'XZQH', '咸丰县','湖北省恩施土家族苗族自治州咸丰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370827', NULL, 'XZQH', '鱼台县','山东省济宁市鱼台县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('422827', NULL, 'XZQH', '来凤县','湖北省恩施土家族苗族自治州来凤县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370828', NULL, 'XZQH', '金乡县','山东省济宁市金乡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340822', NULL, 'XZQH', '怀宁县','安徽省安庆市怀宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('422828', NULL, 'XZQH', '鹤峰县','湖北省恩施土家族苗族自治州鹤峰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370829', NULL, 'XZQH', '嘉祥县','山东省济宁市嘉祥县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340828', NULL, 'XZQH', '岳西县','安徽省安庆市岳西县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340827', NULL, 'XZQH', '望江县','安徽省安庆市望江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340826', NULL, 'XZQH', '宿松县','安徽省安庆市宿松县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340825', NULL, 'XZQH', '太湖县','安徽省安庆市太湖县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370832', NULL, 'XZQH', '梁山县','山东省济宁市梁山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370831', NULL, 'XZQH', '泗水县','山东省济宁市泗水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370830', NULL, 'XZQH', '汶上县','山东省济宁市汶上县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450681', NULL, 'XZQH', '东兴市','广西壮族自治区防城港市东兴市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511181', NULL, 'XZQH', '峨眉山市','四川省乐山市峨眉山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('222424', NULL, 'XZQH', '汪清县','吉林省延边朝鲜族自治州汪清县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('222426', NULL, 'XZQH', '安图县','吉林省延边朝鲜族自治州安图县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431381', NULL, 'XZQH', '冷水江市','湖南省娄底市冷水江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411500', NULL, 'XZQH', '信阳市','河南省信阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431382', NULL, 'XZQH', '涟源市','湖南省娄底市涟源市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411502', NULL, 'XZQH', '浉河区','河南省信阳市浉河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652702', NULL, 'XZQH', '阿拉山口市','新疆维吾尔自治区博尔塔拉蒙古自治州阿拉山口市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411503', NULL, 'XZQH', '平桥区','河南省信阳市平桥区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652700', NULL, 'XZQH', '博尔塔拉蒙古自治州','新疆维吾尔自治区博尔塔拉蒙古自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652701', NULL, 'XZQH', '博乐市','新疆维吾尔自治区博尔塔拉蒙古自治州博乐市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220323', NULL, 'XZQH', '伊通满族自治县','吉林省四平市伊通满族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220322', NULL, 'XZQH', '梨树县','吉林省四平市梨树县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150402', NULL, 'XZQH', '红山区','内蒙古自治区赤峰市红山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150403', NULL, 'XZQH', '元宝山区','内蒙古自治区赤峰市元宝山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150400', NULL, 'XZQH', '赤峰市','内蒙古自治区赤峰市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150404', NULL, 'XZQH', '松山区','内蒙古自治区赤峰市松山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140212', NULL, 'XZQH', '新荣区','山西省大同市新荣区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140211', NULL, 'XZQH', '南郊区','山西省大同市南郊区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710707', NULL, 'XZQH', '信义区','台湾省基隆市信义区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710700', NULL, 'XZQH', '基隆市','台湾省基隆市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710701', NULL, 'XZQH', '中正区','台湾省基隆市中正区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710702', NULL, 'XZQH', '七堵区','台湾省基隆市七堵区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710703', NULL, 'XZQH', '暖暖区','台湾省基隆市暖暖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710704', NULL, 'XZQH', '仁爱区','台湾省基隆市仁爱区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710705', NULL, 'XZQH', '中山区','台湾省基隆市中山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710706', NULL, 'XZQH', '安乐区','台湾省基隆市安乐区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140225', NULL, 'XZQH', '浑源县','山西省大同市浑源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140226', NULL, 'XZQH', '左云县','山西省大同市左云县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140227', NULL, 'XZQH', '大同县','山西省大同市大同县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140221', NULL, 'XZQH', '阳高县','山西省大同市阳高县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140222', NULL, 'XZQH', '天镇县','山西省大同市天镇县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652723', NULL, 'XZQH', '温泉县','新疆维吾尔自治区博尔塔拉蒙古自治州温泉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140223', NULL, 'XZQH', '广灵县','山西省大同市广灵县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652722', NULL, 'XZQH', '精河县','新疆维吾尔自治区博尔塔拉蒙古自治州精河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140224', NULL, 'XZQH', '灵丘县','山西省大同市灵丘县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371482', NULL, 'XZQH', '禹城市','山东省德州市禹城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371481', NULL, 'XZQH', '乐陵市','山东省德州市乐陵市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653001', NULL, 'XZQH', '阿图什市','新疆维吾尔自治区克孜勒苏柯尔克孜自治州阿图什市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653000', NULL, 'XZQH', '克孜勒苏柯尔克孜自治州','新疆维吾尔自治区克孜勒苏柯尔克孜自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450123', NULL, 'XZQH', '隆安县','广西壮族自治区南宁市隆安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220300', NULL, 'XZQH', '四平市','吉林省四平市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511133', NULL, 'XZQH', '马边彝族自治县','四川省乐山市马边彝族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220302', NULL, 'XZQH', '铁西区','吉林省四平市铁西区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450127', NULL, 'XZQH', '横县','广西壮族自治区南宁市横县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450126', NULL, 'XZQH', '宾阳县','广西壮族自治区南宁市宾阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511132', NULL, 'XZQH', '峨边彝族自治县','四川省乐山市峨边彝族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450125', NULL, 'XZQH', '上林县','广西壮族自治区南宁市上林县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220303', NULL, 'XZQH', '铁东区','吉林省四平市铁东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450124', NULL, 'XZQH', '马山县','广西壮族自治区南宁市马山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150428', NULL, 'XZQH', '喀喇沁旗','内蒙古自治区赤峰市喀喇沁旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150429', NULL, 'XZQH', '宁城县','内蒙古自治区赤峰市宁城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131100', NULL, 'XZQH', '衡水市','河北省衡水市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150426', NULL, 'XZQH', '翁牛特旗','内蒙古自治区赤峰市翁牛特旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150424', NULL, 'XZQH', '林西县','内蒙古自治区赤峰市林西县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511129', NULL, 'XZQH', '沐川县','四川省乐山市沐川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150425', NULL, 'XZQH', '克什克腾旗','内蒙古自治区赤峰市克什克腾旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150422', NULL, 'XZQH', '巴林左旗','内蒙古自治区赤峰市巴林左旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131103', NULL, 'XZQH', '冀州区','河北省衡水市冀州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150423', NULL, 'XZQH', '巴林右旗','内蒙古自治区赤峰市巴林右旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131102', NULL, 'XZQH', '桃城区','河北省衡水市桃城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511126', NULL, 'XZQH', '夹江县','四川省乐山市夹江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150421', NULL, 'XZQH', '阿鲁科尔沁旗','内蒙古自治区赤峰市阿鲁科尔沁旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620423', NULL, 'XZQH', '景泰县','甘肃省白银市景泰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620422', NULL, 'XZQH', '会宁县','甘肃省白银市会宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360622', NULL, 'XZQH', '余江县','江西省鹰潭市余江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370800', NULL, 'XZQH', '济宁市','山东省济宁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140203', NULL, 'XZQH', '矿区','山西省大同市矿区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('422801', NULL, 'XZQH', '恩施市','湖北省恩施土家族苗族自治州恩施市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('422800', NULL, 'XZQH', '恩施土家族苗族自治州','湖北省恩施土家族苗族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('422802', NULL, 'XZQH', '利川市','湖北省恩施土家族苗族自治州利川市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340881', NULL, 'XZQH', '桐城市','安徽省安庆市桐城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140200', NULL, 'XZQH', '大同市','山西省大同市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140202', NULL, 'XZQH', '城区','山西省大同市城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320583', NULL, 'XZQH', '昆山市','江苏省苏州市昆山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320582', NULL, 'XZQH', '张家港市','江苏省苏州市张家港市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320581', NULL, 'XZQH', '常熟市','江苏省苏州市常熟市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511112', NULL, 'XZQH', '五通桥区','四川省乐山市五通桥区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511111', NULL, 'XZQH', '沙湾区','四川省乐山市沙湾区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511113', NULL, 'XZQH', '金口河区','四川省乐山市金口河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431322', NULL, 'XZQH', '新化县','湖南省娄底市新化县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431321', NULL, 'XZQH', '双峰县','湖南省娄底市双峰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640300', NULL, 'XZQH', '吴忠市','宁夏回族自治区吴忠市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640302', NULL, 'XZQH', '利通区','宁夏回族自治区吴忠市利通区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640303', NULL, 'XZQH', '红寺堡区','宁夏回族自治区吴忠市红寺堡区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450621', NULL, 'XZQH', '上思县','广西壮族自治区防城港市上思县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511124', NULL, 'XZQH', '井研县','四川省乐山市井研县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511123', NULL, 'XZQH', '犍为县','四川省乐山市犍为县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320585', NULL, 'XZQH', '太仓市','江苏省苏州市太仓市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410481', NULL, 'XZQH', '舞钢市','河南省平顶山市舞钢市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410482', NULL, 'XZQH', '汝州市','河南省平顶山市汝州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430725', NULL, 'XZQH', '桃源县','湖南省常德市桃源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430724', NULL, 'XZQH', '临澧县','湖南省常德市临澧县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430726', NULL, 'XZQH', '石门县','湖南省常德市石门县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421223', NULL, 'XZQH', '崇阳县','湖北省咸宁市崇阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421224', NULL, 'XZQH', '通山县','湖北省咸宁市通山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431302', NULL, 'XZQH', '娄星区','湖南省娄底市娄星区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431300', NULL, 'XZQH', '娄底市','湖南省娄底市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430721', NULL, 'XZQH', '安乡县','湖南省常德市安乡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430723', NULL, 'XZQH', '澧县','湖南省常德市澧县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430722', NULL, 'XZQH', '汉寿县','湖南省常德市汉寿县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640324', NULL, 'XZQH', '同心县','宁夏回族自治区吴忠市同心县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640323', NULL, 'XZQH', '盐池县','宁夏回族自治区吴忠市盐池县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421222', NULL, 'XZQH', '通城县','湖北省咸宁市通城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421221', NULL, 'XZQH', '嘉鱼县','湖北省咸宁市嘉鱼县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451221', NULL, 'XZQH', '南丹县','广西壮族自治区河池市南丹县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511102', NULL, 'XZQH', '市中区','四川省乐山市市中区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451223', NULL, 'XZQH', '凤山县','广西壮族自治区河池市凤山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451222', NULL, 'XZQH', '天峨县','广西壮族自治区河池市天峨县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511100', NULL, 'XZQH', '乐山市','四川省乐山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451225', NULL, 'XZQH', '罗城仫佬族自治县','广西壮族自治区河池市罗城仫佬族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451224', NULL, 'XZQH', '东兰县','广西壮族自治区河池市东兰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451227', NULL, 'XZQH', '巴马瑶族自治县','广西壮族自治区河池市巴马瑶族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450600', NULL, 'XZQH', '防城港市','广西壮族自治区防城港市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451226', NULL, 'XZQH', '环江毛南族自治县','广西壮族自治区河池市环江毛南族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451229', NULL, 'XZQH', '大化瑶族自治县','广西壮族自治区河池市大化瑶族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450602', NULL, 'XZQH', '港口区','广西壮族自治区防城港市港口区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451228', NULL, 'XZQH', '都安瑶族自治县','广西壮族自治区河池市都安瑶族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450603', NULL, 'XZQH', '防城区','广西壮族自治区防城港市防城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410425', NULL, 'XZQH', '郏县','河南省平顶山市郏县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410423', NULL, 'XZQH', '鲁山县','河南省平顶山市鲁山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410421', NULL, 'XZQH', '宝丰县','河南省平顶山市宝丰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410422', NULL, 'XZQH', '叶县','河南省平顶山市叶县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430700', NULL, 'XZQH', '常德市','湖南省常德市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421202', NULL, 'XZQH', '咸安区','湖北省咸宁市咸安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430702', NULL, 'XZQH', '武陵区','湖南省常德市武陵区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430703', NULL, 'XZQH', '鼎城区','湖南省常德市鼎城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421200', NULL, 'XZQH', '咸宁市','湖北省咸宁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211081', NULL, 'XZQH', '灯塔市','辽宁省辽阳市灯塔市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410402', NULL, 'XZQH', '新华区','河南省平顶山市新华区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410400', NULL, 'XZQH', '平顶山市','河南省平顶山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410403', NULL, 'XZQH', '卫东区','河南省平顶山市卫东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410404', NULL, 'XZQH', '石龙区','河南省平顶山市石龙区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640381', NULL, 'XZQH', '青铜峡市','宁夏回族自治区吴忠市青铜峡市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410411', NULL, 'XZQH', '湛河区','河南省平顶山市湛河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370883', NULL, 'XZQH', '邹城市','山东省济宁市邹城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370881', NULL, 'XZQH', '曲阜市','山东省济宁市曲阜市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320508', NULL, 'XZQH', '姑苏区','江苏省苏州市姑苏区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320509', NULL, 'XZQH', '吴江区','江苏省苏州市吴江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320506', NULL, 'XZQH', '吴中区','江苏省苏州市吴中区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320507', NULL, 'XZQH', '相城区','江苏省苏州市相城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513301', NULL, 'XZQH', '康定市','四川省甘孜藏族自治州康定市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320505', NULL, 'XZQH', '虎丘区','江苏省苏州市虎丘区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513300', NULL, 'XZQH', '甘孜藏族自治州','四川省甘孜藏族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220800', NULL, 'XZQH', '白城市','吉林省白城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320500', NULL, 'XZQH', '苏州市','江苏省苏州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220802', NULL, 'XZQH', '洮北区','吉林省白城市洮北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420583', NULL, 'XZQH', '枝江市','湖北省宜昌市枝江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420581', NULL, 'XZQH', '宜都市','湖北省宜昌市宜都市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420582', NULL, 'XZQH', '当阳市','湖北省宜昌市当阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130000', NULL, 'XZQH', '河北省','河北省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350481', NULL, 'XZQH', '永安市','福建省三明市永安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330600', NULL, 'XZQH', '绍兴市','浙江省绍兴市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330603', NULL, 'XZQH', '柯桥区','浙江省绍兴市柯桥区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330602', NULL, 'XZQH', '越城区','浙江省绍兴市越城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330604', NULL, 'XZQH', '上虞区','浙江省绍兴市上虞区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530502', NULL, 'XZQH', '隆阳区','云南省保山市隆阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530500', NULL, 'XZQH', '保山市','云南省保山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421181', NULL, 'XZQH', '麻城市','湖北省黄冈市麻城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421182', NULL, 'XZQH', '武穴市','湖北省黄冈市武穴市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440600', NULL, 'XZQH', '佛山市','广东省佛山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440606', NULL, 'XZQH', '顺德区','广东省佛山市顺德区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451200', NULL, 'XZQH', '河池市','广西壮族自治区河池市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440607', NULL, 'XZQH', '三水区','广东省佛山市三水区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440608', NULL, 'XZQH', '高明区','广东省佛山市高明区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440604', NULL, 'XZQH', '禅城区','广东省佛山市禅城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451202', NULL, 'XZQH', '金城江区','广西壮族自治区河池市金城江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440605', NULL, 'XZQH', '南海区','广东省佛山市南海区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451203', NULL, 'XZQH', '宜州区','广西壮族自治区河池市宜州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530521', NULL, 'XZQH', '施甸县','云南省保山市施甸县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530523', NULL, 'XZQH', '龙陵县','云南省保山市龙陵县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530524', NULL, 'XZQH', '昌宁县','云南省保山市昌宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('820000', NULL, 'XZQH', '澳门特别行政区','澳门特别行政区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('419001', NULL, 'XZQH', '济源市','河南省null济源市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220822', NULL, 'XZQH', '通榆县','吉林省白城市通榆县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410381', NULL, 'XZQH', '偃师市','河南省洛阳市偃师市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210800', NULL, 'XZQH', '营口市','辽宁省营口市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210802', NULL, 'XZQH', '站前区','辽宁省营口市站前区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610122', NULL, 'XZQH', '蓝田县','陕西省西安市蓝田县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210803', NULL, 'XZQH', '西市区','辽宁省营口市西市区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220821', NULL, 'XZQH', '镇赉县','吉林省白城市镇赉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210804', NULL, 'XZQH', '鲅鱼圈区','辽宁省营口市鲅鱼圈区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610124', NULL, 'XZQH', '周至县','陕西省西安市周至县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610115', NULL, 'XZQH', '临潼区','陕西省西安市临潼区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610116', NULL, 'XZQH', '长安区','陕西省西安市长安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610117', NULL, 'XZQH', '高陵区','陕西省西安市高陵区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610118', NULL, 'XZQH', '鄠邑区','陕西省西安市鄠邑区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430781', NULL, 'XZQH', '津市市','湖南省常德市津市市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210000', NULL, 'XZQH', '辽宁省','辽宁省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210811', NULL, 'XZQH', '老边区','辽宁省营口市老边区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610114', NULL, 'XZQH', '阎良区','陕西省西安市阎良区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610113', NULL, 'XZQH', '雁塔区','陕西省西安市雁塔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610112', NULL, 'XZQH', '未央区','陕西省西安市未央区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610111', NULL, 'XZQH', '灞桥区','陕西省西安市灞桥区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610104', NULL, 'XZQH', '莲湖区','陕西省西安市莲湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152530', NULL, 'XZQH', '正蓝旗','内蒙古自治区锡林郭勒盟正蓝旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152531', NULL, 'XZQH', '多伦县','内蒙古自治区锡林郭勒盟多伦县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211122', NULL, 'XZQH', '盘山县','辽宁省盘锦市盘山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140300', NULL, 'XZQH', '阳泉市','山西省阳泉市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140303', NULL, 'XZQH', '矿区','山西省阳泉市矿区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140302', NULL, 'XZQH', '城区','山西省阳泉市城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220382', NULL, 'XZQH', '双辽市','吉林省四平市双辽市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220381', NULL, 'XZQH', '公主岭市','吉林省四平市公主岭市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220882', NULL, 'XZQH', '大安市','吉林省白城市大安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350400', NULL, 'XZQH', '三明市','福建省三明市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220881', NULL, 'XZQH', '洮南市','吉林省白城市洮南市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450110', NULL, 'XZQH', '武鸣区','广西壮族自治区南宁市武鸣区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610102', NULL, 'XZQH', '新城区','陕西省西安市新城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610103', NULL, 'XZQH', '碑林区','陕西省西安市碑林区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610100', NULL, 'XZQH', '西安市','陕西省西安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350402', NULL, 'XZQH', '梅列区','福建省三明市梅列区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350403', NULL, 'XZQH', '三元区','福建省三明市三元区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211102', NULL, 'XZQH', '双台子区','辽宁省盘锦市双台子区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211103', NULL, 'XZQH', '兴隆台区','辽宁省盘锦市兴隆台区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211104', NULL, 'XZQH', '大洼区','辽宁省盘锦市大洼区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211100', NULL, 'XZQH', '盘锦市','辽宁省盘锦市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620525', NULL, 'XZQH', '张家川回族自治县','甘肃省天水市张家川回族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620521', NULL, 'XZQH', '清水县','甘肃省天水市清水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620522', NULL, 'XZQH', '秦安县','甘肃省天水市秦安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620523', NULL, 'XZQH', '甘谷县','甘肃省天水市甘谷县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620524', NULL, 'XZQH', '武山县','甘肃省天水市武山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450100', NULL, 'XZQH', '南宁市','广西壮族自治区南宁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441704', NULL, 'XZQH', '阳东区','广东省阳江市阳东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450102', NULL, 'XZQH', '兴宁区','广西壮族自治区南宁市兴宁区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450103', NULL, 'XZQH', '青秀区','广西壮族自治区南宁市青秀区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441702', NULL, 'XZQH', '江城区','广东省阳江市江城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450105', NULL, 'XZQH', '江南区','广西壮族自治区南宁市江南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411481', NULL, 'XZQH', '永城市','河南省商丘市永城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441700', NULL, 'XZQH', '阳江市','广东省阳江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450107', NULL, 'XZQH', '西乡塘区','广西壮族自治区南宁市西乡塘区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450108', NULL, 'XZQH', '良庆区','广西壮族自治区南宁市良庆区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450109', NULL, 'XZQH', '邕宁区','广西壮族自治区南宁市邕宁区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('630223', NULL, 'XZQH', '互助土族自治县','青海省海东市互助土族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('630224', NULL, 'XZQH', '化隆回族自治县','青海省海东市化隆回族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('630225', NULL, 'XZQH', '循化撒拉族自治县','青海省海东市循化撒拉族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710801', NULL, 'XZQH', '东区','台湾省新竹市东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710800', NULL, 'XZQH', '新竹市','台湾省新竹市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710803', NULL, 'XZQH', '香山区','台湾省新竹市香山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330683', NULL, 'XZQH', '嵊州市','浙江省绍兴市嵊州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710802', NULL, 'XZQH', '北区','台湾省新竹市北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460200', NULL, 'XZQH', '三亚市','海南省三亚市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620500', NULL, 'XZQH', '天水市','甘肃省天水市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460202', NULL, 'XZQH', '海棠区','海南省三亚市海棠区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460203', NULL, 'XZQH', '吉阳区','海南省三亚市吉阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620502', NULL, 'XZQH', '秦州区','甘肃省天水市秦州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620503', NULL, 'XZQH', '麦积区','甘肃省天水市麦积区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330681', NULL, 'XZQH', '诸暨市','浙江省绍兴市诸暨市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460204', NULL, 'XZQH', '天涯区','海南省三亚市天涯区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460205', NULL, 'XZQH', '崖州区','海南省三亚市崖州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420500', NULL, 'XZQH', '宜昌市','湖北省宜昌市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420504', NULL, 'XZQH', '点军区','湖北省宜昌市点军区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420503', NULL, 'XZQH', '伍家岗区','湖北省宜昌市伍家岗区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420502', NULL, 'XZQH', '西陵区','湖北省宜昌市西陵区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420506', NULL, 'XZQH', '夷陵区','湖北省宜昌市夷陵区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530581', NULL, 'XZQH', '腾冲市','云南省保山市腾冲市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420505', NULL, 'XZQH', '猇亭区','湖北省宜昌市猇亭区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441721', NULL, 'XZQH', '阳西县','广东省阳江市阳西县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513338', NULL, 'XZQH', '得荣县','四川省甘孜藏族自治州得荣县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513337', NULL, 'XZQH', '稻城县','四川省甘孜藏族自治州稻城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632500', NULL, 'XZQH', '海南藏族自治州','青海省海南藏族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330624', NULL, 'XZQH', '新昌县','浙江省绍兴市新昌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230128', NULL, 'XZQH', '通河县','黑龙江省哈尔滨市通河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230129', NULL, 'XZQH', '延寿县','黑龙江省哈尔滨市延寿县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230126', NULL, 'XZQH', '巴彦县','黑龙江省哈尔滨市巴彦县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230127', NULL, 'XZQH', '木兰县','黑龙江省哈尔滨市木兰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230124', NULL, 'XZQH', '方正县','黑龙江省哈尔滨市方正县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230125', NULL, 'XZQH', '宾县','黑龙江省哈尔滨市宾县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230123', NULL, 'XZQH', '依兰县','黑龙江省哈尔滨市依兰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('131182', NULL, 'XZQH', '深州市','河北省衡水市深州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513329', NULL, 'XZQH', '新龙县','四川省甘孜藏族自治州新龙县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513328', NULL, 'XZQH', '甘孜县','四川省甘孜藏族自治州甘孜县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341504', NULL, 'XZQH', '叶集区','安徽省六安市叶集区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513327', NULL, 'XZQH', '炉霍县','四川省甘孜藏族自治州炉霍县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341503', NULL, 'XZQH', '裕安区','安徽省六安市裕安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513326', NULL, 'XZQH', '道孚县','四川省甘孜藏族自治州道孚县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341502', NULL, 'XZQH', '金安区','安徽省六安市金安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341500', NULL, 'XZQH', '六安市','安徽省六安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420529', NULL, 'XZQH', '五峰土家族自治县','湖北省宜昌市五峰土家族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420527', NULL, 'XZQH', '秭归县','湖北省宜昌市秭归县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420528', NULL, 'XZQH', '长阳土家族自治县','湖北省宜昌市长阳土家族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420525', NULL, 'XZQH', '远安县','湖北省宜昌市远安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513331', NULL, 'XZQH', '白玉县','四川省甘孜藏族自治州白玉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371500', NULL, 'XZQH', '聊城市','山东省聊城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420526', NULL, 'XZQH', '兴山县','湖北省宜昌市兴山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513332', NULL, 'XZQH', '石渠县','四川省甘孜藏族自治州石渠县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371502', NULL, 'XZQH', '东昌府区','山东省聊城市东昌府区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513330', NULL, 'XZQH', '德格县','四川省甘孜藏族自治州德格县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350430', NULL, 'XZQH', '建宁县','福建省三明市建宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513335', NULL, 'XZQH', '巴塘县','四川省甘孜藏族自治州巴塘县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513336', NULL, 'XZQH', '乡城县','四川省甘孜藏族自治州乡城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513333', NULL, 'XZQH', '色达县','四川省甘孜藏族自治州色达县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513334', NULL, 'XZQH', '理塘县','四川省甘孜藏族自治州理塘县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632521', NULL, 'XZQH', '共和县','青海省海南藏族自治州共和县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350425', NULL, 'XZQH', '大田县','福建省三明市大田县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230108', NULL, 'XZQH', '平房区','黑龙江省哈尔滨市平房区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350424', NULL, 'XZQH', '宁化县','福建省三明市宁化县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230109', NULL, 'XZQH', '松北区','黑龙江省哈尔滨市松北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350427', NULL, 'XZQH', '沙县','福建省三明市沙县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210881', NULL, 'XZQH', '盖州市','辽宁省营口市盖州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350426', NULL, 'XZQH', '尤溪县','福建省三明市尤溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210882', NULL, 'XZQH', '大石桥市','辽宁省营口市大石桥市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350429', NULL, 'XZQH', '泰宁县','福建省三明市泰宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350428', NULL, 'XZQH', '将乐县','福建省三明市将乐县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230100', NULL, 'XZQH', '哈尔滨市','黑龙江省哈尔滨市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513322', NULL, 'XZQH', '泸定县','四川省甘孜藏族自治州泸定县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513323', NULL, 'XZQH', '丹巴县','四川省甘孜藏族自治州丹巴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513324', NULL, 'XZQH', '九龙县','四川省甘孜藏族自治州九龙县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513325', NULL, 'XZQH', '雅江县','四川省甘孜藏族自治州雅江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350421', NULL, 'XZQH', '明溪县','福建省三明市明溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230104', NULL, 'XZQH', '道外区','黑龙江省哈尔滨市道外区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350423', NULL, 'XZQH', '清流县','福建省三明市清流县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230102', NULL, 'XZQH', '道里区','黑龙江省哈尔滨市道里区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230103', NULL, 'XZQH', '南岗区','黑龙江省哈尔滨市南岗区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341525', NULL, 'XZQH', '霍山县','安徽省六安市霍山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341524', NULL, 'XZQH', '金寨县','安徽省六安市金寨县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341523', NULL, 'XZQH', '舒城县','安徽省六安市舒城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341522', NULL, 'XZQH', '霍邱县','安徽省六安市霍邱县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371521', NULL, 'XZQH', '阳谷县','山东省聊城市阳谷县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371526', NULL, 'XZQH', '高唐县','山东省聊城市高唐县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230110', NULL, 'XZQH', '香坊区','黑龙江省哈尔滨市香坊区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230111', NULL, 'XZQH', '呼兰区','黑龙江省哈尔滨市呼兰区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230112', NULL, 'XZQH', '阿城区','黑龙江省哈尔滨市阿城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371523', NULL, 'XZQH', '茌平县','山东省聊城市茌平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230113', NULL, 'XZQH', '双城区','黑龙江省哈尔滨市双城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371522', NULL, 'XZQH', '莘县','山东省聊城市莘县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371525', NULL, 'XZQH', '冠县','山东省聊城市冠县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371524', NULL, 'XZQH', '东阿县','山东省聊城市东阿县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441625', NULL, 'XZQH', '东源县','广东省河源市东源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654022', NULL, 'XZQH', '察布查尔锡伯自治县','新疆维吾尔自治区伊犁哈萨克自治州察布查尔锡伯自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441621', NULL, 'XZQH', '紫金县','广东省河源市紫金县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654021', NULL, 'XZQH', '伊宁县','新疆维吾尔自治区伊犁哈萨克自治州伊宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441622', NULL, 'XZQH', '龙川县','广东省河源市龙川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441623', NULL, 'XZQH', '连平县','广东省河源市连平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441624', NULL, 'XZQH', '和平县','广东省河源市和平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654026', NULL, 'XZQH', '昭苏县','新疆维吾尔自治区伊犁哈萨克自治州昭苏县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654025', NULL, 'XZQH', '新源县','新疆维吾尔自治区伊犁哈萨克自治州新源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654024', NULL, 'XZQH', '巩留县','新疆维吾尔自治区伊犁哈萨克自治州巩留县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654023', NULL, 'XZQH', '霍城县','新疆维吾尔自治区伊犁哈萨克自治州霍城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360781', NULL, 'XZQH', '瑞金市','江西省赣州市瑞金市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350322', NULL, 'XZQH', '仙游县','福建省莆田市仙游县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520330', NULL, 'XZQH', '习水县','贵州省遵义市习水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520328', NULL, 'XZQH', '湄潭县','贵州省遵义市湄潭县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520329', NULL, 'XZQH', '余庆县','贵州省遵义市余庆县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520326', NULL, 'XZQH', '务川仡佬族苗族自治县','贵州省遵义市务川仡佬族苗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520327', NULL, 'XZQH', '凤冈县','贵州省遵义市凤冈县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520324', NULL, 'XZQH', '正安县','贵州省遵义市正安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520325', NULL, 'XZQH', '道真仡佬族苗族自治县','贵州省遵义市道真仡佬族苗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520322', NULL, 'XZQH', '桐梓县','贵州省遵义市桐梓县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520323', NULL, 'XZQH', '绥阳县','贵州省遵义市绥阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654004', NULL, 'XZQH', '霍尔果斯市','新疆维吾尔自治区伊犁哈萨克自治州霍尔果斯市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654003', NULL, 'XZQH', '奎屯市','新疆维吾尔自治区伊犁哈萨克自治州奎屯市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654002', NULL, 'XZQH', '伊宁市','新疆维吾尔自治区伊犁哈萨克自治州伊宁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654000', NULL, 'XZQH', '伊犁哈萨克自治州','新疆维吾尔自治区伊犁哈萨克自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130635', NULL, 'XZQH', '蠡县','河北省保定市蠡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130636', NULL, 'XZQH', '顺平县','河北省保定市顺平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130633', NULL, 'XZQH', '易县','河北省保定市易县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130634', NULL, 'XZQH', '曲阳县','河北省保定市曲阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130637', NULL, 'XZQH', '博野县','河北省保定市博野县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130638', NULL, 'XZQH', '雄县','河北省保定市雄县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130631', NULL, 'XZQH', '望都县','河北省保定市望都县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130632', NULL, 'XZQH', '安新县','河北省保定市安新县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130630', NULL, 'XZQH', '涞源县','河北省保定市涞源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350302', NULL, 'XZQH', '城厢区','福建省莆田市城厢区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350300', NULL, 'XZQH', '莆田市','福建省莆田市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350305', NULL, 'XZQH', '秀屿区','福建省莆田市秀屿区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140181', NULL, 'XZQH', '古交市','山西省太原市古交市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350303', NULL, 'XZQH', '涵江区','福建省莆田市涵江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350304', NULL, 'XZQH', '荔城区','福建省莆田市荔城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330182', NULL, 'XZQH', '建德市','浙江省杭州市建德市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340322', NULL, 'XZQH', '五河县','安徽省蚌埠市五河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340323', NULL, 'XZQH', '固镇县','安徽省蚌埠市固镇县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340321', NULL, 'XZQH', '怀远县','安徽省蚌埠市怀远县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510683', NULL, 'XZQH', '绵竹市','四川省德阳市绵竹市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510682', NULL, 'XZQH', '什邡市','四川省德阳市什邡市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510681', NULL, 'XZQH', '广汉市','四川省德阳市广汉市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540523', NULL, 'XZQH', '桑日县','西藏自治区山南市桑日县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540522', NULL, 'XZQH', '贡嘎县','西藏自治区山南市贡嘎县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540525', NULL, 'XZQH', '曲松县','西藏自治区山南市曲松县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540524', NULL, 'XZQH', '琼结县','西藏自治区山南市琼结县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150500', NULL, 'XZQH', '通辽市','内蒙古自治区通辽市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540521', NULL, 'XZQH', '扎囊县','西藏自治区山南市扎囊县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150502', NULL, 'XZQH', '科尔沁区','内蒙古自治区通辽市科尔沁区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340311', NULL, 'XZQH', '淮上区','安徽省蚌埠市淮上区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540527', NULL, 'XZQH', '洛扎县','西藏自治区山南市洛扎县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540526', NULL, 'XZQH', '措美县','西藏自治区山南市措美县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540529', NULL, 'XZQH', '隆子县','西藏自治区山南市隆子县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540528', NULL, 'XZQH', '加查县','西藏自治区山南市加查县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441600', NULL, 'XZQH', '河源市','广东省河源市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441602', NULL, 'XZQH', '源城区','广东省河源市源城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340304', NULL, 'XZQH', '禹会区','安徽省蚌埠市禹会区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540531', NULL, 'XZQH', '浪卡子县','西藏自治区山南市浪卡子县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540530', NULL, 'XZQH', '错那县','西藏自治区山南市错那县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640422', NULL, 'XZQH', '西吉县','宁夏回族自治区固原市西吉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640423', NULL, 'XZQH', '隆德县','宁夏回族自治区固原市隆德县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640424', NULL, 'XZQH', '泾源县','宁夏回族自治区固原市泾源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340300', NULL, 'XZQH', '蚌埠市','安徽省蚌埠市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640425', NULL, 'XZQH', '彭阳县','宁夏回族自治区固原市彭阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340302', NULL, 'XZQH', '龙子湖区','安徽省蚌埠市龙子湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340303', NULL, 'XZQH', '蚌山区','安徽省蚌埠市蚌山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130681', NULL, 'XZQH', '涿州市','河北省保定市涿州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130683', NULL, 'XZQH', '安国市','河北省保定市安国市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130682', NULL, 'XZQH', '定州市','河北省保定市定州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130684', NULL, 'XZQH', '高碑店市','河北省保定市高碑店市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520303', NULL, 'XZQH', '汇川区','贵州省遵义市汇川区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520302', NULL, 'XZQH', '红花岗区','贵州省遵义市红花岗区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520300', NULL, 'XZQH', '遵义市','贵州省遵义市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520304', NULL, 'XZQH', '播州区','贵州省遵义市播州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411422', NULL, 'XZQH', '睢县','河南省商丘市睢县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411423', NULL, 'XZQH', '宁陵县','河南省商丘市宁陵县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411424', NULL, 'XZQH', '柘城县','河南省商丘市柘城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640402', NULL, 'XZQH', '原州区','宁夏回族自治区固原市原州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411425', NULL, 'XZQH', '虞城县','河南省商丘市虞城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411426', NULL, 'XZQH', '夏邑县','河南省商丘市夏邑县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640400', NULL, 'XZQH', '固原市','宁夏回族自治区固原市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('533423', NULL, 'XZQH', '维西傈僳族自治县','云南省迪庆藏族自治州维西傈僳族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710610', NULL, 'XZQH', '旗津区','台湾省高雄市旗津区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710605', NULL, 'XZQH', '盐埕区','台湾省高雄市盐埕区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330122', NULL, 'XZQH', '桐庐县','浙江省杭州市桐庐县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710604', NULL, 'XZQH', '三民区','台湾省高雄市三民区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710607', NULL, 'XZQH', '新兴区','台湾省高雄市新兴区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710606', NULL, 'XZQH', '前金区','台湾省高雄市前金区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710601', NULL, 'XZQH', '楠梓区','台湾省高雄市楠梓区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710600', NULL, 'XZQH', '高雄市','台湾省高雄市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710603', NULL, 'XZQH', '鼓山区','台湾省高雄市鼓山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710602', NULL, 'XZQH', '左营区','台湾省高雄市左营区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330127', NULL, 'XZQH', '淳安县','浙江省杭州市淳安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710609', NULL, 'XZQH', '前镇区','台湾省高雄市前镇区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710608', NULL, 'XZQH', '苓雅区','台湾省高雄市苓雅区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210381', NULL, 'XZQH', '海城市','辽宁省鞍山市海城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610902', NULL, 'XZQH', '汉滨区','陕西省安康市汉滨区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421281', NULL, 'XZQH', '赤壁市','湖北省咸宁市赤壁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610900', NULL, 'XZQH', '安康市','陕西省安康市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330112', NULL, 'XZQH', '临安区','浙江省杭州市临安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330111', NULL, 'XZQH', '富阳区','浙江省杭州市富阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330110', NULL, 'XZQH', '余杭区','浙江省杭州市余杭区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321324', NULL, 'XZQH', '泗洪县','江苏省宿迁市泗洪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411421', NULL, 'XZQH', '民权县','河南省商丘市民权县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321323', NULL, 'XZQH', '泗阳县','江苏省宿迁市泗阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321322', NULL, 'XZQH', '沭阳县','江苏省宿迁市沭阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140100', NULL, 'XZQH', '太原市','山西省太原市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140109', NULL, 'XZQH', '万柏林区','山西省太原市万柏林区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140108', NULL, 'XZQH', '尖草坪区','山西省太原市尖草坪区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140105', NULL, 'XZQH', '小店区','山西省太原市小店区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140107', NULL, 'XZQH', '杏花岭区','山西省太原市杏花岭区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140106', NULL, 'XZQH', '迎泽区','山西省太原市迎泽区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330104', NULL, 'XZQH', '江干区','浙江省杭州市江干区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330103', NULL, 'XZQH', '下城区','浙江省杭州市下城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330106', NULL, 'XZQH', '西湖区','浙江省杭州市西湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330105', NULL, 'XZQH', '拱墅区','浙江省杭州市拱墅区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330100', NULL, 'XZQH', '杭州市','浙江省杭州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330102', NULL, 'XZQH', '上城区','浙江省杭州市上城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321311', NULL, 'XZQH', '宿豫区','江苏省宿迁市宿豫区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330108', NULL, 'XZQH', '滨江区','浙江省杭州市滨江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330109', NULL, 'XZQH', '萧山区','浙江省杭州市萧山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610929', NULL, 'XZQH', '白河县','陕西省安康市白河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610928', NULL, 'XZQH', '旬阳县','陕西省安康市旬阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610927', NULL, 'XZQH', '镇坪县','陕西省安康市镇坪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610926', NULL, 'XZQH', '平利县','陕西省安康市平利县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150524', NULL, 'XZQH', '库伦旗','内蒙古自治区通辽市库伦旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610925', NULL, 'XZQH', '岚皋县','陕西省安康市岚皋县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150523', NULL, 'XZQH', '开鲁县','内蒙古自治区通辽市开鲁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610924', NULL, 'XZQH', '紫阳县','陕西省安康市紫阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150522', NULL, 'XZQH', '科尔沁左翼后旗','内蒙古自治区通辽市科尔沁左翼后旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610923', NULL, 'XZQH', '宁陕县','陕西省安康市宁陕县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150521', NULL, 'XZQH', '科尔沁左翼中旗','内蒙古自治区通辽市科尔沁左翼中旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610922', NULL, 'XZQH', '石泉县','陕西省安康市石泉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610921', NULL, 'XZQH', '汉阴县','陕西省安康市汉阴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150526', NULL, 'XZQH', '扎鲁特旗','内蒙古自治区通辽市扎鲁特旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150525', NULL, 'XZQH', '奈曼旗','内蒙古自治区通辽市奈曼旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321302', NULL, 'XZQH', '宿城区','江苏省宿迁市宿城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321300', NULL, 'XZQH', '宿迁市','江苏省宿迁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320600', NULL, 'XZQH', '南通市','江苏省南通市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320602', NULL, 'XZQH', '崇川区','江苏省南通市崇川区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140121', NULL, 'XZQH', '清徐县','山西省太原市清徐县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140122', NULL, 'XZQH', '阳曲县','山西省太原市阳曲县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140123', NULL, 'XZQH', '娄烦县','山西省太原市娄烦县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320612', NULL, 'XZQH', '通州区','江苏省南通市通州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320611', NULL, 'XZQH', '港闸区','江苏省南通市港闸区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210321', NULL, 'XZQH', '台安县','辽宁省鞍山市台安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210323', NULL, 'XZQH', '岫岩满族自治县','辽宁省鞍山市岫岩满族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230281', NULL, 'XZQH', '讷河市','黑龙江省齐齐哈尔市讷河市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140110', NULL, 'XZQH', '晋源区','山西省太原市晋源区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150581', NULL, 'XZQH', '霍林郭勒市','内蒙古自治区通辽市霍林郭勒市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650502', NULL, 'XZQH', '伊州区','新疆维吾尔自治区哈密市伊州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650500', NULL, 'XZQH', '哈密市','新疆维吾尔自治区哈密市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320621', NULL, 'XZQH', '海安县','江苏省南通市海安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320623', NULL, 'XZQH', '如东县','江苏省南通市如东县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710637', NULL, 'XZQH', '那玛夏区','台湾省高雄市那玛夏区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710635', NULL, 'XZQH', '茂林区','台湾省高雄市茂林区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710636', NULL, 'XZQH', '桃源区','台湾省高雄市桃源区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710633', NULL, 'XZQH', '甲仙区','台湾省高雄市甲仙区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710634', NULL, 'XZQH', '六龟区','台湾省高雄市六龟区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710630', NULL, 'XZQH', '美浓区','台湾省高雄市美浓区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710632', NULL, 'XZQH', '杉林区','台湾省高雄市杉林区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710631', NULL, 'XZQH', '内门区','台湾省高雄市内门区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411400', NULL, 'XZQH', '商丘市','河南省商丘市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411403', NULL, 'XZQH', '睢阳区','河南省商丘市睢阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411402', NULL, 'XZQH', '梁园区','河南省商丘市梁园区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210300', NULL, 'XZQH', '鞍山市','辽宁省鞍山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210302', NULL, 'XZQH', '铁东区','辽宁省鞍山市铁东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210303', NULL, 'XZQH', '铁西区','辽宁省鞍山市铁西区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210304', NULL, 'XZQH', '立山区','辽宁省鞍山市立山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710622', NULL, 'XZQH', '梓官区','台湾省高雄市梓官区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710623', NULL, 'XZQH', '弥陀区','台湾省高雄市弥陀区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710624', NULL, 'XZQH', '永安区','台湾省高雄市永安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710625', NULL, 'XZQH', '燕巢区','台湾省高雄市燕巢区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710626', NULL, 'XZQH', '阿莲区','台湾省高雄市阿莲区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('623001', NULL, 'XZQH', '合作市','甘肃省甘南藏族自治州合作市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710627', NULL, 'XZQH', '茄萣区','台湾省高雄市茄萣区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710628', NULL, 'XZQH', '湖内区','台湾省高雄市湖内区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710629', NULL, 'XZQH', '旗山区','台湾省高雄市旗山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('623000', NULL, 'XZQH', '甘南藏族自治州','甘肃省甘南藏族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710621', NULL, 'XZQH', '桥头区','台湾省高雄市桥头区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710620', NULL, 'XZQH', '路竹区','台湾省高雄市路竹区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650522', NULL, 'XZQH', '伊吾县','新疆维吾尔自治区哈密市伊吾县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650521', NULL, 'XZQH', '巴里坤哈萨克自治县','新疆维吾尔自治区哈密市巴里坤哈萨克自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654028', NULL, 'XZQH', '尼勒克县','新疆维吾尔自治区伊犁哈萨克自治州尼勒克县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('654027', NULL, 'XZQH', '特克斯县','新疆维吾尔自治区伊犁哈萨克自治州特克斯县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210311', NULL, 'XZQH', '千山区','辽宁省鞍山市千山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710619', NULL, 'XZQH', '冈山区','台湾省高雄市冈山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710613', NULL, 'XZQH', '大寮区','台湾省高雄市大寮区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710614', NULL, 'XZQH', '鸟松区','台湾省高雄市鸟松区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710611', NULL, 'XZQH', '小港区','台湾省高雄市小港区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710612', NULL, 'XZQH', '凤山区','台湾省高雄市凤山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710617', NULL, 'XZQH', '大树区','台湾省高雄市大树区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710618', NULL, 'XZQH', '大社区','台湾省高雄市大社区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710615', NULL, 'XZQH', '林园区','台湾省高雄市林园区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710616', NULL, 'XZQH', '仁武区','台湾省高雄市仁武区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230231', NULL, 'XZQH', '拜泉县','黑龙江省齐齐哈尔市拜泉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513435', NULL, 'XZQH', '甘洛县','四川省凉山彝族自治州甘洛县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370983', NULL, 'XZQH', '肥城市','山东省泰安市肥城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230230', NULL, 'XZQH', '克东县','黑龙江省齐齐哈尔市克东县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513434', NULL, 'XZQH', '越西县','四川省凉山彝族自治州越西县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513433', NULL, 'XZQH', '冕宁县','四川省凉山彝族自治州冕宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513432', NULL, 'XZQH', '喜德县','四川省凉山彝族自治州喜德县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513431', NULL, 'XZQH', '昭觉县','四川省凉山彝族自治州昭觉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513430', NULL, 'XZQH', '金阳县','四川省凉山彝族自治州金阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411381', NULL, 'XZQH', '邓州市','河南省南阳市邓州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370982', NULL, 'XZQH', '新泰市','山东省泰安市新泰市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513429', NULL, 'XZQH', '布拖县','四川省凉山彝族自治州布拖县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513427', NULL, 'XZQH', '宁南县','四川省凉山彝族自治州宁南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513428', NULL, 'XZQH', '普格县','四川省凉山彝族自治州普格县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421321', NULL, 'XZQH', '随县','湖北省随州市随县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513425', NULL, 'XZQH', '会理县','四川省凉山彝族自治州会理县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513426', NULL, 'XZQH', '会东县','四川省凉山彝族自治州会东县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440902', NULL, 'XZQH', '茂南区','广东省茂名市茂南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('623026', NULL, 'XZQH', '碌曲县','甘肃省甘南藏族自治州碌曲县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230221', NULL, 'XZQH', '龙江县','黑龙江省齐齐哈尔市龙江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('623025', NULL, 'XZQH', '玛曲县','甘肃省甘南藏族自治州玛曲县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440900', NULL, 'XZQH', '茂名市','广东省茂名市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610881', NULL, 'XZQH', '神木市','陕西省榆林市神木市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('623027', NULL, 'XZQH', '夏河县','甘肃省甘南藏族自治州夏河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('623022', NULL, 'XZQH', '卓尼县','甘肃省甘南藏族自治州卓尼县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230225', NULL, 'XZQH', '甘南县','黑龙江省齐齐哈尔市甘南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('623021', NULL, 'XZQH', '临潭县','甘肃省甘南藏族自治州临潭县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220781', NULL, 'XZQH', '扶余市','吉林省松原市扶余市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230224', NULL, 'XZQH', '泰来县','黑龙江省齐齐哈尔市泰来县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440904', NULL, 'XZQH', '电白区','广东省茂名市电白区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('623024', NULL, 'XZQH', '迭部县','甘肃省甘南藏族自治州迭部县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230223', NULL, 'XZQH', '依安县','黑龙江省齐齐哈尔市依安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('623023', NULL, 'XZQH', '舟曲县','甘肃省甘南藏族自治州舟曲县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230229', NULL, 'XZQH', '克山县','黑龙江省齐齐哈尔市克山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230227', NULL, 'XZQH', '富裕县','黑龙江省齐齐哈尔市富裕县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513436', NULL, 'XZQH', '美姑县','四川省凉山彝族自治州美姑县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513437', NULL, 'XZQH', '雷波县','四川省凉山彝族自治州雷波县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440104', NULL, 'XZQH', '越秀区','广东省广州市越秀区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440103', NULL, 'XZQH', '荔湾区','广东省广州市荔湾区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440106', NULL, 'XZQH', '天河区','广东省广州市天河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440105', NULL, 'XZQH', '海珠区','广东省广州市海珠区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410782', NULL, 'XZQH', '辉县市','河南省新乡市辉县市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440100', NULL, 'XZQH', '广州市','广东省广州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320684', NULL, 'XZQH', '海门市','江苏省南通市海门市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410781', NULL, 'XZQH', '卫辉市','河南省新乡市卫辉市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421303', NULL, 'XZQH', '曾都区','湖北省随州市曾都区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421300', NULL, 'XZQH', '随州市','湖北省随州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440118', NULL, 'XZQH', '增城区','广东省广州市增城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440117', NULL, 'XZQH', '从化区','广东省广州市从化区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440115', NULL, 'XZQH', '南沙区','广东省广州市南沙区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440114', NULL, 'XZQH', '花都区','广东省广州市花都区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440113', NULL, 'XZQH', '番禺区','广东省广州市番禺区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440112', NULL, 'XZQH', '黄埔区','广东省广州市黄埔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440111', NULL, 'XZQH', '白云区','广东省广州市白云区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510704', NULL, 'XZQH', '游仙区','四川省绵阳市游仙区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510703', NULL, 'XZQH', '涪城区','四川省绵阳市涪城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320682', NULL, 'XZQH', '如皋市','江苏省南通市如皋市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320681', NULL, 'XZQH', '启东市','江苏省南通市启东市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510700', NULL, 'XZQH', '绵阳市','四川省绵阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510923', NULL, 'XZQH', '大英县','四川省遂宁市大英县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510922', NULL, 'XZQH', '射洪县','四川省遂宁市射洪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510921', NULL, 'XZQH', '蓬溪县','四川省遂宁市蓬溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510705', NULL, 'XZQH', '安州区','四川省绵阳市安州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450206', NULL, 'XZQH', '柳江区','广西壮族自治区柳州市柳江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450205', NULL, 'XZQH', '柳北区','广西壮族自治区柳州市柳北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450202', NULL, 'XZQH', '城中区','广西壮族自治区柳州市城中区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450204', NULL, 'XZQH', '柳南区','广西壮族自治区柳州市柳南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450203', NULL, 'XZQH', '鱼峰区','广西壮族自治区柳州市鱼峰区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450200', NULL, 'XZQH', '柳州市','广西壮族自治区柳州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510900', NULL, 'XZQH', '遂宁市','四川省遂宁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510903', NULL, 'XZQH', '船山区','四川省遂宁市船山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510904', NULL, 'XZQH', '安居区','四川省遂宁市安居区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360800', NULL, 'XZQH', '吉安市','江西省吉安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360803', NULL, 'XZQH', '青原区','江西省吉安市青原区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360802', NULL, 'XZQH', '吉州区','江西省吉安市吉州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450222', NULL, 'XZQH', '柳城县','广西壮族自治区柳州市柳城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450224', NULL, 'XZQH', '融安县','广西壮族自治区柳州市融安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450223', NULL, 'XZQH', '鹿寨县','广西壮族自治区柳州市鹿寨县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450226', NULL, 'XZQH', '三江侗族自治县','广西壮族自治区柳州市三江侗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450225', NULL, 'XZQH', '融水苗族自治县','广西壮族自治区柳州市融水苗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230207', NULL, 'XZQH', '碾子山区','黑龙江省齐齐哈尔市碾子山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230208', NULL, 'XZQH', '梅里斯达斡尔族区','黑龙江省齐齐哈尔市梅里斯达斡尔族区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230205', NULL, 'XZQH', '昂昂溪区','黑龙江省齐齐哈尔市昂昂溪区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230206', NULL, 'XZQH', '富拉尔基区','黑龙江省齐齐哈尔市富拉尔基区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230203', NULL, 'XZQH', '建华区','黑龙江省齐齐哈尔市建华区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230204', NULL, 'XZQH', '铁锋区','黑龙江省齐齐哈尔市铁锋区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230202', NULL, 'XZQH', '龙沙区','黑龙江省齐齐哈尔市龙沙区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230200', NULL, 'XZQH', '齐齐哈尔市','黑龙江省齐齐哈尔市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130702', NULL, 'XZQH', '桥东区','河北省张家口市桥东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410711', NULL, 'XZQH', '牧野区','河南省新乡市牧野区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130700', NULL, 'XZQH', '张家口市','河北省张家口市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360828', NULL, 'XZQH', '万安县','江西省吉安市万安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130705', NULL, 'XZQH', '宣化区','河北省张家口市宣化区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360827', NULL, 'XZQH', '遂川县','江西省吉安市遂川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130706', NULL, 'XZQH', '下花园区','河北省张家口市下花园区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130703', NULL, 'XZQH', '桥西区','河北省张家口市桥西区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360829', NULL, 'XZQH', '安福县','江西省吉安市安福县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211000', NULL, 'XZQH', '辽阳市','辽宁省辽阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370902', NULL, 'XZQH', '泰山区','山东省泰安市泰山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430408', NULL, 'XZQH', '蒸湘区','湖南省衡阳市蒸湘区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430407', NULL, 'XZQH', '石鼓区','湖南省衡阳市石鼓区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370900', NULL, 'XZQH', '泰安市','山东省泰安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430406', NULL, 'XZQH', '雁峰区','湖南省衡阳市雁峰区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430405', NULL, 'XZQH', '珠晖区','湖南省衡阳市珠晖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360823', NULL, 'XZQH', '峡江县','江西省吉安市峡江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360824', NULL, 'XZQH', '新干县','江西省吉安市新干县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130709', NULL, 'XZQH', '崇礼区','河北省张家口市崇礼区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360825', NULL, 'XZQH', '永丰县','江西省吉安市永丰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130708', NULL, 'XZQH', '万全区','河北省张家口市万全区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360826', NULL, 'XZQH', '泰和县','江西省吉安市泰和县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430400', NULL, 'XZQH', '衡阳市','湖南省衡阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620102', NULL, 'XZQH', '城关区','甘肃省兰州市城关区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360821', NULL, 'XZQH', '吉安县','江西省吉安市吉安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620100', NULL, 'XZQH', '兰州市','甘肃省兰州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360822', NULL, 'XZQH', '吉水县','江西省吉安市吉水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410703', NULL, 'XZQH', '卫滨区','河南省新乡市卫滨区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410702', NULL, 'XZQH', '红旗区','河南省新乡市红旗区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410700', NULL, 'XZQH', '新乡市','河南省新乡市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211011', NULL, 'XZQH', '太子河区','辽宁省辽阳市太子河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420281', NULL, 'XZQH', '大冶市','湖北省黄石市大冶市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410704', NULL, 'XZQH', '凤泉区','河南省新乡市凤泉区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220700', NULL, 'XZQH', '松原市','吉林省松原市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371203', NULL, 'XZQH', '钢城区','山东省莱芜市钢城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371200', NULL, 'XZQH', '莱芜市','山东省莱芜市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211004', NULL, 'XZQH', '宏伟区','辽宁省辽阳市宏伟区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211005', NULL, 'XZQH', '弓长岭区','辽宁省辽阳市弓长岭区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430412', NULL, 'XZQH', '南岳区','湖南省衡阳市南岳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371202', NULL, 'XZQH', '莱城区','山东省莱芜市莱城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211002', NULL, 'XZQH', '白塔区','辽宁省辽阳市白塔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440982', NULL, 'XZQH', '化州市','广东省茂名市化州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211003', NULL, 'XZQH', '文圣区','辽宁省辽阳市文圣区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440981', NULL, 'XZQH', '高州市','广东省茂名市高州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520281', NULL, 'XZQH', '盘州市','贵州省六盘水市盘州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370000', NULL, 'XZQH', '山东省','山东省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220702', NULL, 'XZQH', '宁江区','吉林省松原市宁江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440983', NULL, 'XZQH', '信宜市','广东省茂名市信宜市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('211021', NULL, 'XZQH', '辽阳县','辽宁省辽阳市辽阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540502', NULL, 'XZQH', '乃东区','西藏自治区山南市乃东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540500', NULL, 'XZQH', '山南市','西藏自治区山南市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370923', NULL, 'XZQH', '东平县','山东省泰安市东平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370921', NULL, 'XZQH', '宁阳县','山东省泰安市宁阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430422', NULL, 'XZQH', '衡南县','湖南省衡阳市衡南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430421', NULL, 'XZQH', '衡阳县','湖南省衡阳市衡阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510781', NULL, 'XZQH', '江油市','四川省绵阳市江油市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430426', NULL, 'XZQH', '祁东县','湖南省衡阳市祁东县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430424', NULL, 'XZQH', '衡东县','湖南省衡阳市衡东县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430423', NULL, 'XZQH', '衡山县','湖南省衡阳市衡山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370911', NULL, 'XZQH', '岱岳区','山东省泰安市岱岳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220721', NULL, 'XZQH', '前郭尔罗斯蒙古族自治县','吉林省松原市前郭尔罗斯蒙古族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220722', NULL, 'XZQH', '长岭县','吉林省松原市长岭县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220723', NULL, 'XZQH', '乾安县','吉林省松原市乾安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360830', NULL, 'XZQH', '永新县','江西省吉安市永新县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652800', NULL, 'XZQH', '巴音郭楞蒙古自治州','新疆维吾尔自治区巴音郭楞蒙古自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652801', NULL, 'XZQH', '库尔勒市','新疆维吾尔自治区巴音郭楞蒙古自治州库尔勒市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420222', NULL, 'XZQH', '阳新县','湖北省黄石市阳新县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510727', NULL, 'XZQH', '平武县','四川省绵阳市平武县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('445381', NULL, 'XZQH', '罗定市','广东省云浮市罗定市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130731', NULL, 'XZQH', '涿鹿县','河北省张家口市涿鹿县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130730', NULL, 'XZQH', '怀来县','河北省张家口市怀来县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130732', NULL, 'XZQH', '赤城县','河北省张家口市赤城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510722', NULL, 'XZQH', '三台县','四川省绵阳市三台县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510723', NULL, 'XZQH', '盐亭县','四川省绵阳市盐亭县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510725', NULL, 'XZQH', '梓潼县','四川省绵阳市梓潼县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510726', NULL, 'XZQH', '北川羌族自治县','四川省绵阳市北川羌族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513401', NULL, 'XZQH', '西昌市','四川省凉山彝族自治州西昌市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513400', NULL, 'XZQH', '凉山彝族自治州','四川省凉山彝族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652826', NULL, 'XZQH', '焉耆回族自治县','新疆维吾尔自治区巴音郭楞蒙古自治州焉耆回族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652825', NULL, 'XZQH', '且末县','新疆维吾尔自治区巴音郭楞蒙古自治州且末县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652828', NULL, 'XZQH', '和硕县','新疆维吾尔自治区巴音郭楞蒙古自治州和硕县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652827', NULL, 'XZQH', '和静县','新疆维吾尔自治区巴音郭楞蒙古自治州和静县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652829', NULL, 'XZQH', '博湖县','新疆维吾尔自治区巴音郭楞蒙古自治州博湖县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130728', NULL, 'XZQH', '怀安县','河北省张家口市怀安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130727', NULL, 'XZQH', '阳原县','河北省张家口市阳原县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130726', NULL, 'XZQH', '蔚县','河北省张家口市蔚县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130725', NULL, 'XZQH', '尚义县','河北省张家口市尚义县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652822', NULL, 'XZQH', '轮台县','新疆维吾尔自治区巴音郭楞蒙古自治州轮台县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130724', NULL, 'XZQH', '沽源县','河北省张家口市沽源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130723', NULL, 'XZQH', '康保县','河北省张家口市康保县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652824', NULL, 'XZQH', '若羌县','新疆维吾尔自治区巴音郭楞蒙古自治州若羌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130722', NULL, 'XZQH', '张北县','河北省张家口市张北县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652823', NULL, 'XZQH', '尉犁县','新疆维吾尔自治区巴音郭楞蒙古自治州尉犁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620123', NULL, 'XZQH', '榆中县','甘肃省兰州市榆中县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620121', NULL, 'XZQH', '永登县','甘肃省兰州市永登县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620122', NULL, 'XZQH', '皋兰县','甘肃省兰州市皋兰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420204', NULL, 'XZQH', '下陆区','湖北省黄石市下陆区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420205', NULL, 'XZQH', '铁山区','湖北省黄石市铁山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420200', NULL, 'XZQH', '黄石市','湖北省黄石市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360881', NULL, 'XZQH', '井冈山市','江西省吉安市井冈山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420202', NULL, 'XZQH', '黄石港区','湖北省黄石市黄石港区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420203', NULL, 'XZQH', '西塞山区','湖北省黄石市西塞山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410728', NULL, 'XZQH', '长垣县','河南省新乡市长垣县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410726', NULL, 'XZQH', '延津县','河南省新乡市延津县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410727', NULL, 'XZQH', '封丘县','河南省新乡市封丘县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410721', NULL, 'XZQH', '新乡县','河南省新乡市新乡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410724', NULL, 'XZQH', '获嘉县','河南省新乡市获嘉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430482', NULL, 'XZQH', '常宁市','湖南省衡阳市常宁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620104', NULL, 'XZQH', '西固区','甘肃省兰州市西固区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410725', NULL, 'XZQH', '原阳县','河南省新乡市原阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430481', NULL, 'XZQH', '耒阳市','湖南省衡阳市耒阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620103', NULL, 'XZQH', '七里河区','甘肃省兰州市七里河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620105', NULL, 'XZQH', '安宁区','甘肃省兰州市安宁区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620111', NULL, 'XZQH', '红古区','甘肃省兰州市红古区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530181', NULL, 'XZQH', '安宁市','云南省昆明市安宁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513423', NULL, 'XZQH', '盐源县','四川省凉山彝族自治州盐源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513424', NULL, 'XZQH', '德昌县','四川省凉山彝族自治州德昌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('513422', NULL, 'XZQH', '木里藏族自治县','四川省凉山彝族自治州木里藏族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('659001', NULL, 'XZQH', '石河子市','新疆维吾尔自治区null石河子市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441521', NULL, 'XZQH', '海丰县','广东省汕尾市海丰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441523', NULL, 'XZQH', '陆河县','广东省汕尾市陆河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530381', NULL, 'XZQH', '宣威市','云南省曲靖市宣威市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520221', NULL, 'XZQH', '水城县','贵州省六盘水市水城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('659009', NULL, 'XZQH', '昆玉市','新疆维吾尔自治区null昆玉市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('659008', NULL, 'XZQH', '可克达拉市','新疆维吾尔自治区null可克达拉市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('659007', NULL, 'XZQH', '双河市','新疆维吾尔自治区null双河市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('659006', NULL, 'XZQH', '铁门关市','新疆维吾尔自治区null铁门关市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('659005', NULL, 'XZQH', '北屯市','新疆维吾尔自治区null北屯市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('659004', NULL, 'XZQH', '五家渠市','新疆维吾尔自治区null五家渠市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('659003', NULL, 'XZQH', '图木舒克市','新疆维吾尔自治区null图木舒克市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('659002', NULL, 'XZQH', '阿拉尔市','新疆维吾尔自治区null阿拉尔市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340222', NULL, 'XZQH', '繁昌县','安徽省芜湖市繁昌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340221', NULL, 'XZQH', '芜湖县','安徽省芜湖市芜湖县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340223', NULL, 'XZQH', '南陵县','安徽省芜湖市南陵县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340225', NULL, 'XZQH', '无为县','安徽省芜湖市无为县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710535', NULL, 'XZQH', '山上区','台湾省台南市山上区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710534', NULL, 'XZQH', '大内区','台湾省台南市大内区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710537', NULL, 'XZQH', '安定区','台湾省台南市安定区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710536', NULL, 'XZQH', '新市区','台湾省台南市新市区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360681', NULL, 'XZQH', '贵溪市','江西省鹰潭市贵溪市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652900', NULL, 'XZQH', '阿克苏地区','新疆维吾尔自治区阿克苏地区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652901', NULL, 'XZQH', '阿克苏市','新疆维吾尔自治区阿克苏地区阿克苏市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441500', NULL, 'XZQH', '汕尾市','广东省汕尾市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441502', NULL, 'XZQH', '城区','广东省汕尾市城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520200', NULL, 'XZQH', '六盘水市','贵州省六盘水市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350206', NULL, 'XZQH', '湖里区','福建省厦门市湖里区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350205', NULL, 'XZQH', '海沧区','福建省厦门市海沧区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350203', NULL, 'XZQH', '思明区','福建省厦门市思明区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350200', NULL, 'XZQH', '厦门市','福建省厦门市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('512022', NULL, 'XZQH', '乐至县','四川省资阳市乐至县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('512021', NULL, 'XZQH', '安岳县','四川省资阳市安岳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140981', NULL, 'XZQH', '原平市','山西省忻州市原平市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520203', NULL, 'XZQH', '六枝特区','贵州省六盘水市六枝特区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520201', NULL, 'XZQH', '钟山区','贵州省六盘水市钟山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430511', NULL, 'XZQH', '北塔区','湖南省邵阳市北塔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('445321', NULL, 'XZQH', '新兴县','广东省云浮市新兴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('445322', NULL, 'XZQH', '郁南县','广东省云浮市郁南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('512000', NULL, 'XZQH', '资阳市','四川省资阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('512002', NULL, 'XZQH', '雁江区','四川省资阳市雁江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430500', NULL, 'XZQH', '邵阳市','湖南省邵阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430502', NULL, 'XZQH', '双清区','湖南省邵阳市双清区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430503', NULL, 'XZQH', '大祥区','湖南省邵阳市大祥区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210281', NULL, 'XZQH', '瓦房店市','辽宁省大连市瓦房店市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210283', NULL, 'XZQH', '庄河市','辽宁省大连市庄河市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150600', NULL, 'XZQH', '鄂尔多斯市','内蒙古自治区鄂尔多斯市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540422', NULL, 'XZQH', '米林县','西藏自治区林芝市米林县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540421', NULL, 'XZQH', '工布江达县','西藏自治区林芝市工布江达县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150602', NULL, 'XZQH', '东胜区','内蒙古自治区鄂尔多斯市东胜区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540424', NULL, 'XZQH', '波密县','西藏自治区林芝市波密县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150603', NULL, 'XZQH', '康巴什区','内蒙古自治区鄂尔多斯市康巴什区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540423', NULL, 'XZQH', '墨脱县','西藏自治区林芝市墨脱县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540426', NULL, 'XZQH', '朗县','西藏自治区林芝市朗县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540425', NULL, 'XZQH', '察隅县','西藏自治区林芝市察隅县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370202', NULL, 'XZQH', '市南区','山东省青岛市市南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370203', NULL, 'XZQH', '市北区','山东省青岛市市北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370200', NULL, 'XZQH', '青岛市','山东省青岛市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340202', NULL, 'XZQH', '镜湖区','安徽省芜湖市镜湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340203', NULL, 'XZQH', '弋江区','安徽省芜湖市弋江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('445300', NULL, 'XZQH', '云浮市','广东省云浮市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340200', NULL, 'XZQH', '芜湖市','安徽省芜湖市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340207', NULL, 'XZQH', '鸠江区','安徽省芜湖市鸠江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340208', NULL, 'XZQH', '三山区','安徽省芜湖市三山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('445303', NULL, 'XZQH', '云安区','广东省云浮市云安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('445302', NULL, 'XZQH', '云城区','广东省云浮市云城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620300', NULL, 'XZQH', '金昌市','甘肃省金昌市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530323', NULL, 'XZQH', '师宗县','云南省曲靖市师宗县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530324', NULL, 'XZQH', '罗平县','云南省曲靖市罗平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530321', NULL, 'XZQH', '马龙县','云南省曲靖市马龙县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530322', NULL, 'XZQH', '陆良县','云南省曲靖市陆良县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530325', NULL, 'XZQH', '富源县','云南省曲靖市富源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530326', NULL, 'XZQH', '会泽县','云南省曲靖市会泽县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150624', NULL, 'XZQH', '鄂托克旗','内蒙古自治区鄂尔多斯市鄂托克旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540402', NULL, 'XZQH', '巴宜区','西藏自治区林芝市巴宜区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150625', NULL, 'XZQH', '杭锦旗','内蒙古自治区鄂尔多斯市杭锦旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150626', NULL, 'XZQH', '乌审旗','内蒙古自治区鄂尔多斯市乌审旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150627', NULL, 'XZQH', '伊金霍洛旗','内蒙古自治区鄂尔多斯市伊金霍洛旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150621', NULL, 'XZQH', '达拉特旗','内蒙古自治区鄂尔多斯市达拉特旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150622', NULL, 'XZQH', '准格尔旗','内蒙古自治区鄂尔多斯市准格尔旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540400', NULL, 'XZQH', '林芝市','西藏自治区林芝市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150623', NULL, 'XZQH', '鄂托克前旗','内蒙古自治区鄂尔多斯市鄂托克前旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640500', NULL, 'XZQH', '中卫市','宁夏回族自治区中卫市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411327', NULL, 'XZQH', '社旗县','河南省南阳市社旗县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411328', NULL, 'XZQH', '唐河县','河南省南阳市唐河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411329', NULL, 'XZQH', '新野县','河南省南阳市新野县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411323', NULL, 'XZQH', '西峡县','河南省南阳市西峡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411324', NULL, 'XZQH', '镇平县','河南省南阳市镇平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640502', NULL, 'XZQH', '沙坡头区','宁夏回族自治区中卫市沙坡头区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411325', NULL, 'XZQH', '内乡县','河南省南阳市内乡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411326', NULL, 'XZQH', '淅川县','河南省南阳市淅川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411330', NULL, 'XZQH', '桐柏县','河南省南阳市桐柏县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210224', NULL, 'XZQH', '长海县','辽宁省大连市长海县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610803', NULL, 'XZQH', '横山区','陕西省榆林市横山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('421381', NULL, 'XZQH', '广水市','湖北省随州市广水市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610802', NULL, 'XZQH', '榆阳区','陕西省榆林市榆阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610800', NULL, 'XZQH', '榆林市','陕西省榆林市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530303', NULL, 'XZQH', '沾益区','云南省曲靖市沾益区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530300', NULL, 'XZQH', '曲靖市','云南省曲靖市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530302', NULL, 'XZQH', '麒麟区','云南省曲靖市麒麟区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411322', NULL, 'XZQH', '方城县','河南省南阳市方城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411321', NULL, 'XZQH', '南召县','河南省南阳市南召县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320722', NULL, 'XZQH', '东海县','江苏省连云港市东海县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320723', NULL, 'XZQH', '灌云县','江苏省连云港市灌云县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320724', NULL, 'XZQH', '灌南县','江苏省连云港市灌南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('630100', NULL, 'XZQH', '西宁市','青海省西宁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411302', NULL, 'XZQH', '宛城区','河南省南阳市宛城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411303', NULL, 'XZQH', '卧龙区','河南省南阳市卧龙区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640522', NULL, 'XZQH', '海原县','宁夏回族自治区中卫市海原县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('640521', NULL, 'XZQH', '中宁县','宁夏回族自治区中卫市中宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330000', NULL, 'XZQH', '浙江省','浙江省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140902', NULL, 'XZQH', '忻府区','山西省忻州市忻府区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140900', NULL, 'XZQH', '忻州市','山西省忻州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650421', NULL, 'XZQH', '鄯善县','新疆维吾尔自治区吐鲁番市鄯善县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650422', NULL, 'XZQH', '托克逊县','新疆维吾尔自治区吐鲁番市托克逊县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320700', NULL, 'XZQH', '连云港市','江苏省连云港市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411300', NULL, 'XZQH', '南阳市','河南省南阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320706', NULL, 'XZQH', '海州区','江苏省连云港市海州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320707', NULL, 'XZQH', '赣榆区','江苏省连云港市赣榆区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320703', NULL, 'XZQH', '连云区','江苏省连云港市连云区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710511', NULL, 'XZQH', '玉井区','台湾省台南市玉井区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710510', NULL, 'XZQH', '左镇区','台湾省台南市左镇区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710501', NULL, 'XZQH', '中西区','台湾省台南市中西区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710502', NULL, 'XZQH', '东区','台湾省台南市东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710503', NULL, 'XZQH', '南区','台湾省台南市南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710504', NULL, 'XZQH', '北区','台湾省台南市北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710505', NULL, 'XZQH', '安平区','台湾省台南市安平区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710506', NULL, 'XZQH', '安南区','台湾省台南市安南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350213', NULL, 'XZQH', '翔安区','福建省厦门市翔安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710507', NULL, 'XZQH', '永康区','台湾省台南市永康区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350212', NULL, 'XZQH', '同安区','福建省厦门市同安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710508', NULL, 'XZQH', '归仁区','台湾省台南市归仁区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350211', NULL, 'XZQH', '集美区','福建省厦门市集美区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710509', NULL, 'XZQH', '新化区','台湾省台南市新化区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('810000', NULL, 'XZQH', '香港特别行政区','香港特别行政区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650402', NULL, 'XZQH', '高昌区','新疆维吾尔自治区吐鲁番市高昌区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650400', NULL, 'XZQH', '吐鲁番市','新疆维吾尔自治区吐鲁番市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140924', NULL, 'XZQH', '繁峙县','山西省忻州市繁峙县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140925', NULL, 'XZQH', '宁武县','山西省忻州市宁武县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140922', NULL, 'XZQH', '五台县','山西省忻州市五台县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140923', NULL, 'XZQH', '代县','山西省忻州市代县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710500', NULL, 'XZQH', '台南市','台湾省台南市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140921', NULL, 'XZQH', '定襄县','山西省忻州市定襄县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140929', NULL, 'XZQH', '岢岚县','山西省忻州市岢岚县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140928', NULL, 'XZQH', '五寨县','山西省忻州市五寨县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140927', NULL, 'XZQH', '神池县','山西省忻州市神池县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140926', NULL, 'XZQH', '静乐县','山西省忻州市静乐县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140000', NULL, 'XZQH', '山西省','山西省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710531', NULL, 'XZQH', '柳营区','台湾省台南市柳营区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610830', NULL, 'XZQH', '清涧县','陕西省榆林市清涧县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710530', NULL, 'XZQH', '下营区','台湾省台南市下营区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140930', NULL, 'XZQH', '河曲县','山西省忻州市河曲县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610831', NULL, 'XZQH', '子洲县','陕西省榆林市子洲县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710533', NULL, 'XZQH', '善化区','台湾省台南市善化区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140931', NULL, 'XZQH', '保德县','山西省忻州市保德县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710532', NULL, 'XZQH', '盐水区','台湾省台南市盐水区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140932', NULL, 'XZQH', '偏关县','山西省忻州市偏关县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710527', NULL, 'XZQH', '白河区','台湾省台南市白河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710528', NULL, 'XZQH', '东山区','台湾省台南市东山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710529', NULL, 'XZQH', '六甲区','台湾省台南市六甲区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710523', NULL, 'XZQH', '学甲区','台湾省台南市学甲区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710524', NULL, 'XZQH', '北门区','台湾省台南市北门区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710525', NULL, 'XZQH', '新营区','台湾省台南市新营区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710526', NULL, 'XZQH', '后壁区','台湾省台南市后壁区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210203', NULL, 'XZQH', '西岗区','辽宁省大连市西岗区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210204', NULL, 'XZQH', '沙河口区','辽宁省大连市沙河口区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210200', NULL, 'XZQH', '大连市','辽宁省大连市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210202', NULL, 'XZQH', '中山区','辽宁省大连市中山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610829', NULL, 'XZQH', '吴堡县','陕西省榆林市吴堡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610827', NULL, 'XZQH', '米脂县','陕西省榆林市米脂县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610828', NULL, 'XZQH', '佳县','陕西省榆林市佳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710522', NULL, 'XZQH', '将军区','台湾省台南市将军区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710521', NULL, 'XZQH', '七股区','台湾省台南市七股区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610822', NULL, 'XZQH', '府谷县','陕西省榆林市府谷县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710520', NULL, 'XZQH', '西港区','台湾省台南市西港区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610825', NULL, 'XZQH', '定边县','陕西省榆林市定边县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610826', NULL, 'XZQH', '绥德县','陕西省榆林市绥德县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610824', NULL, 'XZQH', '靖边县','陕西省榆林市靖边县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710518', NULL, 'XZQH', '麻豆区','台湾省台南市麻豆区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710519', NULL, 'XZQH', '佳里区','台湾省台南市佳里区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511400', NULL, 'XZQH', '眉山市','四川省眉山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610000', NULL, 'XZQH', '陕西省','陕西省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710516', NULL, 'XZQH', '龙崎区','台湾省台南市龙崎区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710517', NULL, 'XZQH', '官田区','台湾省台南市官田区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710514', NULL, 'XZQH', '仁德区','台湾省台南市仁德区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710515', NULL, 'XZQH', '关庙区','台湾省台南市关庙区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710512', NULL, 'XZQH', '楠西区','台湾省台南市楠西区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710513', NULL, 'XZQH', '南化区','台湾省台南市南化区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210214', NULL, 'XZQH', '普兰店区','辽宁省大连市普兰店区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210212', NULL, 'XZQH', '旅顺口区','辽宁省大连市旅顺口区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210213', NULL, 'XZQH', '金州区','辽宁省大连市金州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210211', NULL, 'XZQH', '甘井子区','辽宁省大连市甘井子区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450304', NULL, 'XZQH', '象山区','广西壮族自治区桂林市象山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411282', NULL, 'XZQH', '灵宝市','河南省三门峡市灵宝市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450305', NULL, 'XZQH', '七星区','广西壮族自治区桂林市七星区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451024', NULL, 'XZQH', '德保县','广西壮族自治区百色市德保县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451023', NULL, 'XZQH', '平果县','广西壮族自治区百色市平果县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451022', NULL, 'XZQH', '田东县','广西壮族自治区百色市田东县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511300', NULL, 'XZQH', '南充市','四川省南充市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450300', NULL, 'XZQH', '桂林市','广西壮族自治区桂林市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451029', NULL, 'XZQH', '田林县','广西壮族自治区百色市田林县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451028', NULL, 'XZQH', '乐业县','广西壮族自治区百色市乐业县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411281', NULL, 'XZQH', '义马市','河南省三门峡市义马市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450302', NULL, 'XZQH', '秀峰区','广西壮族自治区桂林市秀峰区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451027', NULL, 'XZQH', '凌云县','广西壮族自治区百色市凌云县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450303', NULL, 'XZQH', '叠彩区','广西壮族自治区桂林市叠彩区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451026', NULL, 'XZQH', '那坡县','广西壮族自治区百色市那坡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230381', NULL, 'XZQH', '虎林市','黑龙江省鸡西市虎林市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230382', NULL, 'XZQH', '密山市','黑龙江省鸡西市密山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451031', NULL, 'XZQH', '隆林各族自治县','广西壮族自治区百色市隆林各族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451030', NULL, 'XZQH', '西林县','广西壮族自治区百色市西林县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('469021', NULL, 'XZQH', '定安县','海南省null定安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('469022', NULL, 'XZQH', '屯昌县','海南省null屯昌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('469023', NULL, 'XZQH', '澄迈县','海南省null澄迈县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('469024', NULL, 'XZQH', '临高县','海南省null临高县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('469025', NULL, 'XZQH', '白沙黎族自治县','海南省null白沙黎族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('469026', NULL, 'XZQH', '昌江黎族自治县','海南省null昌江黎族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('469027', NULL, 'XZQH', '乐东黎族自治县','海南省null乐东黎族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('469028', NULL, 'XZQH', '陵水黎族自治县','海南省null陵水黎族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('469029', NULL, 'XZQH', '保亭黎族苗族自治县','海南省null保亭黎族苗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450311', NULL, 'XZQH', '雁山区','广西壮族自治区桂林市雁山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450312', NULL, 'XZQH', '临桂区','广西壮族自治区桂林市临桂区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510802', NULL, 'XZQH', '利州区','四川省广元市利州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510800', NULL, 'XZQH', '广元市','四川省广元市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('469030', NULL, 'XZQH', '琼中黎族苗族自治县','海南省null琼中黎族苗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451021', NULL, 'XZQH', '田阳县','广西壮族自治区百色市田阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420302', NULL, 'XZQH', '茅箭区','湖北省十堰市茅箭区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420300', NULL, 'XZQH', '十堰市','湖北省十堰市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511321', NULL, 'XZQH', '南部县','四川省南充市南部县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420304', NULL, 'XZQH', '郧阳区','湖北省十堰市郧阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511323', NULL, 'XZQH', '蓬安县','四川省南充市蓬安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420303', NULL, 'XZQH', '张湾区','湖北省十堰市张湾区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511322', NULL, 'XZQH', '营山县','四川省南充市营山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220681', NULL, 'XZQH', '临江市','吉林省白山市临江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510812', NULL, 'XZQH', '朝天区','四川省广元市朝天区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510811', NULL, 'XZQH', '昭化区','四川省广元市昭化区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510821', NULL, 'XZQH', '旺苍县','四川省广元市旺苍县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511304', NULL, 'XZQH', '嘉陵区','四川省南充市嘉陵区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511302', NULL, 'XZQH', '顺庆区','四川省南充市顺庆区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511303', NULL, 'XZQH', '高坪区','四川省南充市高坪区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510824', NULL, 'XZQH', '苍溪县','四川省广元市苍溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510823', NULL, 'XZQH', '剑阁县','四川省广元市剑阁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510822', NULL, 'XZQH', '青川县','四川省广元市青川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440811', NULL, 'XZQH', '麻章区','广东省湛江市麻章区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440804', NULL, 'XZQH', '坡头区','广东省湛江市坡头区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440802', NULL, 'XZQH', '赤坎区','广东省湛江市赤坎区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230321', NULL, 'XZQH', '鸡东县','黑龙江省鸡西市鸡东县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440803', NULL, 'XZQH', '霞山区','广东省湛江市霞山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440800', NULL, 'XZQH', '湛江市','广东省湛江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511325', NULL, 'XZQH', '西充县','四川省南充市西充县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511324', NULL, 'XZQH', '仪陇县','四川省南充市仪陇县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450323', NULL, 'XZQH', '灵川县','广西壮族自治区桂林市灵川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450325', NULL, 'XZQH', '兴安县','广西壮族自治区桂林市兴安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450324', NULL, 'XZQH', '全州县','广西壮族自治区桂林市全州县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451002', NULL, 'XZQH', '右江区','广西壮族自治区百色市右江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450321', NULL, 'XZQH', '阳朔县','广西壮族自治区桂林市阳朔县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451000', NULL, 'XZQH', '百色市','广西壮族自治区百色市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('630000', NULL, 'XZQH', '青海省','青海省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440000', NULL, 'XZQH', '广东省','广东省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411221', NULL, 'XZQH', '渑池县','河南省三门峡市渑池县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450327', NULL, 'XZQH', '灌阳县','广西壮族自治区桂林市灌阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450326', NULL, 'XZQH', '永福县','广西壮族自治区桂林市永福县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450329', NULL, 'XZQH', '资源县','广西壮族自治区桂林市资源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450328', NULL, 'XZQH', '龙胜各族自治县','广西壮族自治区桂林市龙胜各族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('442000', NULL, 'XZQH', '中山市','广东省中山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220623', NULL, 'XZQH', '长白朝鲜族自治县','吉林省白山市长白朝鲜族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220622', NULL, 'XZQH', '靖宇县','吉林省白山市靖宇县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440825', NULL, 'XZQH', '徐闻县','广东省湛江市徐闻县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220621', NULL, 'XZQH', '抚松县','吉林省白山市抚松县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440823', NULL, 'XZQH', '遂溪县','广东省湛江市遂溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230302', NULL, 'XZQH', '鸡冠区','黑龙江省鸡西市鸡冠区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450332', NULL, 'XZQH', '恭城瑶族自治县','广西壮族自治区桂林市恭城瑶族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230303', NULL, 'XZQH', '恒山区','黑龙江省鸡西市恒山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450331', NULL, 'XZQH', '荔浦县','广西壮族自治区桂林市荔浦县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230300', NULL, 'XZQH', '鸡西市','黑龙江省鸡西市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450330', NULL, 'XZQH', '平乐县','广西壮族自治区桂林市平乐县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230306', NULL, 'XZQH', '城子河区','黑龙江省鸡西市城子河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230307', NULL, 'XZQH', '麻山区','黑龙江省鸡西市麻山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230304', NULL, 'XZQH', '滴道区','黑龙江省鸡西市滴道区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230305', NULL, 'XZQH', '梨树区','黑龙江省鸡西市梨树区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411224', NULL, 'XZQH', '卢氏县','河南省三门峡市卢氏县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360729', NULL, 'XZQH', '全南县','江西省赣州市全南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130804', NULL, 'XZQH', '鹰手营子矿区','河北省承德市鹰手营子矿区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360728', NULL, 'XZQH', '定南县','江西省赣州市定南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130802', NULL, 'XZQH', '双桥区','河北省承德市双桥区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130803', NULL, 'XZQH', '双滦区','河北省承德市双滦区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130800', NULL, 'XZQH', '承德市','河北省承德市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511381', NULL, 'XZQH', '阆中市','四川省南充市阆中市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620200', NULL, 'XZQH', '嘉峪关市','甘肃省嘉峪关市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360722', NULL, 'XZQH', '信丰县','江西省赣州市信丰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360723', NULL, 'XZQH', '大余县','江西省赣州市大余县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360724', NULL, 'XZQH', '上犹县','江西省赣州市上犹县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360725', NULL, 'XZQH', '崇义县','江西省赣州市崇义县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360726', NULL, 'XZQH', '安远县','江西省赣州市安远县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360727', NULL, 'XZQH', '龙南县','江西省赣州市龙南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420381', NULL, 'XZQH', '丹江口市','湖北省十堰市丹江口市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370100', NULL, 'XZQH', '济南市','山东省济南市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220602', NULL, 'XZQH', '浑江区','吉林省白山市浑江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370102', NULL, 'XZQH', '历下区','山东省济南市历下区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220605', NULL, 'XZQH', '江源区','吉林省白山市江源区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520181', NULL, 'XZQH', '清镇市','贵州省贵阳市清镇市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370104', NULL, 'XZQH', '槐荫区','山东省济南市槐荫区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632222', NULL, 'XZQH', '祁连县','青海省海北藏族自治州祁连县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370103', NULL, 'XZQH', '市中区','山东省济南市市中区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632221', NULL, 'XZQH', '门源回族自治县','青海省海北藏族自治州门源回族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220600', NULL, 'XZQH', '白山市','吉林省白山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632224', NULL, 'XZQH', '刚察县','青海省海北藏族自治州刚察县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370105', NULL, 'XZQH', '天桥区','山东省济南市天桥区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632223', NULL, 'XZQH', '海晏县','青海省海北藏族自治州海晏县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130822', NULL, 'XZQH', '兴隆县','河北省承德市兴隆县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('460400', NULL, 'XZQH', '儋州市','海南省儋州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130821', NULL, 'XZQH', '承德县','河北省承德市承德县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130826', NULL, 'XZQH', '丰宁满族自治县','河北省承德市丰宁满族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441581', NULL, 'XZQH', '陆丰市','广东省汕尾市陆丰市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130827', NULL, 'XZQH', '宽城满族自治县','河北省承德市宽城满族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130824', NULL, 'XZQH', '滦平县','河北省承德市滦平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130825', NULL, 'XZQH', '隆化县','河北省承德市隆化县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360702', NULL, 'XZQH', '章贡区','江西省赣州市章贡区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430525', NULL, 'XZQH', '洞口县','湖南省邵阳市洞口县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360703', NULL, 'XZQH', '南康区','江西省赣州市南康区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430524', NULL, 'XZQH', '隆回县','湖南省邵阳市隆回县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360704', NULL, 'XZQH', '赣县区','江西省赣州市赣县区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430523', NULL, 'XZQH', '邵阳县','湖南省邵阳市邵阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130828', NULL, 'XZQH', '围场满族蒙古族自治县','河北省承德市围场满族蒙古族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430522', NULL, 'XZQH', '新邵县','湖南省邵阳市新邵县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370113', NULL, 'XZQH', '长清区','山东省济南市长清区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430521', NULL, 'XZQH', '邵东县','湖南省邵阳市邵东县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370112', NULL, 'XZQH', '历城区','山东省济南市历城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360700', NULL, 'XZQH', '赣州市','江西省赣州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370114', NULL, 'XZQH', '章丘区','山东省济南市章丘区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430529', NULL, 'XZQH', '城步苗族自治县','湖南省邵阳市城步苗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430528', NULL, 'XZQH', '新宁县','湖南省邵阳市新宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430527', NULL, 'XZQH', '绥宁县','湖南省邵阳市绥宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('445224', NULL, 'XZQH', '惠来县','广东省揭阳市惠来县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532532', NULL, 'XZQH', '河口瑶族自治县','云南省红河哈尼族彝族自治州河口瑶族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('445222', NULL, 'XZQH', '揭西县','广东省揭阳市揭西县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370124', NULL, 'XZQH', '平阴县','山东省济南市平阴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370126', NULL, 'XZQH', '商河县','山东省济南市商河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361000', NULL, 'XZQH', '抚州市','江西省抚州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370125', NULL, 'XZQH', '济阳县','山东省济南市济阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361002', NULL, 'XZQH', '临川区','江西省抚州市临川区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361003', NULL, 'XZQH', '东乡区','江西省抚州市东乡区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532527', NULL, 'XZQH', '泸西县','云南省红河哈尼族彝族自治州泸西县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532528', NULL, 'XZQH', '元阳县','云南省红河哈尼族彝族自治州元阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532525', NULL, 'XZQH', '石屏县','云南省红河哈尼族彝族自治州石屏县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410611', NULL, 'XZQH', '淇滨区','河南省鹤壁市淇滨区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532523', NULL, 'XZQH', '屏边苗族自治县','云南省红河哈尼族彝族自治州屏边苗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532524', NULL, 'XZQH', '建水县','云南省红河哈尼族彝族自治州建水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532529', NULL, 'XZQH', '红河县','云南省红河哈尼族彝族自治州红河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420325', NULL, 'XZQH', '房县','湖北省十堰市房县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420322', NULL, 'XZQH', '郧西县','湖北省十堰市郧西县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420323', NULL, 'XZQH', '竹山县','湖北省十堰市竹山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420324', NULL, 'XZQH', '竹溪县','湖北省十堰市竹溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532531', NULL, 'XZQH', '绿春县','云南省红河哈尼族彝族自治州绿春县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532530', NULL, 'XZQH', '金平苗族瑶族傣族自治县','云南省红河哈尼族彝族自治州金平苗族瑶族傣族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410600', NULL, 'XZQH', '鹤壁市','河南省鹤壁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510626', NULL, 'XZQH', '罗江县','四川省德阳市罗江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410603', NULL, 'XZQH', '山城区','河南省鹤壁市山城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410602', NULL, 'XZQH', '鹤山区','河南省鹤壁市鹤山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510623', NULL, 'XZQH', '中江县','四川省德阳市中江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361030', NULL, 'XZQH', '广昌县','江西省抚州市广昌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('445281', NULL, 'XZQH', '普宁市','广东省揭阳市普宁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361027', NULL, 'XZQH', '金溪县','江西省抚州市金溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361026', NULL, 'XZQH', '宜黄县','江西省抚州市宜黄县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361025', NULL, 'XZQH', '乐安县','江西省抚州市乐安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361024', NULL, 'XZQH', '崇仁县','江西省抚州市崇仁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361023', NULL, 'XZQH', '南丰县','江西省抚州市南丰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361022', NULL, 'XZQH', '黎川县','江西省抚州市黎川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361021', NULL, 'XZQH', '南城县','江西省抚州市南城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371100', NULL, 'XZQH', '日照市','山东省日照市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440881', NULL, 'XZQH', '廉江市','广东省湛江市廉江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371102', NULL, 'XZQH', '东港区','山东省日照市东港区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440882', NULL, 'XZQH', '雷州市','广东省湛江市雷州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371103', NULL, 'XZQH', '岚山区','山东省日照市岚山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('361028', NULL, 'XZQH', '资溪县','江西省抚州市资溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440883', NULL, 'XZQH', '吴川市','广东省湛江市吴川市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('469006', NULL, 'XZQH', '万宁市','海南省null万宁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532501', NULL, 'XZQH', '个旧市','云南省红河哈尼族彝族自治州个旧市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('469005', NULL, 'XZQH', '文昌市','海南省null文昌市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532502', NULL, 'XZQH', '开远市','云南省红河哈尼族彝族自治州开远市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652923', NULL, 'XZQH', '库车县','新疆维吾尔自治区阿克苏地区库车县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652922', NULL, 'XZQH', '温宿县','新疆维吾尔自治区阿克苏地区温宿县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532500', NULL, 'XZQH', '红河哈尼族彝族自治州','云南省红河哈尼族彝族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('469002', NULL, 'XZQH', '琼海市','海南省null琼海市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('469001', NULL, 'XZQH', '五指山市','海南省null五指山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532503', NULL, 'XZQH', '蒙自市','云南省红河哈尼族彝族自治州蒙自市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532504', NULL, 'XZQH', '弥勒市','云南省红河哈尼族彝族自治州弥勒市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652929', NULL, 'XZQH', '柯坪县','新疆维吾尔自治区阿克苏地区柯坪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652928', NULL, 'XZQH', '阿瓦提县','新疆维吾尔自治区阿克苏地区阿瓦提县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652925', NULL, 'XZQH', '新和县','新疆维吾尔自治区阿克苏地区新和县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652924', NULL, 'XZQH', '沙雅县','新疆维吾尔自治区阿克苏地区沙雅县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652927', NULL, 'XZQH', '乌什县','新疆维吾尔自治区阿克苏地区乌什县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652926', NULL, 'XZQH', '拜城县','新疆维吾尔自治区阿克苏地区拜城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430581', NULL, 'XZQH', '武冈市','湖南省邵阳市武冈市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520115', NULL, 'XZQH', '观山湖区','贵州省贵阳市观山湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510600', NULL, 'XZQH', '德阳市','四川省德阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520113', NULL, 'XZQH', '白云区','贵州省贵阳市白云区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410621', NULL, 'XZQH', '浚县','河南省鹤壁市浚县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410622', NULL, 'XZQH', '淇县','河南省鹤壁市淇县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510603', NULL, 'XZQH', '旌阳区','四川省德阳市旌阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('451081', NULL, 'XZQH', '靖西市','广西壮族自治区百色市靖西市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360730', NULL, 'XZQH', '宁都县','江西省赣州市宁都县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632200', NULL, 'XZQH', '海北藏族自治州','青海省海北藏族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371122', NULL, 'XZQH', '莒县','山东省日照市莒县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360735', NULL, 'XZQH', '石城县','江西省赣州市石城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360734', NULL, 'XZQH', '寻乌县','江西省赣州市寻乌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('469007', NULL, 'XZQH', '东方市','海南省null东方市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520122', NULL, 'XZQH', '息烽县','贵州省贵阳市息烽县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360733', NULL, 'XZQH', '会昌县','江西省赣州市会昌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520123', NULL, 'XZQH', '修文县','贵州省贵阳市修文县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360732', NULL, 'XZQH', '兴国县','江西省赣州市兴国县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371121', NULL, 'XZQH', '五莲县','山东省日照市五莲县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360731', NULL, 'XZQH', '于都县','江西省赣州市于都县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520121', NULL, 'XZQH', '开阳县','贵州省贵阳市开阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410902', NULL, 'XZQH', '华龙区','河南省濮阳市华龙区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410900', NULL, 'XZQH', '濮阳市','河南省濮阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('719005', NULL, 'XZQH', '南投县','台湾省null南投县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370321', NULL, 'XZQH', '桓台县','山东省淄博市桓台县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621022', NULL, 'XZQH', '环县','甘肃省庆阳市环县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('719006', NULL, 'XZQH', '嘉义县','台湾省null嘉义县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370322', NULL, 'XZQH', '高青县','山东省淄博市高青县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621021', NULL, 'XZQH', '庆城县','甘肃省庆阳市庆城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('719003', NULL, 'XZQH', '苗栗县','台湾省null苗栗县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('719004', NULL, 'XZQH', '彰化县','台湾省null彰化县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('719001', NULL, 'XZQH', '宜兰县','台湾省null宜兰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621026', NULL, 'XZQH', '宁县','甘肃省庆阳市宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('719002', NULL, 'XZQH', '新竹县','台湾省null新竹县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621025', NULL, 'XZQH', '正宁县','甘肃省庆阳市正宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621024', NULL, 'XZQH', '合水县','甘肃省庆阳市合水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410100', NULL, 'XZQH', '郑州市','河南省郑州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621023', NULL, 'XZQH', '华池县','甘肃省庆阳市华池县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('719009', NULL, 'XZQH', '台东县','台湾省null台东县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('719007', NULL, 'XZQH', '云林县','台湾省null云林县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370323', NULL, 'XZQH', '沂源县','山东省淄博市沂源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('719008', NULL, 'XZQH', '屏东县','台湾省null屏东县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520527', NULL, 'XZQH', '赫章县','贵州省毕节市赫章县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621027', NULL, 'XZQH', '镇原县','甘肃省庆阳市镇原县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320000', NULL, 'XZQH', '江苏省','江苏省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520526', NULL, 'XZQH', '威宁彝族回族苗族自治县','贵州省毕节市威宁彝族回族苗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520525', NULL, 'XZQH', '纳雍县','贵州省毕节市纳雍县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520524', NULL, 'XZQH', '织金县','贵州省毕节市织金县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520523', NULL, 'XZQH', '金沙县','贵州省毕节市金沙县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520522', NULL, 'XZQH', '黔西县','贵州省毕节市黔西县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520521', NULL, 'XZQH', '大方县','贵州省毕节市大方县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130430', NULL, 'XZQH', '邱县','河北省邯郸市邱县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('433101', NULL, 'XZQH', '吉首市','湖南省湘西土家族苗族自治州吉首市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('433100', NULL, 'XZQH', '湘西土家族苗族自治州','湖南省湘西土家族苗族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130434', NULL, 'XZQH', '魏县','河北省邯郸市魏县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130433', NULL, 'XZQH', '馆陶县','河北省邯郸市馆陶县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130432', NULL, 'XZQH', '广平县','河北省邯郸市广平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130431', NULL, 'XZQH', '鸡泽县','河北省邯郸市鸡泽县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130435', NULL, 'XZQH', '曲周县','河北省邯郸市曲周县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321112', NULL, 'XZQH', '丹徒区','江苏省镇江市丹徒区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621002', NULL, 'XZQH', '西峰区','甘肃省庆阳市西峰区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410122', NULL, 'XZQH', '中牟县','河南省郑州市中牟县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621000', NULL, 'XZQH', '庆阳市','甘肃省庆阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370300', NULL, 'XZQH', '淄博市','山东省淄博市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370303', NULL, 'XZQH', '张店区','山东省淄博市张店区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370304', NULL, 'XZQH', '博山区','山东省淄博市博山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370302', NULL, 'XZQH', '淄川区','山东省淄博市淄川区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370305', NULL, 'XZQH', '临淄区','山东省淄博市临淄区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370306', NULL, 'XZQH', '周村区','山东省淄博市周村区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150721', NULL, 'XZQH', '阿荣旗','内蒙古自治区呼伦贝尔市阿荣旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150722', NULL, 'XZQH', '莫力达瓦达斡尔族自治旗','内蒙古自治区呼伦贝尔市莫力达瓦达斡尔族自治旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540300', NULL, 'XZQH', '昌都市','西藏自治区昌都市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150723', NULL, 'XZQH', '鄂伦春自治旗','内蒙古自治区呼伦贝尔市鄂伦春自治旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150724', NULL, 'XZQH', '鄂温克族自治旗','内蒙古自治区呼伦贝尔市鄂温克族自治旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540302', NULL, 'XZQH', '卡若区','西藏自治区昌都市卡若区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150725', NULL, 'XZQH', '陈巴尔虎旗','内蒙古自治区呼伦贝尔市陈巴尔虎旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150726', NULL, 'XZQH', '新巴尔虎左旗','内蒙古自治区呼伦贝尔市新巴尔虎左旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410108', NULL, 'XZQH', '惠济区','河南省郑州市惠济区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140881', NULL, 'XZQH', '永济市','山西省运城市永济市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140882', NULL, 'XZQH', '河津市','山西省运城市河津市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410106', NULL, 'XZQH', '上街区','河南省郑州市上街区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410104', NULL, 'XZQH', '管城回族区','河南省郑州市管城回族区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410105', NULL, 'XZQH', '金水区','河南省郑州市金水区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410102', NULL, 'XZQH', '中原区','河南省郑州市中原区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321111', NULL, 'XZQH', '润州区','江苏省镇江市润州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410103', NULL, 'XZQH', '二七区','河南省郑州市二七区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('719010', NULL, 'XZQH', '花莲县','台湾省null花莲县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('719011', NULL, 'XZQH', '澎湖县','台湾省null澎湖县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321102', NULL, 'XZQH', '京口区','江苏省镇江市京口区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('719012', NULL, 'XZQH', '金门县','台湾省null金门县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('719013', NULL, 'XZQH', '连江县','台湾省null连江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330824', NULL, 'XZQH', '开化县','浙江省衢州市开化县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320800', NULL, 'XZQH', '淮安市','江苏省淮安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330825', NULL, 'XZQH', '龙游县','浙江省衢州市龙游县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330822', NULL, 'XZQH', '常山县','浙江省衢州市常山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320804', NULL, 'XZQH', '淮阴区','江苏省淮安市淮阴区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320803', NULL, 'XZQH', '淮安区','江苏省淮安市淮安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150727', NULL, 'XZQH', '新巴尔虎右旗','内蒙古自治区呼伦贝尔市新巴尔虎右旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321183', NULL, 'XZQH', '句容市','江苏省镇江市句容市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530821', NULL, 'XZQH', '宁洱哈尼族彝族自治县','云南省普洱市宁洱哈尼族彝族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321181', NULL, 'XZQH', '丹阳市','江苏省镇江市丹阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530823', NULL, 'XZQH', '景东彝族自治县','云南省普洱市景东彝族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321182', NULL, 'XZQH', '扬中市','江苏省镇江市扬中市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530822', NULL, 'XZQH', '墨江哈尼族自治县','云南省普洱市墨江哈尼族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530825', NULL, 'XZQH', '镇沅彝族哈尼族拉祜族自治县','云南省普洱市镇沅彝族哈尼族拉祜族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530824', NULL, 'XZQH', '景谷傣族彝族自治县','云南省普洱市景谷傣族彝族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530827', NULL, 'XZQH', '孟连傣族拉祜族佤族自治县','云南省普洱市孟连傣族拉祜族佤族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530826', NULL, 'XZQH', '江城哈尼族彝族自治县','云南省普洱市江城哈尼族彝族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530829', NULL, 'XZQH', '西盟佤族自治县','云南省普洱市西盟佤族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530828', NULL, 'XZQH', '澜沧拉祜族自治县','云南省普洱市澜沧拉祜族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420981', NULL, 'XZQH', '应城市','湖北省孝感市应城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420982', NULL, 'XZQH', '安陆市','湖北省孝感市安陆市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420984', NULL, 'XZQH', '汉川市','湖北省孝感市汉川市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530000', NULL, 'XZQH', '云南省','云南省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130424', NULL, 'XZQH', '成安县','河北省邯郸市成安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540328', NULL, 'XZQH', '芒康县','西藏自治区昌都市芒康县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130425', NULL, 'XZQH', '大名县','河北省邯郸市大名县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540329', NULL, 'XZQH', '洛隆县','西藏自治区昌都市洛隆县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130426', NULL, 'XZQH', '涉县','河北省邯郸市涉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130427', NULL, 'XZQH', '磁县','河北省邯郸市磁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150702', NULL, 'XZQH', '海拉尔区','内蒙古自治区呼伦贝尔市海拉尔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540324', NULL, 'XZQH', '丁青县','西藏自治区昌都市丁青县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540325', NULL, 'XZQH', '察雅县','西藏自治区昌都市察雅县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540326', NULL, 'XZQH', '八宿县','西藏自治区昌都市八宿县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130423', NULL, 'XZQH', '临漳县','河北省邯郸市临漳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150703', NULL, 'XZQH', '扎赉诺尔区','内蒙古自治区呼伦贝尔市扎赉诺尔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540327', NULL, 'XZQH', '左贡县','西藏自治区昌都市左贡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540321', NULL, 'XZQH', '江达县','西藏自治区昌都市江达县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150700', NULL, 'XZQH', '呼伦贝尔市','内蒙古自治区呼伦贝尔市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540322', NULL, 'XZQH', '贡觉县','西藏自治区昌都市贡觉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540323', NULL, 'XZQH', '类乌齐县','西藏自治区昌都市类乌齐县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540330', NULL, 'XZQH', '边坝县','西藏自治区昌都市边坝县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330803', NULL, 'XZQH', '衢江区','浙江省衢州市衢江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330802', NULL, 'XZQH', '柯城区','浙江省衢州市柯城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330800', NULL, 'XZQH', '衢州市','浙江省衢州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530800', NULL, 'XZQH', '普洱市','云南省普洱市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530802', NULL, 'XZQH', '思茅区','云南省普洱市思茅区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140830', NULL, 'XZQH', '芮城县','山西省运城市芮城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710427', NULL, 'XZQH', '龙井区','台湾省台中市龙井区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710426', NULL, 'XZQH', '大肚区','台湾省台中市大肚区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710425', NULL, 'XZQH', '乌日区','台湾省台中市乌日区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710424', NULL, 'XZQH', '大安区','台湾省台中市大安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710429', NULL, 'XZQH', '和平区','台湾省台中市和平区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710428', NULL, 'XZQH', '雾峰区','台湾省台中市雾峰区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441400', NULL, 'XZQH', '梅州市','广东省梅州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441402', NULL, 'XZQH', '梅江区','广东省梅州市梅江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441403', NULL, 'XZQH', '梅县区','广东省梅州市梅县区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130402', NULL, 'XZQH', '邯山区','河北省邯郸市邯山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130403', NULL, 'XZQH', '丛台区','河北省邯郸市丛台区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130404', NULL, 'XZQH', '复兴区','河北省邯郸市复兴区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130406', NULL, 'XZQH', '峰峰矿区','河北省邯郸市峰峰矿区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130407', NULL, 'XZQH', '肥乡区','河北省邯郸市肥乡区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130408', NULL, 'XZQH', '永年区','河北省邯郸市永年区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710420', NULL, 'XZQH', '大雅区','台湾省台中市大雅区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710421', NULL, 'XZQH', '新小区','台湾省台中市新小区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710422', NULL, 'XZQH', '石冈区','台湾省台中市石冈区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130400', NULL, 'XZQH', '邯郸市','河北省邯郸市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710423', NULL, 'XZQH', '外埔区','台湾省台中市外埔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710414', NULL, 'XZQH', '清水区','台湾省台中市清水区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710413', NULL, 'XZQH', '大甲区','台湾省台中市大甲区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710416', NULL, 'XZQH', '梧栖区','台湾省台中市梧栖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710415', NULL, 'XZQH', '沙鹿区','台湾省台中市沙鹿区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710418', NULL, 'XZQH', '神冈区','台湾省台中市神冈区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710417', NULL, 'XZQH', '后里区','台湾省台中市后里区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710419', NULL, 'XZQH', '潭子区','台湾省台中市潭子区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110000', NULL, 'XZQH', '北京市','北京市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710406', NULL, 'XZQH', '西屯区','台湾省台中市西屯区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710407', NULL, 'XZQH', '南屯区','台湾省台中市南屯区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710408', NULL, 'XZQH', '北屯区','台湾省台中市北屯区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710409', NULL, 'XZQH', '丰原区','台湾省台中市丰原区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710402', NULL, 'XZQH', '东区','台湾省台中市东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350111', NULL, 'XZQH', '晋安区','福建省福州市晋安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710403', NULL, 'XZQH', '西区','台湾省台中市西区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710404', NULL, 'XZQH', '南区','台湾省台中市南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710405', NULL, 'XZQH', '北区','台湾省台中市北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710410', NULL, 'XZQH', '大里区','台湾省台中市大里区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710412', NULL, 'XZQH', '东势区','台湾省台中市东势区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710411', NULL, 'XZQH', '太平区','台湾省台中市太平区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511500', NULL, 'XZQH', '宜宾市','四川省宜宾市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511503', NULL, 'XZQH', '南溪区','四川省宜宾市南溪区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511502', NULL, 'XZQH', '翠屏区','四川省宜宾市翠屏区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431023', NULL, 'XZQH', '永兴县','湖南省郴州市永兴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431022', NULL, 'XZQH', '宜章县','湖南省郴州市宜章县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431021', NULL, 'XZQH', '桂阳县','湖南省郴州市桂阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431024', NULL, 'XZQH', '嘉禾县','湖南省郴州市嘉禾县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431025', NULL, 'XZQH', '临武县','湖南省郴州市临武县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431026', NULL, 'XZQH', '汝城县','湖南省郴州市汝城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431027', NULL, 'XZQH', '桂东县','湖南省郴州市桂东县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350128', NULL, 'XZQH', '平潭县','福建省福州市平潭县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431028', NULL, 'XZQH', '安仁县','湖南省郴州市安仁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350124', NULL, 'XZQH', '闽清县','福建省福州市闽清县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350123', NULL, 'XZQH', '罗源县','福建省福州市罗源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140829', NULL, 'XZQH', '平陆县','山西省运城市平陆县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511521', NULL, 'XZQH', '宜宾县','四川省宜宾市宜宾县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140828', NULL, 'XZQH', '夏县','山西省运城市夏县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350125', NULL, 'XZQH', '永泰县','福建省福州市永泰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140827', NULL, 'XZQH', '垣曲县','山西省运城市垣曲县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350122', NULL, 'XZQH', '连江县','福建省福州市连江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350121', NULL, 'XZQH', '闽侯县','福建省福州市闽侯县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710401', NULL, 'XZQH', '中区','台湾省台中市中区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140821', NULL, 'XZQH', '临猗县','山西省运城市临猗县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610700', NULL, 'XZQH', '汉中市','陕西省汉中市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710400', NULL, 'XZQH', '台中市','台湾省台中市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140822', NULL, 'XZQH', '万荣县','山西省运城市万荣县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140825', NULL, 'XZQH', '新绛县','山西省运城市新绛县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140826', NULL, 'XZQH', '绛县','山西省运城市绛县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140823', NULL, 'XZQH', '闻喜县','山西省运城市闻喜县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610702', NULL, 'XZQH', '汉台区','陕西省汉中市汉台区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140824', NULL, 'XZQH', '稷山县','山西省运城市稷山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440310', NULL, 'XZQH', '坪山区','广东省深圳市坪山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350105', NULL, 'XZQH', '马尾区','福建省福州市马尾区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440300', NULL, 'XZQH', '深圳市','广东省深圳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440303', NULL, 'XZQH', '罗湖区','广东省深圳市罗湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440304', NULL, 'XZQH', '福田区','广东省深圳市福田区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440307', NULL, 'XZQH', '龙岗区','广东省深圳市龙岗区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610730', NULL, 'XZQH', '佛坪县','陕西省汉中市佛坪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440308', NULL, 'XZQH', '盐田区','广东省深圳市盐田区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350100', NULL, 'XZQH', '福州市','福建省福州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440305', NULL, 'XZQH', '南山区','广东省深圳市南山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440306', NULL, 'XZQH', '宝安区','广东省深圳市宝安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350102', NULL, 'XZQH', '鼓楼区','福建省福州市鼓楼区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350104', NULL, 'XZQH', '仓山区','福建省福州市仓山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440309', NULL, 'XZQH', '龙华区','广东省深圳市龙华区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350103', NULL, 'XZQH', '台江区','福建省福州市台江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610726', NULL, 'XZQH', '宁强县','陕西省汉中市宁强县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610727', NULL, 'XZQH', '略阳县','陕西省汉中市略阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330382', NULL, 'XZQH', '乐清市','浙江省温州市乐清市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610724', NULL, 'XZQH', '西乡县','陕西省汉中市西乡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330381', NULL, 'XZQH', '瑞安市','浙江省温州市瑞安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140802', NULL, 'XZQH', '盐湖区','山西省运城市盐湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610725', NULL, 'XZQH', '勉县','陕西省汉中市勉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610722', NULL, 'XZQH', '城固县','陕西省汉中市城固县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140800', NULL, 'XZQH', '运城市','山西省运城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610723', NULL, 'XZQH', '洋县','陕西省汉中市洋县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610721', NULL, 'XZQH', '南郑区','陕西省汉中市南郑区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610728', NULL, 'XZQH', '镇巴县','陕西省汉中市镇巴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610729', NULL, 'XZQH', '留坝县','陕西省汉中市留坝县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320812', NULL, 'XZQH', '清江浦区','江苏省淮安市清江浦区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320813', NULL, 'XZQH', '洪泽区','江苏省淮安市洪泽区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340523', NULL, 'XZQH', '和县','安徽省马鞍山市和县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340522', NULL, 'XZQH', '含山县','安徽省马鞍山市含山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340521', NULL, 'XZQH', '当涂县','安徽省马鞍山市当涂县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321100', NULL, 'XZQH', '镇江市','江苏省镇江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330881', NULL, 'XZQH', '江山市','浙江省衢州市江山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320826', NULL, 'XZQH', '涟水县','江苏省淮安市涟水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150784', NULL, 'XZQH', '额尔古纳市','内蒙古自治区呼伦贝尔市额尔古纳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150783', NULL, 'XZQH', '扎兰屯市','内蒙古自治区呼伦贝尔市扎兰屯市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150782', NULL, 'XZQH', '牙克石市','内蒙古自治区呼伦贝尔市牙克石市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150781', NULL, 'XZQH', '满洲里市','内蒙古自治区呼伦贝尔市满洲里市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150785', NULL, 'XZQH', '根河市','内蒙古自治区呼伦贝尔市根河市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130481', NULL, 'XZQH', '武安市','河北省邯郸市武安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320830', NULL, 'XZQH', '盱眙县','江苏省淮安市盱眙县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320831', NULL, 'XZQH', '金湖县','江苏省淮安市金湖县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431000', NULL, 'XZQH', '郴州市','湖南省郴州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411202', NULL, 'XZQH', '湖滨区','河南省三门峡市湖滨区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340500', NULL, 'XZQH', '马鞍山市','安徽省马鞍山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411203', NULL, 'XZQH', '陕州区','河南省三门峡市陕州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340504', NULL, 'XZQH', '雨山区','安徽省马鞍山市雨山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340503', NULL, 'XZQH', '花山区','安徽省马鞍山市花山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340506', NULL, 'XZQH', '博望区','安徽省马鞍山市博望区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411200', NULL, 'XZQH', '三门峡市','河南省三门峡市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431003', NULL, 'XZQH', '苏仙区','湖南省郴州市苏仙区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('431002', NULL, 'XZQH', '北湖区','湖南省郴州市北湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520500', NULL, 'XZQH', '毕节市','贵州省毕节市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520502', NULL, 'XZQH', '七星关区','贵州省毕节市七星关区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500119', NULL, 'XZQH', '南川区','重庆市重庆市南川区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500116', NULL, 'XZQH', '江津区','重庆市重庆市江津区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500115', NULL, 'XZQH', '长寿区','重庆市重庆市长寿区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500118', NULL, 'XZQH', '永川区','重庆市重庆市永川区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500117', NULL, 'XZQH', '合川区','重庆市重庆市合川区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411100', NULL, 'XZQH', '漯河市','河南省漯河市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411102', NULL, 'XZQH', '源汇区','河南省漯河市源汇区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500120', NULL, 'XZQH', '璧山区','重庆市重庆市璧山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632802', NULL, 'XZQH', '德令哈市','青海省海西蒙古族藏族自治州德令哈市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632801', NULL, 'XZQH', '格尔木市','青海省海西蒙古族藏族自治州格尔木市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632800', NULL, 'XZQH', '海西蒙古族藏族自治州','青海省海西蒙古族藏族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310000', NULL, 'XZQH', '上海市','上海市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411104', NULL, 'XZQH', '召陵区','河南省漯河市召陵区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411103', NULL, 'XZQH', '郾城区','河南省漯河市郾城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330329', NULL, 'XZQH', '泰顺县','浙江省温州市泰顺县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330327', NULL, 'XZQH', '苍南县','浙江省温州市苍南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330328', NULL, 'XZQH', '文成县','浙江省温州市文成县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220503', NULL, 'XZQH', '二道江区','吉林省通化市二道江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220502', NULL, 'XZQH', '东昌区','吉林省通化市东昌区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330326', NULL, 'XZQH', '平阳县','浙江省温州市平阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220500', NULL, 'XZQH', '通化市','吉林省通化市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330324', NULL, 'XZQH', '永嘉县','浙江省温州市永嘉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350982', NULL, 'XZQH', '福鼎市','福建省宁德市福鼎市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350981', NULL, 'XZQH', '福安市','福建省宁德市福安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230421', NULL, 'XZQH', '萝北县','黑龙江省鹤岗市萝北县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230422', NULL, 'XZQH', '绥滨县','黑龙江省鹤岗市绥滨县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371082', NULL, 'XZQH', '荣成市','山东省威海市荣成市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371083', NULL, 'XZQH', '乳山市','山东省威海市乳山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411121', NULL, 'XZQH', '舞阳县','河南省漯河市舞阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411122', NULL, 'XZQH', '临颍县','河南省漯河市临颍县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632822', NULL, 'XZQH', '都兰县','青海省海西蒙古族藏族自治州都兰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350181', NULL, 'XZQH', '福清市','福建省福州市福清市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632821', NULL, 'XZQH', '乌兰县','青海省海西蒙古族藏族自治州乌兰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350182', NULL, 'XZQH', '长乐区','福建省福州市长乐区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450481', NULL, 'XZQH', '岑溪市','广西壮族自治区梧州市岑溪市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('632823', NULL, 'XZQH', '天峻县','青海省海西蒙古族藏族自治州天峻县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330305', NULL, 'XZQH', '洞头区','浙江省温州市洞头区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500153', NULL, 'XZQH', '荣昌区','重庆市重庆市荣昌区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330300', NULL, 'XZQH', '温州市','浙江省温州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500154', NULL, 'XZQH', '开州区','重庆市重庆市开州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500151', NULL, 'XZQH', '铜梁区','重庆市重庆市铜梁区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500152', NULL, 'XZQH', '潼南区','重庆市重庆市潼南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330303', NULL, 'XZQH', '龙湾区','浙江省温州市龙湾区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220524', NULL, 'XZQH', '柳河县','吉林省通化市柳河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330304', NULL, 'XZQH', '瓯海区','浙江省温州市瓯海区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220523', NULL, 'XZQH', '辉南县','吉林省通化市辉南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500155', NULL, 'XZQH', '梁平区','重庆市重庆市梁平区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330302', NULL, 'XZQH', '鹿城区','浙江省温州市鹿城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220521', NULL, 'XZQH', '通化县','吉林省通化市通化县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500156', NULL, 'XZQH', '武隆区','重庆市重庆市武隆区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620821', NULL, 'XZQH', '泾川县','甘肃省平凉市泾川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620822', NULL, 'XZQH', '灵台县','甘肃省平凉市灵台县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610629', NULL, 'XZQH', '洛川县','陕西省延安市洛川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620823', NULL, 'XZQH', '崇信县','甘肃省平凉市崇信县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620824', NULL, 'XZQH', '华亭县','甘肃省平凉市华亭县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620825', NULL, 'XZQH', '庄浪县','甘肃省平凉市庄浪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231081', NULL, 'XZQH', '绥芬河市','黑龙江省牡丹江市绥芬河市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610626', NULL, 'XZQH', '吴起县','陕西省延安市吴起县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620826', NULL, 'XZQH', '静宁县','甘肃省平凉市静宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610625', NULL, 'XZQH', '志丹县','陕西省延安市志丹县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231083', NULL, 'XZQH', '海林市','黑龙江省牡丹江市海林市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610628', NULL, 'XZQH', '富县','陕西省延安市富县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610627', NULL, 'XZQH', '甘泉县','陕西省延安市甘泉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231085', NULL, 'XZQH', '穆棱市','黑龙江省牡丹江市穆棱市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610622', NULL, 'XZQH', '延川县','陕西省延安市延川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231084', NULL, 'XZQH', '宁安市','黑龙江省牡丹江市宁安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610621', NULL, 'XZQH', '延长县','陕西省延安市延长县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231086', NULL, 'XZQH', '东宁市','黑龙江省牡丹江市东宁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610623', NULL, 'XZQH', '子长县','陕西省延安市子长县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450400', NULL, 'XZQH', '梧州市','广西壮族自治区梧州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610630', NULL, 'XZQH', '宜川县','陕西省延安市宜川县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610631', NULL, 'XZQH', '黄龙县','陕西省延安市黄龙县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450403', NULL, 'XZQH', '万秀区','广西壮族自治区梧州市万秀区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450405', NULL, 'XZQH', '长洲区','广西壮族自治区梧州市长洲区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450406', NULL, 'XZQH', '龙圩区','广西壮族自治区梧州市龙圩区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511425', NULL, 'XZQH', '青神县','四川省眉山市青神县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511423', NULL, 'XZQH', '洪雅县','四川省眉山市洪雅县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511424', NULL, 'XZQH', '丹棱县','四川省眉山市丹棱县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610632', NULL, 'XZQH', '黄陵县','陕西省延安市黄陵县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210521', NULL, 'XZQH', '本溪满族自治县','辽宁省本溪市本溪满族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210522', NULL, 'XZQH', '桓仁满族自治县','辽宁省本溪市桓仁满族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653225', NULL, 'XZQH', '策勒县','新疆维吾尔自治区和田地区策勒县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653224', NULL, 'XZQH', '洛浦县','新疆维吾尔自治区和田地区洛浦县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653227', NULL, 'XZQH', '民丰县','新疆维吾尔自治区和田地区民丰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653226', NULL, 'XZQH', '于田县','新疆维吾尔自治区和田地区于田县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220582', NULL, 'XZQH', '集安市','吉林省通化市集安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220581', NULL, 'XZQH', '梅河口市','吉林省通化市梅河口市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511421', NULL, 'XZQH', '仁寿县','四川省眉山市仁寿县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450421', NULL, 'XZQH', '苍梧县','广西壮族自治区梧州市苍梧县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450422', NULL, 'XZQH', '藤县','广西壮族自治区梧州市藤县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450423', NULL, 'XZQH', '蒙山县','广西壮族自治区梧州市蒙山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430281', NULL, 'XZQH', '醴陵市','湖南省株洲市醴陵市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511403', NULL, 'XZQH', '彭山区','四川省眉山市彭山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511402', NULL, 'XZQH', '东坡区','四川省眉山市东坡区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360000', NULL, 'XZQH', '江西省','江西省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210505', NULL, 'XZQH', '南芬区','辽宁省本溪市南芬区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210504', NULL, 'XZQH', '明山区','辽宁省本溪市明山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210503', NULL, 'XZQH', '溪湖区','辽宁省本溪市溪湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210502', NULL, 'XZQH', '平山区','辽宁省本溪市平山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210500', NULL, 'XZQH', '本溪市','辽宁省本溪市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653222', NULL, 'XZQH', '墨玉县','新疆维吾尔自治区和田地区墨玉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653223', NULL, 'XZQH', '皮山县','新疆维吾尔自治区和田地区皮山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('653221', NULL, 'XZQH', '和田县','新疆维吾尔自治区和田地区和田县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430203', NULL, 'XZQH', '芦淞区','湖南省株洲市芦淞区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430204', NULL, 'XZQH', '石峰区','湖南省株洲市石峰区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430202', NULL, 'XZQH', '荷塘区','湖南省株洲市荷塘区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430200', NULL, 'XZQH', '株洲市','湖南省株洲市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152923', NULL, 'XZQH', '额济纳旗','内蒙古自治区阿拉善盟额济纳旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130500', NULL, 'XZQH', '邢台市','河北省邢台市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152921', NULL, 'XZQH', '阿拉善左旗','内蒙古自治区阿拉善盟阿拉善左旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152922', NULL, 'XZQH', '阿拉善右旗','内蒙古自治区阿拉善盟阿拉善右旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130503', NULL, 'XZQH', '桥西区','河北省邢台市桥西区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130502', NULL, 'XZQH', '桥东区','河北省邢台市桥东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370281', NULL, 'XZQH', '胶州市','山东省青岛市胶州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370282', NULL, 'XZQH', '即墨区','山东省青岛市即墨区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441422', NULL, 'XZQH', '大埔县','广东省梅州市大埔县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371002', NULL, 'XZQH', '环翠区','山东省威海市环翠区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370283', NULL, 'XZQH', '平度市','山东省青岛市平度市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371003', NULL, 'XZQH', '文登区','山东省威海市文登区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370285', NULL, 'XZQH', '莱西市','山东省青岛市莱西市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441426', NULL, 'XZQH', '平远县','广东省梅州市平远县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430211', NULL, 'XZQH', '天元区','湖南省株洲市天元区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441424', NULL, 'XZQH', '五华县','广东省梅州市五华县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('371000', NULL, 'XZQH', '威海市','山东省威海市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441423', NULL, 'XZQH', '丰顺县','广东省梅州市丰顺县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441427', NULL, 'XZQH', '蕉岭县','广东省梅州市蕉岭县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420000', NULL, 'XZQH', '湖北省','湖北省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430223', NULL, 'XZQH', '攸县','湖南省株洲市攸县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430224', NULL, 'XZQH', '茶陵县','湖南省株洲市茶陵县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430221', NULL, 'XZQH', '株洲县','湖南省株洲市株洲县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430225', NULL, 'XZQH', '炎陵县','湖南省株洲市炎陵县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('152900', NULL, 'XZQH', '阿拉善盟','内蒙古自治区阿拉善盟', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620802', NULL, 'XZQH', '崆峒区','甘肃省平凉市崆峒区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620800', NULL, 'XZQH', '平凉市','甘肃省平凉市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420923', NULL, 'XZQH', '云梦县','湖北省孝感市云梦县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420922', NULL, 'XZQH', '大悟县','湖北省孝感市大悟县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420921', NULL, 'XZQH', '孝昌县','湖北省孝感市孝昌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321081', NULL, 'XZQH', '仪征市','江苏省扬州市仪征市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510503', NULL, 'XZQH', '纳溪区','四川省泸州市纳溪区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510504', NULL, 'XZQH', '龙马潭区','四川省泸州市龙马潭区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510500', NULL, 'XZQH', '泸州市','四川省泸州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321084', NULL, 'XZQH', '高邮市','江苏省扬州市高邮市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510502', NULL, 'XZQH', '江阳区','四川省泸州市江阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542500', NULL, 'XZQH', '阿里地区','西藏自治区阿里地区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231000', NULL, 'XZQH', '牡丹江市','黑龙江省牡丹江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231002', NULL, 'XZQH', '东安区','黑龙江省牡丹江市东安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231003', NULL, 'XZQH', '阳明区','黑龙江省牡丹江市阳明区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231004', NULL, 'XZQH', '爱民区','黑龙江省牡丹江市爱民区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231005', NULL, 'XZQH', '西安区','黑龙江省牡丹江市西安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420900', NULL, 'XZQH', '孝感市','湖北省孝感市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420902', NULL, 'XZQH', '孝南区','湖北省孝感市孝南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510522', NULL, 'XZQH', '合江县','四川省泸州市合江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510521', NULL, 'XZQH', '泸县','四川省泸州市泸县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510524', NULL, 'XZQH', '叙永县','四川省泸州市叙永县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510525', NULL, 'XZQH', '古蔺县','四川省泸州市古蔺县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500103', NULL, 'XZQH', '渝中区','重庆市重庆市渝中区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500102', NULL, 'XZQH', '涪陵区','重庆市重庆市涪陵区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370214', NULL, 'XZQH', '城阳区','山东省青岛市城阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500101', NULL, 'XZQH', '万州区','重庆市重庆市万州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370213', NULL, 'XZQH', '李沧区','山东省青岛市李沧区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500100', NULL, 'XZQH', '重庆市','重庆市重庆市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370212', NULL, 'XZQH', '崂山区','山东省青岛市崂山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370211', NULL, 'XZQH', '黄岛区','山东省青岛市黄岛区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441481', NULL, 'XZQH', '兴宁市','广东省梅州市兴宁市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542521', NULL, 'XZQH', '普兰县','西藏自治区阿里地区普兰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542522', NULL, 'XZQH', '札达县','西藏自治区阿里地区札达县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542523', NULL, 'XZQH', '噶尔县','西藏自治区阿里地区噶尔县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542524', NULL, 'XZQH', '日土县','西藏自治区阿里地区日土县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542525', NULL, 'XZQH', '革吉县','西藏自治区阿里地区革吉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542526', NULL, 'XZQH', '改则县','西藏自治区阿里地区改则县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542527', NULL, 'XZQH', '措勤县','西藏自治区阿里地区措勤县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230400', NULL, 'XZQH', '鹤岗市','黑龙江省鹤岗市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230402', NULL, 'XZQH', '向阳区','黑龙江省鹤岗市向阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500110', NULL, 'XZQH', '綦江区','重庆市重庆市綦江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500112', NULL, 'XZQH', '渝北区','重庆市重庆市渝北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500111', NULL, 'XZQH', '大足区','重庆市重庆市大足区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500114', NULL, 'XZQH', '黔江区','重庆市重庆市黔江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('231025', NULL, 'XZQH', '林口县','黑龙江省牡丹江市林口县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500113', NULL, 'XZQH', '巴南区','重庆市重庆市巴南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230407', NULL, 'XZQH', '兴山区','黑龙江省鹤岗市兴山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230404', NULL, 'XZQH', '南山区','黑龙江省鹤岗市南山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230403', NULL, 'XZQH', '工农区','黑龙江省鹤岗市工农区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230406', NULL, 'XZQH', '东山区','黑龙江省鹤岗市东山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230405', NULL, 'XZQH', '兴安区','黑龙江省鹤岗市兴安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410923', NULL, 'XZQH', '南乐县','河南省濮阳市南乐县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410922', NULL, 'XZQH', '清丰县','河南省濮阳市清丰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410927', NULL, 'XZQH', '台前县','河南省濮阳市台前县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500104', NULL, 'XZQH', '大渡口区','重庆市重庆市大渡口区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410926', NULL, 'XZQH', '范县','河南省濮阳市范县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500105', NULL, 'XZQH', '江北区','重庆市重庆市江北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500106', NULL, 'XZQH', '沙坪坝区','重庆市重庆市沙坪坝区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500107', NULL, 'XZQH', '九龙坡区','重庆市重庆市九龙坡区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500108', NULL, 'XZQH', '南岸区','重庆市重庆市南岸区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500109', NULL, 'XZQH', '北碚区','重庆市重庆市北碚区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410928', NULL, 'XZQH', '濮阳县','河南省濮阳市濮阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340421', NULL, 'XZQH', '凤台县','安徽省淮南市凤台县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340422', NULL, 'XZQH', '寿县','安徽省淮南市寿县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410000', NULL, 'XZQH', '河南省','河南省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621125', NULL, 'XZQH', '漳县','甘肃省定西市漳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320100', NULL, 'XZQH', '南京市','江苏省南京市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621124', NULL, 'XZQH', '临洮县','甘肃省定西市临洮县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530100', NULL, 'XZQH', '昆明市','云南省昆明市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621123', NULL, 'XZQH', '渭源县','甘肃省定西市渭源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320102', NULL, 'XZQH', '玄武区','江苏省南京市玄武区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621122', NULL, 'XZQH', '陇西县','甘肃省定西市陇西县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320105', NULL, 'XZQH', '建邺区','江苏省南京市建邺区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621121', NULL, 'XZQH', '通渭县','甘肃省定西市通渭县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320104', NULL, 'XZQH', '秦淮区','江苏省南京市秦淮区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320106', NULL, 'XZQH', '鼓楼区','江苏省南京市鼓楼区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520424', NULL, 'XZQH', '关岭布依族苗族自治县','贵州省安顺市关岭布依族苗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520423', NULL, 'XZQH', '镇宁布依族苗族自治县','贵州省安顺市镇宁布依族苗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520422', NULL, 'XZQH', '普定县','贵州省安顺市普定县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530103', NULL, 'XZQH', '盘龙区','云南省昆明市盘龙区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621126', NULL, 'XZQH', '岷县','甘肃省定西市岷县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520425', NULL, 'XZQH', '紫云苗族布依族自治县','贵州省安顺市紫云苗族布依族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530102', NULL, 'XZQH', '五华区','云南省昆明市五华区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320925', NULL, 'XZQH', '建湖县','江苏省盐城市建湖县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320924', NULL, 'XZQH', '射阳县','江苏省盐城市射阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320923', NULL, 'XZQH', '阜宁县','江苏省盐城市阜宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320922', NULL, 'XZQH', '滨海县','江苏省盐城市滨海县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530111', NULL, 'XZQH', '官渡区','云南省昆明市官渡区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141081', NULL, 'XZQH', '侯马市','山西省临汾市侯马市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320921', NULL, 'XZQH', '响水县','江苏省盐城市响水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141082', NULL, 'XZQH', '霍州市','山西省临汾市霍州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652301', NULL, 'XZQH', '昌吉市','新疆维吾尔自治区昌吉回族自治州昌吉市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652300', NULL, 'XZQH', '昌吉回族自治州','新疆维吾尔自治区昌吉回族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652302', NULL, 'XZQH', '阜康市','新疆维吾尔自治区昌吉回族自治州阜康市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150981', NULL, 'XZQH', '丰镇市','内蒙古自治区乌兰察布市丰镇市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530900', NULL, 'XZQH', '临沧市','云南省临沧市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530902', NULL, 'XZQH', '临翔区','云南省临沧市临翔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140781', NULL, 'XZQH', '介休市','山西省晋中市介休市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520403', NULL, 'XZQH', '平坝区','贵州省安顺市平坝区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520402', NULL, 'XZQH', '西秀区','贵州省安顺市西秀区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520400', NULL, 'XZQH', '安顺市','贵州省安顺市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130582', NULL, 'XZQH', '沙河市','河北省邢台市沙河市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130581', NULL, 'XZQH', '南宫市','河北省邢台市南宫市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320904', NULL, 'XZQH', '大丰区','江苏省盐城市大丰区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320903', NULL, 'XZQH', '盐都区','江苏省盐城市盐都区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320902', NULL, 'XZQH', '亭湖区','江苏省盐城市亭湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320900', NULL, 'XZQH', '盐城市','江苏省盐城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320118', NULL, 'XZQH', '高淳区','江苏省南京市高淳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320117', NULL, 'XZQH', '溧水区','江苏省南京市溧水区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320116', NULL, 'XZQH', '六合区','江苏省南京市六合区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510322', NULL, 'XZQH', '富顺县','四川省自贡市富顺县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320115', NULL, 'XZQH', '江宁区','江苏省南京市江宁区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510321', NULL, 'XZQH', '荣县','四川省自贡市荣县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320114', NULL, 'XZQH', '雨花台区','江苏省南京市雨花台区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320113', NULL, 'XZQH', '栖霞区','江苏省南京市栖霞区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320111', NULL, 'XZQH', '浦口区','江苏省南京市浦口区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150924', NULL, 'XZQH', '兴和县','内蒙古自治区乌兰察布市兴和县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150923', NULL, 'XZQH', '商都县','内蒙古自治区乌兰察布市商都县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530927', NULL, 'XZQH', '沧源佤族自治县','云南省临沧市沧源佤族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141029', NULL, 'XZQH', '乡宁县','山西省临汾市乡宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150922', NULL, 'XZQH', '化德县','内蒙古自治区乌兰察布市化德县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141028', NULL, 'XZQH', '吉县','山西省临汾市吉县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150921', NULL, 'XZQH', '卓资县','内蒙古自治区乌兰察布市卓资县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710312', NULL, 'XZQH', '新屋区','台湾省桃园市新屋区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530924', NULL, 'XZQH', '镇康县','云南省临沧市镇康县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710313', NULL, 'XZQH', '复兴区','台湾省桃园市复兴区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530923', NULL, 'XZQH', '永德县','云南省临沧市永德县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710310', NULL, 'XZQH', '大园区','台湾省桃园市大园区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530926', NULL, 'XZQH', '耿马傣族佤族自治县','云南省临沧市耿马傣族佤族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710311', NULL, 'XZQH', '观音区','台湾省桃园市观音区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530925', NULL, 'XZQH', '双江拉祜族佤族布朗族傣族自治县','云南省临沧市双江拉祜族佤族布朗族傣族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321282', NULL, 'XZQH', '靖江市','江苏省泰州市靖江市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141023', NULL, 'XZQH', '襄汾县','山西省临汾市襄汾县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652327', NULL, 'XZQH', '吉木萨尔县','新疆维吾尔自治区昌吉回族自治州吉木萨尔县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321283', NULL, 'XZQH', '泰兴市','江苏省泰州市泰兴市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141022', NULL, 'XZQH', '翼城县','山西省临汾市翼城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652328', NULL, 'XZQH', '木垒哈萨克自治县','新疆维吾尔自治区昌吉回族自治州木垒哈萨克自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141021', NULL, 'XZQH', '曲沃县','山西省临汾市曲沃县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530922', NULL, 'XZQH', '云县','云南省临沧市云县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321281', NULL, 'XZQH', '兴化市','江苏省泰州市兴化市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530921', NULL, 'XZQH', '凤庆县','云南省临沧市凤庆县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141027', NULL, 'XZQH', '浮山县','山西省临汾市浮山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652323', NULL, 'XZQH', '呼图壁县','新疆维吾尔自治区昌吉回族自治州呼图壁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141026', NULL, 'XZQH', '安泽县','山西省临汾市安泽县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652324', NULL, 'XZQH', '玛纳斯县','新疆维吾尔自治区昌吉回族自治州玛纳斯县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141025', NULL, 'XZQH', '古县','山西省临汾市古县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('652325', NULL, 'XZQH', '奇台县','新疆维吾尔自治区昌吉回族自治州奇台县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141024', NULL, 'XZQH', '洪洞县','山西省临汾市洪洞县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110114', NULL, 'XZQH', '昌平区','北京市北京市昌平区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110115', NULL, 'XZQH', '大兴区','北京市北京市大兴区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110116', NULL, 'XZQH', '怀柔区','北京市北京市怀柔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110117', NULL, 'XZQH', '平谷区','北京市北京市平谷区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141030', NULL, 'XZQH', '大宁县','山西省临汾市大宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110111', NULL, 'XZQH', '房山区','北京市北京市房山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110112', NULL, 'XZQH', '通州区','北京市北京市通州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110113', NULL, 'XZQH', '顺义区','北京市北京市顺义区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710306', NULL, 'XZQH', '芦竹区','台湾省桃园市芦竹区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710305', NULL, 'XZQH', '杨梅区','台湾省桃园市杨梅区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710304', NULL, 'XZQH', '八德区','台湾省桃园市八德区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150929', NULL, 'XZQH', '四子王旗','内蒙古自治区乌兰察布市四子王旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710303', NULL, 'XZQH', '平镇区','台湾省桃园市平镇区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110118', NULL, 'XZQH', '密云区','北京市北京市密云区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150927', NULL, 'XZQH', '察哈尔右翼中旗','内蒙古自治区乌兰察布市察哈尔右翼中旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710309', NULL, 'XZQH', '龟山区','台湾省桃园市龟山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110119', NULL, 'XZQH', '延庆区','北京市北京市延庆区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150928', NULL, 'XZQH', '察哈尔右翼后旗','内蒙古自治区乌兰察布市察哈尔右翼后旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710308', NULL, 'XZQH', '龙潭区','台湾省桃园市龙潭区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150925', NULL, 'XZQH', '凉城县','内蒙古自治区乌兰察布市凉城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710307', NULL, 'XZQH', '大溪区','台湾省桃园市大溪区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150926', NULL, 'XZQH', '察哈尔右翼前旗','内蒙古自治区乌兰察布市察哈尔右翼前旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140725', NULL, 'XZQH', '寿阳县','山西省晋中市寿阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540221', NULL, 'XZQH', '南木林县','西藏自治区日喀则市南木林县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140724', NULL, 'XZQH', '昔阳县','山西省晋中市昔阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540222', NULL, 'XZQH', '江孜县','西藏自治区日喀则市江孜县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140727', NULL, 'XZQH', '祁县','山西省晋中市祁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540223', NULL, 'XZQH', '定日县','西藏自治区日喀则市定日县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140726', NULL, 'XZQH', '太谷县','山西省晋中市太谷县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540224', NULL, 'XZQH', '萨迦县','西藏自治区日喀则市萨迦县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140721', NULL, 'XZQH', '榆社县','山西省晋中市榆社县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540225', NULL, 'XZQH', '达孜区','西藏自治区日喀则市达孜区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710300', NULL, 'XZQH', '桃园市','台湾省桃园市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150800', NULL, 'XZQH', '巴彦淖尔市','内蒙古自治区巴彦淖尔市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540226', NULL, 'XZQH', '昂仁县','西藏自治区日喀则市昂仁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710301', NULL, 'XZQH', '桃园区','台湾省桃园市桃园区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140723', NULL, 'XZQH', '和顺县','山西省晋中市和顺县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130521', NULL, 'XZQH', '邢台县','河北省邢台市邢台县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540227', NULL, 'XZQH', '谢通门县','西藏自治区日喀则市谢通门县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('710302', NULL, 'XZQH', '中坜区','台湾省桃园市中坜区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140722', NULL, 'XZQH', '左权县','山西省晋中市左权县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130522', NULL, 'XZQH', '临城县','河北省邢台市临城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150802', NULL, 'XZQH', '临河区','内蒙古自治区巴彦淖尔市临河区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540228', NULL, 'XZQH', '白朗县','西藏自治区日喀则市白朗县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141032', NULL, 'XZQH', '永和县','山西省临汾市永和县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130523', NULL, 'XZQH', '内丘县','河北省邢台市内丘县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540229', NULL, 'XZQH', '仁布县','西藏自治区日喀则市仁布县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141031', NULL, 'XZQH', '隰县','山西省临汾市隰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130524', NULL, 'XZQH', '柏乡县','河北省邢台市柏乡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141034', NULL, 'XZQH', '汾西县','山西省临汾市汾西县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130525', NULL, 'XZQH', '隆尧县','河北省邢台市隆尧县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141033', NULL, 'XZQH', '蒲县','山西省临汾市蒲县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130526', NULL, 'XZQH', '任县','河北省邢台市任县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130527', NULL, 'XZQH', '南和县','河北省邢台市南和县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130528', NULL, 'XZQH', '宁晋县','河北省邢台市宁晋县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130529', NULL, 'XZQH', '巨鹿县','河北省邢台市巨鹿县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330903', NULL, 'XZQH', '普陀区','浙江省舟山市普陀区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330902', NULL, 'XZQH', '定海区','浙江省舟山市定海区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330900', NULL, 'XZQH', '舟山市','浙江省舟山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140728', NULL, 'XZQH', '平遥县','山西省晋中市平遥县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140729', NULL, 'XZQH', '灵石县','山西省晋中市灵石县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540231', NULL, 'XZQH', '定结县','西藏自治区日喀则市定结县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540230', NULL, 'XZQH', '康马县','西藏自治区日喀则市康马县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130532', NULL, 'XZQH', '平乡县','河北省邢台市平乡县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130533', NULL, 'XZQH', '威县','河北省邢台市威县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130530', NULL, 'XZQH', '新河县','河北省邢台市新河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130531', NULL, 'XZQH', '广宗县','河北省邢台市广宗县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530113', NULL, 'XZQH', '东川区','云南省昆明市东川区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530112', NULL, 'XZQH', '西山区','云南省昆明市西山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530115', NULL, 'XZQH', '晋宁区','云南省昆明市晋宁区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141002', NULL, 'XZQH', '尧都区','山西省临汾市尧都区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530114', NULL, 'XZQH', '呈贡区','云南省昆明市呈贡区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('141000', NULL, 'XZQH', '临汾市','山西省临汾市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130534', NULL, 'XZQH', '清河县','河北省邢台市清河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130535', NULL, 'XZQH', '临西县','河北省邢台市临西县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370406', NULL, 'XZQH', '山亭区','山东省枣庄市山亭区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370405', NULL, 'XZQH', '台儿庄区','山东省枣庄市台儿庄区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370404', NULL, 'XZQH', '峄城区','山东省枣庄市峄城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370403', NULL, 'XZQH', '薛城区','山东省枣庄市薛城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370402', NULL, 'XZQH', '市中区','山东省枣庄市市中区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('370400', NULL, 'XZQH', '枣庄市','山东省枣庄市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150823', NULL, 'XZQH', '乌拉特前旗','内蒙古自治区巴彦淖尔市乌拉特前旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420881', NULL, 'XZQH', '钟祥市','湖北省荆门市钟祥市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150822', NULL, 'XZQH', '磴口县','内蒙古自治区巴彦淖尔市磴口县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150825', NULL, 'XZQH', '乌拉特后旗','内蒙古自治区巴彦淖尔市乌拉特后旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150824', NULL, 'XZQH', '乌拉特中旗','内蒙古自治区巴彦淖尔市乌拉特中旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540200', NULL, 'XZQH', '日喀则市','西藏自治区日喀则市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150821', NULL, 'XZQH', '五原县','内蒙古自治区巴彦淖尔市五原县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540202', NULL, 'XZQH', '桑珠孜区','西藏自治区日喀则市桑珠孜区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530126', NULL, 'XZQH', '石林彝族自治县','云南省昆明市石林彝族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530125', NULL, 'XZQH', '宜良县','云南省昆明市宜良县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530124', NULL, 'XZQH', '富民县','云南省昆明市富民县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530129', NULL, 'XZQH', '寻甸回族彝族自治县','云南省昆明市寻甸回族彝族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530128', NULL, 'XZQH', '禄劝彝族苗族自治县','云南省昆明市禄劝彝族苗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('530127', NULL, 'XZQH', '嵩明县','云南省昆明市嵩明县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330922', NULL, 'XZQH', '嵊泗县','浙江省舟山市嵊泗县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330921', NULL, 'XZQH', '岱山县','浙江省舟山市岱山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150826', NULL, 'XZQH', '杭锦后旗','内蒙古自治区巴彦淖尔市杭锦后旗', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350922', NULL, 'XZQH', '古田县','福建省宁德市古田县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350921', NULL, 'XZQH', '霞浦县','福建省宁德市霞浦县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210421', NULL, 'XZQH', '抚顺县','辽宁省抚顺市抚顺县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210422', NULL, 'XZQH', '新宾满族自治县','辽宁省抚顺市新宾满族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350926', NULL, 'XZQH', '柘荣县','福建省宁德市柘荣县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210423', NULL, 'XZQH', '清原满族自治县','辽宁省抚顺市清原满族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350925', NULL, 'XZQH', '周宁县','福建省宁德市周宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350924', NULL, 'XZQH', '寿宁县','福建省宁德市寿宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350923', NULL, 'XZQH', '屏南县','福建省宁德市屏南县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341024', NULL, 'XZQH', '祁门县','安徽省黄山市祁门县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341023', NULL, 'XZQH', '黟县','安徽省黄山市黟县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341022', NULL, 'XZQH', '休宁县','安徽省黄山市休宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511600', NULL, 'XZQH', '广安市','四川省广安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511602', NULL, 'XZQH', '广安区','四川省广安市广安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511603', NULL, 'XZQH', '前锋区','四川省广安市前锋区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610603', NULL, 'XZQH', '安塞区','陕西省延安市安塞区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610602', NULL, 'XZQH', '宝塔区','陕西省延安市宝塔区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610600', NULL, 'XZQH', '延安市','陕西省延安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350902', NULL, 'XZQH', '蕉城区','福建省宁德市蕉城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350900', NULL, 'XZQH', '宁德市','福建省宁德市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540237', NULL, 'XZQH', '岗巴县','西藏自治区日喀则市岗巴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540236', NULL, 'XZQH', '萨嘎县','西藏自治区日喀则市萨嘎县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511622', NULL, 'XZQH', '武胜县','四川省广安市武胜县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540233', NULL, 'XZQH', '亚东县','西藏自治区日喀则市亚东县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511621', NULL, 'XZQH', '岳池县','四川省广安市岳池县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540232', NULL, 'XZQH', '仲巴县','西藏自治区日喀则市仲巴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540235', NULL, 'XZQH', '聂拉木县','西藏自治区日喀则市聂拉木县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511623', NULL, 'XZQH', '邻水县','四川省广安市邻水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('540234', NULL, 'XZQH', '吉隆县','西藏自治区日喀则市吉隆县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350000', NULL, 'XZQH', '福建省','福建省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('320981', NULL, 'XZQH', '东台市','江苏省盐城市东台市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330281', NULL, 'XZQH', '余姚市','浙江省宁波市余姚市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140700', NULL, 'XZQH', '晋中市','山西省晋中市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('140702', NULL, 'XZQH', '榆次区','山西省晋中市榆次区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330282', NULL, 'XZQH', '慈溪市','浙江省宁波市慈溪市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330225', NULL, 'XZQH', '象山县','浙江省宁波市象山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330226', NULL, 'XZQH', '宁海县','浙江省宁波市宁海县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330213', NULL, 'XZQH', '奉化区','浙江省宁波市奉化区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330212', NULL, 'XZQH', '鄞州区','浙江省宁波市鄞州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330211', NULL, 'XZQH', '镇海区','浙江省宁波市镇海区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650205', NULL, 'XZQH', '乌尔禾区','新疆维吾尔自治区克拉玛依市乌尔禾区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650203', NULL, 'XZQH', '克拉玛依区','新疆维吾尔自治区克拉玛依市克拉玛依区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650204', NULL, 'XZQH', '白碱滩区','新疆维吾尔自治区克拉玛依市白碱滩区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650202', NULL, 'XZQH', '独山子区','新疆维吾尔自治区克拉玛依市独山子区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('650200', NULL, 'XZQH', '克拉玛依市','新疆维吾尔自治区克拉玛依市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621100', NULL, 'XZQH', '定西市','甘肃省定西市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('621102', NULL, 'XZQH', '安定区','甘肃省定西市安定区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340406', NULL, 'XZQH', '潘集区','安徽省淮南市潘集区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340405', NULL, 'XZQH', '八公山区','安徽省淮南市八公山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340404', NULL, 'XZQH', '谢家集区','安徽省淮南市谢家集区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340403', NULL, 'XZQH', '田家庵区','安徽省淮南市田家庵区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340402', NULL, 'XZQH', '大通区','安徽省淮南市大通区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341004', NULL, 'XZQH', '徽州区','安徽省黄山市徽州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('340400', NULL, 'XZQH', '淮南市','安徽省淮南市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341002', NULL, 'XZQH', '屯溪区','安徽省黄山市屯溪区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341003', NULL, 'XZQH', '黄山区','安徽省黄山市黄山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341000', NULL, 'XZQH', '黄山市','安徽省黄山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('341021', NULL, 'XZQH', '歙县','安徽省黄山市歙县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321204', NULL, 'XZQH', '姜堰区','江苏省泰州市姜堰区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321203', NULL, 'XZQH', '高港区','江苏省泰州市高港区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321202', NULL, 'XZQH', '海陵区','江苏省泰州市海陵区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('321200', NULL, 'XZQH', '泰州市','江苏省泰州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360926', NULL, 'XZQH', '铜鼓县','江西省宜春市铜鼓县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310110', NULL, 'XZQH', '杨浦区','上海市上海市杨浦区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310109', NULL, 'XZQH', '虹口区','上海市上海市虹口区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360921', NULL, 'XZQH', '奉新县','江西省宜春市奉新县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310104', NULL, 'XZQH', '徐汇区','上海市上海市徐汇区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310101', NULL, 'XZQH', '黄浦区','上海市上海市黄浦区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360925', NULL, 'XZQH', '靖安县','江西省宜春市靖安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310107', NULL, 'XZQH', '普陀区','上海市上海市普陀区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411000', NULL, 'XZQH', '许昌市','河南省许昌市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360924', NULL, 'XZQH', '宜丰县','江西省宜春市宜丰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360923', NULL, 'XZQH', '上高县','江西省宜春市上高县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310105', NULL, 'XZQH', '长宁区','上海市上海市长宁区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411002', NULL, 'XZQH', '魏都区','河南省许昌市魏都区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360922', NULL, 'XZQH', '万载县','江西省宜春市万载县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310106', NULL, 'XZQH', '静安区','上海市上海市静安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411003', NULL, 'XZQH', '建安区','河南省许昌市建安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440282', NULL, 'XZQH', '南雄市','广东省韶关市南雄市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440281', NULL, 'XZQH', '乐昌市','广东省韶关市乐昌市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310100', NULL, 'XZQH', '上海市','上海市上海市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610582', NULL, 'XZQH', '华阴市','陕西省渭南市华阴市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610581', NULL, 'XZQH', '韩城市','陕西省渭南市韩城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220403', NULL, 'XZQH', '西安区','吉林省辽源市西安区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220402', NULL, 'XZQH', '龙山区','吉林省辽源市龙山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('350881', NULL, 'XZQH', '漳平市','福建省龙岩市漳平市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220400', NULL, 'XZQH', '辽源市','吉林省辽源市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542429', NULL, 'XZQH', '巴青县','西藏自治区那曲市巴青县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542428', NULL, 'XZQH', '班戈县','西藏自治区那曲市班戈县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542427', NULL, 'XZQH', '索县','西藏自治区那曲市索县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542426', NULL, 'XZQH', '申扎县','西藏自治区那曲市申扎县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542425', NULL, 'XZQH', '安多县','西藏自治区那曲市安多县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500237', NULL, 'XZQH', '巫山县','重庆市null巫山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542424', NULL, 'XZQH', '聂荣县','西藏自治区那曲市聂荣县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500236', NULL, 'XZQH', '奉节县','重庆市null奉节县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542423', NULL, 'XZQH', '比如县','西藏自治区那曲市比如县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542422', NULL, 'XZQH', '嘉黎县','西藏自治区那曲市嘉黎县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500238', NULL, 'XZQH', '巫溪县','重庆市null巫溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542421', NULL, 'XZQH', '色尼区','西藏自治区那曲市色尼区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500243', NULL, 'XZQH', '彭水苗族土家族自治县','重庆市null彭水苗族土家族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542430', NULL, 'XZQH', '尼玛县','西藏自治区那曲市尼玛县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542431', NULL, 'XZQH', '双湖县','西藏自治区那曲市双湖县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500240', NULL, 'XZQH', '石柱土家族自治县','重庆市null石柱土家族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500241', NULL, 'XZQH', '秀山土家族苗族自治县','重庆市null秀山土家族苗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500242', NULL, 'XZQH', '酉阳土家族苗族自治县','重庆市null酉阳土家族苗族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360902', NULL, 'XZQH', '袁州区','江西省宜春市袁州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360900', NULL, 'XZQH', '宜春市','江西省宜春市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310120', NULL, 'XZQH', '奉贤区','上海市上海市奉贤区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330205', NULL, 'XZQH', '江北区','浙江省宁波市江北区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330203', NULL, 'XZQH', '海曙区','浙江省宁波市海曙区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330200', NULL, 'XZQH', '宁波市','浙江省宁波市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310116', NULL, 'XZQH', '金山区','上海市上海市金山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310117', NULL, 'XZQH', '松江区','上海市上海市松江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310118', NULL, 'XZQH', '青浦区','上海市上海市青浦区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310112', NULL, 'XZQH', '闵行区','上海市上海市闵行区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310113', NULL, 'XZQH', '宝山区','上海市上海市宝山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('330206', NULL, 'XZQH', '北仑区','浙江省宁波市北仑区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310114', NULL, 'XZQH', '嘉定区','上海市上海市嘉定区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310115', NULL, 'XZQH', '浦东新区','上海市上海市浦东新区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610523', NULL, 'XZQH', '大荔县','陕西省渭南市大荔县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610522', NULL, 'XZQH', '潼关县','陕西省渭南市潼关县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610525', NULL, 'XZQH', '澄城县','陕西省渭南市澄城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610524', NULL, 'XZQH', '合阳县','陕西省渭南市合阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610527', NULL, 'XZQH', '白水县','陕西省渭南市白水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610526', NULL, 'XZQH', '蒲城县','陕西省渭南市蒲城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610528', NULL, 'XZQH', '富平县','陕西省渭南市富平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('310151', NULL, 'XZQH', '崇明区','上海市上海市崇明区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440224', NULL, 'XZQH', '仁化县','广东省韶关市仁化县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440222', NULL, 'XZQH', '始兴县','广东省韶关市始兴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440229', NULL, 'XZQH', '翁源县','广东省韶关市翁源县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360112', NULL, 'XZQH', '新建区','江西省南昌市新建区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360111', NULL, 'XZQH', '青山湖区','江西省南昌市青山湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511528', NULL, 'XZQH', '兴文县','四川省宜宾市兴文县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511529', NULL, 'XZQH', '屏山县','四川省宜宾市屏山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511526', NULL, 'XZQH', '珙县','四川省宜宾市珙县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511527', NULL, 'XZQH', '筠连县','四川省宜宾市筠连县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511524', NULL, 'XZQH', '长宁县','四川省宜宾市长宁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511525', NULL, 'XZQH', '高县','四川省宜宾市高县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('511523', NULL, 'XZQH', '江安县','四川省宜宾市江安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410883', NULL, 'XZQH', '孟州市','河南省焦作市孟州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620982', NULL, 'XZQH', '敦煌市','甘肃省酒泉市敦煌市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620981', NULL, 'XZQH', '玉门市','甘肃省酒泉市玉门市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440233', NULL, 'XZQH', '新丰县','广东省韶关市新丰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440232', NULL, 'XZQH', '乳源瑶族自治县','广东省韶关市乳源瑶族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360121', NULL, 'XZQH', '南昌县','江西省南昌市南昌县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360123', NULL, 'XZQH', '安义县','江西省南昌市安义县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360124', NULL, 'XZQH', '进贤县','江西省南昌市进贤县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610500', NULL, 'XZQH', '渭南市','陕西省渭南市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610503', NULL, 'XZQH', '华州区','陕西省渭南市华州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('610502', NULL, 'XZQH', '临渭区','陕西省渭南市临渭区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210411', NULL, 'XZQH', '顺城区','辽宁省抚顺市顺城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411025', NULL, 'XZQH', '襄城县','河南省许昌市襄城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411024', NULL, 'XZQH', '鄢陵县','河南省许昌市鄢陵县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440200', NULL, 'XZQH', '韶关市','广东省韶关市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440203', NULL, 'XZQH', '武江区','广东省韶关市武江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440205', NULL, 'XZQH', '曲江区','广东省韶关市曲江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('440204', NULL, 'XZQH', '浈江区','广东省韶关市浈江区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450521', NULL, 'XZQH', '合浦县','广西壮族自治区北海市合浦县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430382', NULL, 'XZQH', '韶山市','湖南省湘潭市韶山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430381', NULL, 'XZQH', '湘乡市','湖南省湘潭市湘乡市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210400', NULL, 'XZQH', '抚顺市','辽宁省抚顺市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360105', NULL, 'XZQH', '湾里区','江西省南昌市湾里区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210404', NULL, 'XZQH', '望花区','辽宁省抚顺市望花区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360104', NULL, 'XZQH', '青云谱区','江西省南昌市青云谱区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210403', NULL, 'XZQH', '东洲区','辽宁省抚顺市东洲区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('210402', NULL, 'XZQH', '新抚区','辽宁省抚顺市新抚区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360100', NULL, 'XZQH', '南昌市','江西省南昌市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360103', NULL, 'XZQH', '西湖区','江西省南昌市西湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220422', NULL, 'XZQH', '东辽县','吉林省辽源市东辽县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360102', NULL, 'XZQH', '东湖区','江西省南昌市东湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('220421', NULL, 'XZQH', '东丰县','吉林省辽源市东丰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411082', NULL, 'XZQH', '长葛市','河南省许昌市长葛市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450500', NULL, 'XZQH', '北海市','广西壮族自治区北海市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('411081', NULL, 'XZQH', '禹州市','河南省许昌市禹州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450503', NULL, 'XZQH', '银海区','广西壮族自治区北海市银海区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620000', NULL, 'XZQH', '甘肃省','甘肃省', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450502', NULL, 'XZQH', '海城区','广西壮族自治区北海市海城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130608', NULL, 'XZQH', '清苑区','河北省保定市清苑区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130609', NULL, 'XZQH', '徐水区','河北省保定市徐水区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130607', NULL, 'XZQH', '满城区','河北省保定市满城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620924', NULL, 'XZQH', '阿克塞哈萨克族自治县','甘肃省酒泉市阿克塞哈萨克族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130606', NULL, 'XZQH', '莲池区','河北省保定市莲池区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620923', NULL, 'XZQH', '肃北蒙古族自治县','甘肃省酒泉市肃北蒙古族自治县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620922', NULL, 'XZQH', '瓜州县','甘肃省酒泉市瓜州县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620921', NULL, 'XZQH', '金塔县','甘肃省酒泉市金塔县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130602', NULL, 'XZQH', '竞秀区','河北省保定市竞秀区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130600', NULL, 'XZQH', '保定市','河北省保定市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420800', NULL, 'XZQH', '荆门市','湖北省荆门市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('450512', NULL, 'XZQH', '铁山港区','广西壮族自治区北海市铁山港区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441324', NULL, 'XZQH', '龙门县','广东省惠州市龙门县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441323', NULL, 'XZQH', '惠东县','广东省惠州市惠东县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520382', NULL, 'XZQH', '仁怀市','贵州省遵义市仁怀市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420804', NULL, 'XZQH', '掇刀区','湖北省荆门市掇刀区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441322', NULL, 'XZQH', '博罗县','广东省惠州市博罗县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420802', NULL, 'XZQH', '东宝区','湖北省荆门市东宝区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('520381', NULL, 'XZQH', '赤水市','贵州省遵义市赤水市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420100', NULL, 'XZQH', '武汉市','湖北省武汉市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360981', NULL, 'XZQH', '丰城市','江西省宜春市丰城市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420102', NULL, 'XZQH', '江岸区','湖北省武汉市江岸区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360983', NULL, 'XZQH', '高安市','江西省宜春市高安市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420103', NULL, 'XZQH', '江汉区','湖北省武汉市江汉区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('360982', NULL, 'XZQH', '樟树市','江西省宜春市樟树市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420104', NULL, 'XZQH', '硚口区','湖北省武汉市硚口区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110108', NULL, 'XZQH', '海淀区','北京市北京市海淀区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420105', NULL, 'XZQH', '汉阳区','湖北省武汉市汉阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110107', NULL, 'XZQH', '石景山区','北京市北京市石景山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420106', NULL, 'XZQH', '武昌区','湖北省武汉市武昌区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420107', NULL, 'XZQH', '青山区','湖北省武汉市青山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110109', NULL, 'XZQH', '门头沟区','北京市北京市门头沟区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110106', NULL, 'XZQH', '丰台区','北京市北京市丰台区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410882', NULL, 'XZQH', '沁阳市','河南省焦作市沁阳市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110105', NULL, 'XZQH', '朝阳区','北京市北京市朝阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110100', NULL, 'XZQH', '北京市','北京市北京市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110102', NULL, 'XZQH', '西城区','北京市北京市西城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('110101', NULL, 'XZQH', '东城区','北京市北京市东城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430321', NULL, 'XZQH', '湘潭县','湖南省湘潭市湘潭县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130624', NULL, 'XZQH', '阜平县','河北省保定市阜平县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130623', NULL, 'XZQH', '涞水县','河北省保定市涞水县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130629', NULL, 'XZQH', '容城县','河北省保定市容城县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620902', NULL, 'XZQH', '肃州区','甘肃省酒泉市肃州区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130628', NULL, 'XZQH', '高阳县','河北省保定市高阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130627', NULL, 'XZQH', '唐县','河北省保定市唐县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('620900', NULL, 'XZQH', '酒泉市','甘肃省酒泉市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('130626', NULL, 'XZQH', '定兴县','河北省保定市定兴县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420111', NULL, 'XZQH', '洪山区','湖北省武汉市洪山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420114', NULL, 'XZQH', '蔡甸区','湖北省武汉市蔡甸区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420115', NULL, 'XZQH', '江夏区','湖北省武汉市江夏区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420112', NULL, 'XZQH', '东西湖区','湖北省武汉市东西湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420113', NULL, 'XZQH', '汉南区','湖北省武汉市汉南区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420822', NULL, 'XZQH', '沙洋县','湖北省荆门市沙洋县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420116', NULL, 'XZQH', '黄陂区','湖北省武汉市黄陂区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420821', NULL, 'XZQH', '京山县','湖北省荆门市京山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('420117', NULL, 'XZQH', '新洲区','湖北省武汉市新洲区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441300', NULL, 'XZQH', '惠州市','广东省惠州市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441303', NULL, 'XZQH', '惠阳区','广东省惠州市惠阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('441302', NULL, 'XZQH', '惠城区','广东省惠州市惠城区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150900', NULL, 'XZQH', '乌兰察布市','内蒙古自治区乌兰察布市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('150902', NULL, 'XZQH', '集宁区','内蒙古自治区乌兰察布市集宁区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430300', NULL, 'XZQH', '湘潭市','湖南省湘潭市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510421', NULL, 'XZQH', '米易县','四川省攀枝花市米易县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532331', NULL, 'XZQH', '禄丰县','云南省楚雄彝族自治州禄丰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430304', NULL, 'XZQH', '岳塘区','湖南省湘潭市岳塘区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('430302', NULL, 'XZQH', '雨湖区','湖南省湘潭市雨湖区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('542400', NULL, 'XZQH', '那曲市','西藏自治区那曲市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532329', NULL, 'XZQH', '武定县','云南省楚雄彝族自治州武定县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532328', NULL, 'XZQH', '元谋县','云南省楚雄彝族自治州元谋县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532327', NULL, 'XZQH', '永仁县','云南省楚雄彝族自治州永仁县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410811', NULL, 'XZQH', '山阳区','河南省焦作市山阳区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532326', NULL, 'XZQH', '大姚县','云南省楚雄彝族自治州大姚县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532325', NULL, 'XZQH', '姚安县','云南省楚雄彝族自治州姚安县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532324', NULL, 'XZQH', '南华县','云南省楚雄彝族自治州南华县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510411', NULL, 'XZQH', '仁和区','四川省攀枝花市仁和区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532323', NULL, 'XZQH', '牟定县','云南省楚雄彝族自治州牟定县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532322', NULL, 'XZQH', '双柏县','云南省楚雄彝族自治州双柏县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230524', NULL, 'XZQH', '饶河县','黑龙江省双鸭山市饶河县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230521', NULL, 'XZQH', '集贤县','黑龙江省双鸭山市集贤县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230523', NULL, 'XZQH', '宝清县','黑龙江省双鸭山市宝清县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500231', NULL, 'XZQH', '垫江县','重庆市null垫江县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230522', NULL, 'XZQH', '友谊县','黑龙江省双鸭山市友谊县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500230', NULL, 'XZQH', '丰都县','重庆市null丰都县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500233', NULL, 'XZQH', '忠县','重庆市null忠县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500235', NULL, 'XZQH', '云阳县','重庆市null云阳县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410804', NULL, 'XZQH', '马村区','河南省焦作市马村区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410803', NULL, 'XZQH', '中站区','河南省焦作市中站区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('500229', NULL, 'XZQH', '城口县','重庆市null城口县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410802', NULL, 'XZQH', '解放区','河南省焦作市解放区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510422', NULL, 'XZQH', '盐边县','四川省攀枝花市盐边县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410800', NULL, 'XZQH', '焦作市','河南省焦作市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('433127', NULL, 'XZQH', '永顺县','湖南省湘西土家族苗族自治州永顺县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('433126', NULL, 'XZQH', '古丈县','湖南省湘西土家族苗族自治州古丈县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('433125', NULL, 'XZQH', '保靖县','湖南省湘西土家族苗族自治州保靖县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('433130', NULL, 'XZQH', '龙山县','湖南省湘西土家族苗族自治州龙山县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532301', NULL, 'XZQH', '楚雄市','云南省楚雄彝族自治州楚雄市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230503', NULL, 'XZQH', '岭东区','黑龙江省双鸭山市岭东区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230502', NULL, 'XZQH', '尖山区','黑龙江省双鸭山市尖山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230505', NULL, 'XZQH', '四方台区','黑龙江省双鸭山市四方台区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230506', NULL, 'XZQH', '宝山区','黑龙江省双鸭山市宝山区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('532300', NULL, 'XZQH', '楚雄彝族自治州','云南省楚雄彝族自治州', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('230500', NULL, 'XZQH', '双鸭山市','黑龙江省双鸭山市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('433122', NULL, 'XZQH', '泸溪县','湖南省湘西土家族苗族自治州泸溪县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('433123', NULL, 'XZQH', '凤凰县','湖南省湘西土家族苗族自治州凤凰县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('433124', NULL, 'XZQH', '花垣县','湖南省湘西土家族苗族自治州花垣县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410825', NULL, 'XZQH', '温县','河南省焦作市温县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410823', NULL, 'XZQH', '武陟县','河南省焦作市武陟县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510400', NULL, 'XZQH', '攀枝花市','四川省攀枝花市', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410822', NULL, 'XZQH', '博爱县','河南省焦作市博爱县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510403', NULL, 'XZQH', '西区','四川省攀枝花市西区', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('410821', NULL, 'XZQH', '修武县','河南省焦作市修武县', NULL, '0', '1');
INSERT INTO SYS_DICT (CODE, BCODE,DICT_CODE,CAPTION,FULL_CAPTION,SHORT_CUT,READONLY,ACTIVE) VALUES ('510402', NULL, 'XZQH', '东区','四川省攀枝花市东区', NULL, '0', '1');


ALTER TABLE public.bridge ALTER COLUMN service_day DROP NOT NULL;

ALTER TABLE public.bridge ALTER COLUMN email DROP NOT NULL;

CREATE TABLE "public"."env_data" (
        "item_id" int4 NOT NULL,
        "item_value" float8 NOT NULL,
        "item_time" text COLLATE "default" NOT NULL
);

CREATE TABLE "public"."env_id_dict" (
        "item_id" int4 NOT NULL,
        "caption" text COLLATE "default" NOT NULL,
        "unit" text COLLATE "default" NOT NULL,
        CONSTRAINT "env_id_dict_pk" UNIQUE ("item_id")
);



----除湿系统

CREATE TABLE ENV_SERVER (
        ID CHAR(32) NOT NULL,
        CODE TEXT NOT NULL,
        SEQ TEXT NOT NULL,
        BRIDGE_ID TEXT NOT NULL,
        IP_ADDRESS TEXT NOT NULL,
        REMARK TEXT,
        ACTIVE CHAR(1) NOT NULL,
        CREATE_TIME CHAR(14) NOT NULL,
        MODIFY_TIME CHAR(14) NOT NULL,
        CREATOR CHAR(32) NOT NULL,
        MODIFIER CHAR(32) NOT NULL);
ALTER TABLE ENV_SERVER ADD PRIMARY KEY (ID);
COMMENT ON TABLE ENV_SERVER IS '除湿系统采集服务器';
COMMENT ON COLUMN ENV_SERVER.ID IS 'id';
COMMENT ON COLUMN ENV_SERVER.CODE IS '采集服务器编号';
COMMENT ON COLUMN ENV_SERVER.SEQ IS '内部编号';
COMMENT ON COLUMN ENV_SERVER.BRIDGE_ID IS '采集服务器所属桥梁Id';
COMMENT ON COLUMN ENV_SERVER.IP_ADDRESS IS '采集服务器ip';
COMMENT ON COLUMN ENV_SERVER.REMARK IS '备注';
COMMENT ON COLUMN ENV_SERVER.ACTIVE IS '是否可用';
COMMENT ON COLUMN ENV_SERVER.CREATE_TIME IS '创建时间（YYYYMMDDHH24MISS）不可修改';
COMMENT ON COLUMN ENV_SERVER.MODIFY_TIME IS '最后修改时间（YYYYMMDDHH24MISS）,初始值:20000101010101';
COMMENT ON COLUMN ENV_SERVER.CREATOR IS '创建人ID';
COMMENT ON COLUMN ENV_SERVER.MODIFIER IS '最后修改人ID';

CREATE TABLE ENV_TRANSDUCER (
        ID CHAR(32) NOT NULL,
        SERVER_ID TEXT NOT NULL,
        SYSTEM TEXT NOT NULL,
        ORGANIZE INTEGER NOT NULL,
        CODE INTEGER NOT NULL,
        SEQ INTEGER NOT NULL,
        NAME TEXT NOT NULL,
        POSITION TEXT NOT NULL,
        CABLE_TYPE TEXT NOT NULL,
        UNIT TEXT NOT NULL,
        REMARK TEXT,
        ACTIVE CHAR(1) NOT NULL,
        CREATE_TIME CHAR(14) NOT NULL,
        MODIFY_TIME CHAR(14) NOT NULL,
        CREATOR CHAR(32) NOT NULL,
        MODIFIER CHAR(32) NOT NULL);
ALTER TABLE ENV_TRANSDUCER ADD PRIMARY KEY (ID);
COMMENT ON TABLE ENV_TRANSDUCER IS '传感器';
COMMENT ON COLUMN ENV_TRANSDUCER.ID IS 'id';
COMMENT ON COLUMN ENV_TRANSDUCER.SERVER_ID IS '所属服务器ID';
COMMENT ON COLUMN ENV_TRANSDUCER.SYSTEM IS '所属系统';
COMMENT ON COLUMN ENV_TRANSDUCER.ORGANIZE IS '所属组';
COMMENT ON COLUMN ENV_TRANSDUCER.CODE IS '传感器编号';
COMMENT ON COLUMN ENV_TRANSDUCER.SEQ IS '内部编号';
COMMENT ON COLUMN ENV_TRANSDUCER.NAME IS '名称';
COMMENT ON COLUMN ENV_TRANSDUCER.POSITION IS '所属方位';
COMMENT ON COLUMN ENV_TRANSDUCER.CABLE_TYPE IS '所属主缆';
COMMENT ON COLUMN ENV_TRANSDUCER.UNIT IS '数据单位';
COMMENT ON COLUMN ENV_TRANSDUCER.REMARK IS '备注';
COMMENT ON COLUMN ENV_TRANSDUCER.ACTIVE IS '是否可用';
COMMENT ON COLUMN ENV_TRANSDUCER.CREATE_TIME IS '创建时间（YYYYMMDDHH24MISS）不可修改';
COMMENT ON COLUMN ENV_TRANSDUCER.MODIFY_TIME IS '最后修改时间（YYYYMMDDHH24MISS）,初始值:20000101010101';
COMMENT ON COLUMN ENV_TRANSDUCER.CREATOR IS '创建人ID';
COMMENT ON COLUMN ENV_TRANSDUCER.MODIFIER IS '最后修改人ID';